<?php
/**
* © Copyright 2011 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris-retain
* @subpackage retain
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v1.1
* @since v1.1
* @filesource 
*/ 
/** 
* Class iHRIS_Finanical
* 
* @access public
*/


class iHRIS_Financial extends iHRIS_Yearly{
    /**
     * Get the field names that are yearly inputs
     * @returns array of string
     */ 
    public function yearlyInputs() {
        return  array('total_budget','total_recurrent_budget','total_personnel_budget','donor_contribution','exchange_USD','inflation');
    }


    public function validate() {
        parent::validate();
        $years = $this->fields['total_budget']->getKeys();
        foreach ($years as $year) {
            if (
                $this->fields['total_budget']->keyIsSet($year) &&
                $this->fields['total_recurrent_budget']->keyIsSet($year) &&
                $this->fields['total_budget']->getValueOfKey($year) < $this->fields['total_recurrent_budget']->getValueOfKey($year)
                ) {
                $this->fields['total_recurrent_budget']->setInvalidMessage('recurrent_exceeds_total');
                break;
            }
        }

        $years = $this->fields['total_personnel_budget']->getKeys();
        foreach ($years as $year) {
            if (
                $this->fields['total_personnel_budget']->keyIsSet($year) &&
                $this->fields['donor_contribution']->keyIsSet($year) &&
                $this->fields['total_personnel_budget']->getValueOfKey($year) < $this->fields['donor_contribution']->getValueOfKey($year)
                ) {
                $this->fields['donor_contribution']->setInvalidMessage("donor_exceeds_personnel");
                break;
            }
        }

    }


    public function getCurrencyCode() {
        $default = '';
        $currency = $this->getField('currency');
        if (!$currency instanceof I2CE_FormField_MAP) {
            I2CE::raiseError("Invalid currency");
            return $default;
        }
        if ($currency->getDBValue() == 'currency|USD') {
            return '$';
        }
        $currencyObj = $currency->getMappedFormObject();
        if (!$currencyObj instanceof iHRIS_Currency) {
            I2CE::raiseError("Invalid currency object");
            return $default;
        }
        $code = $currencyObj->symbol;
        if (!$code) {
            $code = $currencyObj->code;
        }
        return $code;        
    }


    protected function getProjection() {
        $ff = I2CE_FormFactory::instance();
        $projObj = $ff->createContainer($this->getParent());
        if (!$projObj instanceof iHRIS_Projection) {
            return false;
        }
        $projObj->populate();
        return $projObj;
    }

    protected function getMultiplier() {
        $multiplier = 1;
        if (! ($projObj = $this->getProjection())) {
            return $multiplier;
        }
        if ( ! ( $multField  =  $this->getField('currency_multiplier')) instanceof I2CE_FormField_MAP) {
            return $multiplier;
        }
        if ( ! ( $multObj = $multField->getMappedFormObject()) instanceof iHRIS_CurrencyMultiplier) {
            return $multiplier;
        }
        if (! ($amntField = $multObj->getField('amount')) instanceof I2CE_FormField_INT || !$amntField->isValid()) {
            return $multiplier;
        }
        return max(1,$amntField->getValue());
    }



    public function getInflationRate($year,$default =0.0) {
        return $this->getRate($year,'inflation',$default);
    }

    public function getExchangeRate($year, $default = 1.0) {
        if ($this->getField('currency')->getDBValue() == 'currency|USD') {
            return 1.0;
        }
        return $this->getRate($year,'exchange_USD',$default);
    }


    protected function getRate($year,$field, $default) {
        $default = (float)$default;
        if ( ! ($rate = $this->getField($field)) instanceof I2CE_FormField_ASSOC_LIST) {
            I2CE::raiseError("Not list");
            return $default;
        }
        if ( $rate->keyIsSet($year)) {   
            return  (float) $rate->getValueOfKey($year);
        } else {
            return (float) $this->getLSQValue($year, $field,$default);
        }
    }




    public function fixup() {
        $inflation = $this->getField('inflation');
        if (!$inflation instanceof I2CE_FormField_ASSOC_LIST) {
            return false;
        }
        if (! ($projObj = $this->getProjection())) {
            return $res;
        }
        $start_year = $projObj->start_year->year();        
        $duration = $projObj->duration;
        $bad_years  = array();
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            if ( !$inflation->keyIsSet($year)) {
                $bad_years[] = $year;
            }
        }
        foreach ($bad_years as $year) {
            $inflation->setValueOfKey($year,$this->getInflationRate($year));

        }
        $fields = array('total_budget'=>true,'total_recurrent_budget'=>true,'exchange_USD'=>false);
        foreach ($fields as $field=>$as_int) {
            $this->getInflatedBudget($field,$as_int,true,1);
        }
        $fields = array('total_personnel_budget'=>true);
        foreach ($fields as $field=>$as_int) {
            $this->getProportionalBudget($field,$as_int,true,1);
        }
    }



    public function getTotalHealthBudgetsByYear() {
        return $this->getInflatedBudget('total_budget');
    }

    public function getTotalSalaryBudgetsByYear() {
        return $this->getProportionalBudget('total_personnel_budget');
    }

    public function getTotalDonorContributionsByYear() {
        return $this->getInflatedBudget('donor_contribution');
    }

    public function getTotalRecurrentBudgetByYear() {
        return $this->getInflatedBudget('total_recurrent_budget');
    }



    public function getInflatedBudget($field, $as_int = true, $set_in_form = false, $multiplier = null) {
        $res = array();
        if (! ($projObj = $this->getProjection())) {
            return $res;
        }
        $start_year = $projObj->start_year->year();        
        $duration = $projObj->duration;
        if (!is_int($multiplier)) {
            $multiplier = $this->getMultiplier();
        }

        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $res[$year] = 0;
        }        
        $fieldObj = $this->getField($field);
        if (!$fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
            I2CE::raiseError("No field $field");
            return $res;
        }
        //first we get anything that was filled in.
        $has_something = false;
        $t_res = $res;
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            if ( !$fieldObj->keyIsSet($year)) {
                $t_res[$year] = false;
                continue;
            }

            if($has_something === false) {
                $has_something = $year;
            }
            $t_res[$year] = max(0, $fieldObj->getValueOfKey($year));
        }
        if ($has_something === false) {            
            return $res;
        }
        $res= $t_res;
        //next we fill in any missings years based on inflation data
        if ($has_something > $start_year) {
            //use  the first year that is filled and deflate backwards
            for ($year = $has_something - 1; $year >= $start_year; $year--) {
                $rate = $this->getInflationRate($year);
                $res[$year]  = $res[$year + 1] / ( 1.0 + ($rate/100.0));
            }
        }
        //we know now that the first year is filled
        for ( $year = $start_year +1; $year < $start_year + $duration; $year++) {
            if ($res[$year] !== false) {
                continue;
            }
            $rate = $this->getInflationRate($year-1); //get the previous year's inflation rate
            $res[$year]  = $res[$year -1 ] * ( 1.0 + ($rate/100));            
            if ($set_in_form) {
                if ($as_int) {
                    $fieldObj->setValueOfKey($year,(int) $res[$year]);
                } else {
                    $fieldObj->setValueOfKey($year,(float) $res[$year]);
                }
            }
        }
        foreach ($res as &$amount) {
            $amount *= $multiplier;
        }
        unset($amount);
        return $res;
    }




    public function getProportionalBudget($field, $as_int = true, $set_in_form = false, $multiplier = null) {
        $res = array();
        if (! ($projObj = $this->getProjection())) {
            return $res;
        }
        $start_year = $projObj->start_year->year();        
        $duration = $projObj->duration;
        if (!is_int($multiplier)) {
            $multiplier = $this->getMultiplier();
        }

        $res = array();
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $res[$year] = 0;
        }        
        
        $totalFieldObj = $this->getField('total_budget');
        if (!$totalFieldObj instanceof I2CE_FormField_ASSOC_LIST) {
            I2CE::raiseError("No total budget field");
            return $res;
        }

        $fieldObj = $this->getField($field);
        if (!$fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
            I2CE::raiseError("No field $field");
            return $res;
        }
        //first we get anything that was filled in.
        $has_something = false;
        $t_res = $res;
        $totalRes = array();
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            if ($totalFieldObj->keyIsSet($year)) {
                $totalRes[$year] = max(0, $totalFieldObj->getValueOfKey($year));
            }
            if ( !$fieldObj->keyIsSet($year)) {
                $t_res[$year] = false;
                continue;
            }

            if($has_something === false) {
                $has_something = $year;
            }
            $t_res[$year] = max(0, $fieldObj->getValueOfKey($year));
        }
        if ($has_something === false) {            
            return $res;
        }
        $res= $t_res;
        //next we fill in any missings years based on inflation data
        if ($has_something > $start_year) {
            //use  the first year that is filled and deflate backwards
            for ($year = $has_something - 1; $year >= $start_year; $year--) {
                $rate = $this->getInflationRate($year);
                $res[$year]  = $res[$year + 1] / ( 1.0 + ($rate/100.0));
            }
        }
        //we know now that the first year is filled
        if (array_key_exists($start_year,$totalRes) && $totalRes[$start_year] != 0) {
            $prop = ((float) $res[$start_year]) / ((float) $totalRes[$start_year]);
        } else {
            //invalid data.... just fall back to inflation;
            $prop = false;
        }
        for ( $year = $start_year +1; $year < $start_year + $duration; $year++) {
            if ($res[$year] !== false) {
                continue;
            }
            if ($prop !== false  && array_key_exists($year, $totalRes)) {
                $res[$year]  = $prop * $totalRes[$year];
                if ($set_in_form) {
                    if ($as_int) {
                        $fieldObj->setValueOfKey($year,(int) $res[$year]);
                    } else {
                        $fieldObj->setValueOfKey($year,(float) $res[$year]);
                    }
                }
            } else{
                $rate = $this->getInflationRate($year-1); //get the previous year's inflation rate
                $res[$year]  = $res[$year -1 ] * ( 1.0 + ($rate/100));            
                if ($set_in_form) {
                    if ($as_int) {
                        $fieldObj->setValueOfKey($year,(int) $res[$year]);
                    } else {
                        $fieldObj->setValueOfKey($year,(float) $res[$year]);
                    }
                }
            }
        }
        foreach ($res as &$amount) {
            $amount *= $multiplier;
        }
        unset($amount);
        return $res;
    }


    protected function getRateData($field) {
        $rates= $this->getField($field);
        if (!$rates instanceof I2CE_FormField_ASSOC_LIST) {
            I2CE::raiseError("Invalid $field");
            return false;
        }
        return $rates;
    }



    protected $lsq_m = array();
    protected $lsq_b = array();
    /**
     *Get the Linear Least Squares fit of the inflation  for the given year
     * http://mathworld.wolfram.com/LeastSquaresFitting.html
     * @paarm int $year
     */
    protected function getLSQValue($year,$field,$default) {
        if (!array_key_exists($field,$this->lsq_m) ||!array_key_existS($field,$this->lsq_b)) {
            if ( ! ($inflation = $this->getRateData($field))) {
                return $default;
            }
            $data = $inflation->getValue();
            foreach ($data as $dx=>$dy) {
                if (!is_numeric($dx) || !is_numeric($dy)) {
                    unset($data[$dx]);
                }                
            }
            $n = count($data);
            if ($n == 0) {
                return $default;
            }
            if ($n == 1) { //we simply assume constant 
                reset($data);
                return (float) current($data);
            }
            //$n >= 2
            $sum_xy = 0.0;
            $sum_xx = 0.0;
            $sum_y = 0.0;
            $sum_x = 0.0;
            $n = (float) $n;
            foreach ($data as $dx=>$dy) {
                $sum_xy += ((float) $dx)*((float)$dy);
                $sum_xx += ((float) $dx)*((float)$dx);
                $sum_x += ((float) $dx);
                $sum_y += ((float) $dy);
                
            }

            $d = (float) ($n*$sum_xx - $sum_x*$sum_x );
            $this->lsq_m[$field] = ( $n * $sum_xy -  $sum_x*$sum_y ) / $d; 
            $this->lsq_b[$field] = ($sum_y * $sum_xx - $sum_x*$sum_xy ) / $d;
        }
        return (((float) $year)* ($this->lsq_m[$field])) + ($this->lsq_b[$field]);
    }




}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
