<?php
/**
* © Copyright 2010 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris
* @subpackage plan
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v3.1.5
* @since v3.1.5
* @filesource 
*/ 
/** 
* Class iHRIS_Intervention
* 
* @access public
*/


abstract class iHRIS_Intervention extends iHRIS_YearlyList {



    protected $formulas = array();

    public function getTotalFormula() {
        return $this->formulas['total'];
    }
    public function getTargetFormula() {
        return $this->formulas['target'];
    }


    protected function loadFormulas() {
        if ($this->hasMeta('total_formula')) {
            $this->formulas['total'] = $this->getMeta('total_formula');
        } else {

        }
        if ($this->hasMeta('target_formula')) {
            $this->formulas['target'] = $this->getMeta('target_formula');
        } else {
            $this->formulas['target'] = false;
        }
        foreach ($this->formulas as $type=>$formula) {      
            $data_fields = array();
            $this->data_fields[$type] = array();
            if (!$formula) {
                continue;
            }
            preg_match_all( '/<<(.*?)>>/',$formula,$data_fields);
            if (!array_key_exists(1,$data_fields) || count($data_fields[1]) == 0) {
                I2CE::raiseError("No data fields in $type formula for " . get_class($this));
            } 
            foreach ($data_fields[1] as $data_field) {
                if (in_array($data_field,$this->data_fields[$type])) {
                    continue;
                }
                if (!array_key_exists($data_field,$this->fields) || ! $this->fields[$data_field] instanceof I2CE_FormField) {
                    I2CE::raiseError("$type formula in " . get_class($this) . ": " . $this->formulas[$type] . " references $data_field which is is not a formfield", E_USER_WARNING);
                    continue;
                }                   
                $this->data_fields[$type][] = $data_field;
            }
        }
        $ref_fields =array();
        foreach ($this->formulas as $type=>$formula) {
            $ref_fields = array_merge($ref_fields,$this->data_fields[$type]);
        }
        $unused_fields = array_diff(array_diff(array_keys($this->fields),$ref_fields),array('description','enabled','pool','target_all','other_yearly','other','projected_costs','i2ce_hidden','category'));
        foreach ($unused_fields as $i=>$field) {
            if ($this->fields[$field] instanceof I2CE_FormField_STRING_LINE || $this->fields[$field] instanceof I2CE_FormField_STRING_MLINE) {
                unset($unused_fields[$i]);
            }
        }
        if (count($unused_fields) > 0) {
            I2CE::raiseError("Warning: " . implode(",", $unused_fields)  . " have been created but not used in the any of the fomulas of " .get_class($this));
        }

    }

    public function startYearOnly() {
        $res = false;
        if ($this->hasMeta('start_year_only')) {
            $res = $this->getMeta('start_year_only');
        }
        return $res;
    }


    /**
     * Validate all fields that are marked as required or unique.
     *
     * This will check all the fields in this form and if they're required or unique it
     * will perform the required checks 
     */
    public function validate() {
        parent::validate();
        $fields = array();
        if ($this->hasMeta('validate_numbers')) {
            $fields = $this->getMeta('validate_numbers');
        }
        foreach ($fields as $field) {
            if (! ($fieldObj = $this->getField($field)) instanceof I2CE_FormField) {
                continue;
            }
            $val = $fieldObj->getValue();
            if ( $val != "" && !I2CE_Validate::checkNumber( $val, -1 ) ) {
                $fieldObj->setInvalidMessage('not_a_number');
            }            
        }
    }

    protected $data_fields =  array();
    /**
     * Create a new instance of a I2CE_Form object.
     * @param I2CE_FieldContainer_Factory $factory
     * @param string $name The name of this form
     * @param integer $id
     */
    public function __construct( $factory, $name, $id='0' ) {
        parent::__construct($factory,$name,$id);
        $this->loadFormulas();
    }

    public function getDuration() {
        if (    $this->startYearOnly()) {
            $duration = 1; 
        } else {
            $duration = $this->projection->duration;
        }
        return $duration;
    }


    protected function addOtherYearlySupport($sheet,$row_offset ,$col_offset = 0) {
        $this->populateChild('othercost');
        $other_support = 0;
        $cost = 0;
        $start_year = $this->projection->start_year->year();        
        $duration = $this->getDuration();
        $row_offset += 3 + count($this->data_fields['total']); 
        foreach ($this->getChildren('othercost') as $otherCost) {
            if (!$otherCost instanceof iHRIS_OtherCost) {
                continue;
            }
            //first check to see if we add
            $add = false;
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                if ($otherCost->getCostByYear($year) != 0) {
                    $add = true;
                }
            }
            if (!$add) {
                continue;
            }
            $label_col =  PHPExcel_Cell::stringFromColumnIndex( $col_offset);
            $row = $row_offset + $other_support;
            $data =array();
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $data[$year] = array(
                    'value'=>$otherCost->getCostByYear($year),
                    'style'=>array(
                        'numberformat'=>array('code'=>'#,##0')
                        )
                    );
            }
            $label = "Other Activity Cost";
            if ($otherCost->description) {
                $label .= "\n\t" . $otherCost->description;
            }
            $this->projection->addYearlyDataArrayToExcel($sheet,$row,$label,$data );
            $other_support++;
        }
        return $other_support;
    }



    

    public function createDataEntryWorkSheet($sheet,$row_offset = 0, $col_offset = 0) {
        $formulas = array();
        if (!$this->ensureProjection()) {
            return $formulas;
        }
        $this->projection->addExcelHeader($sheet,$this->getDisplayName() . ': ' . $this->getField('description')->getValue(),$row_offset );
        
        $this->addSelectedCadres($sheet,$row_offset + 1,$col_offset);   //goes on first row
        $duration =    $this->getDuration();
        if ($duration == 1) {
            $this->projection->addStartYearHeaderToExcel($sheet,$row_offset + 2, $col_offset);
        } else {
            $this->projection->addYearlyHeaderToExcel($sheet,$row_offset + 2, $col_offset);
        }
        $sorted_data_fields = array();
        $template = "i" . substr(get_class($this),7).  '.html';
        $template_file = I2CE::getFileSearch()->search( 'TEMPLATES', $template );
        if (!$template_file ) {
            I2CE::raiseError("Could not find template " . $template);
        } else {
            $doc = new DOMDocument();
            $doc->loadHTMLFile($template_file);
            $xpath = new DOMXPath( $doc );
            $fields = $xpath->query("//span[@type='form']");
            if ($fields instanceof DOMNodeList) {
                foreach ($fields as $field) {
                    $fieldname = $field->getAttribute('name');
                    if (($pos = strpos($fieldname,':')) !== false) {
                        $form  = substr($fieldname,$pos);
                        $fieldname = substr($fieldname,$pos + 1);
                    }
                    if (!in_array($fieldname,$sorted_data_fields)) {
                        $sorted_data_fields[] = $fieldname;
                    }
                }
            }
        }
        if (array_key_exists('total',$this->data_fields) && is_array($this->data_fields['total'])) {
            $sorted_data_fields = array_intersect($sorted_data_fields, $this->data_fields['total']);
        } else {
            $sorted_data_fields = array();
        }
        $t_sorted_data_fields = $sorted_data_fields;
        $bad_fields = array();
        if (array_key_exists('total',$this->data_fields) && is_array($this->data_fields['total'])) {
            foreach ($this->data_fields['total'] as $data_field) {
                if (!in_array($data_field, $sorted_data_fields)) {
                    $sorted_data_fields[] = $data_field;
                    $bad_fields[] = $data_field;
                }
            }
        }
        if (count($bad_fields) > 0) {
            I2CE::raiseError("Data fields" . implode(",", $bad_fields) . " not in $template.\nFound: " . implode(",", $t_sorted_data_fields), E_USER_WARNING);
        }
        $this->data_fields['total'] = array_values($sorted_data_fields);
        $row  = 0;
        foreach ($this->data_fields['total'] as $field) {
            $row++;
            if (!array_key_exists($field,$this->fields) || ! ($fieldObj = $this->fields[$field]) instanceof I2CE_FormField) {
                continue;
            }          
            if ($duration == 1) {
                $this->projection->addStartYearDataToExcel($sheet, 2+$row + $row_offset,$fieldObj, $col_offset );
            } else {
                $this->projection->addYearlyDataToExcel($sheet, 2+$row + $row_offset,$fieldObj, $col_offset );
            } 
        }
        $other_support = $this->addOtherYearlySupport($sheet, $row_offset, $col_offset);
        $formulas['yearly'] = $this->addYearlyTotalToExcel($sheet , $other_support,$row_offset, $col_offset);
        $formulas['row_offset'] = $row_offset + count($this->data_fields['total']) + 3 + $other_support + 1;

        $formulas['col_offset'] = $col_offset + $duration + 1; //there is an extra col for possible description of other
        $range = PHPExcel_Cell::stringFromColumnIndex($col_offset) . ($row_offset+2) . ':' . PHPExcel_Cell::stringFromColumnIndex($formulas['col_offset'] -1) . ($formulas['row_offset'] -1);
        $sheet->getStyle($range)->applyFromArray(iHRIS_Projection::$excelBlockStyle);
        $sheet->getColumnDimension(PHPExcel_Cell::stringFromColumnIndex($col_offset))->setAutoSize(true);
        return $formulas;
    }


    protected $data_row_offset = false;
    protected function excelreplacer($matches) {
        $field = $matches[1];
        if ( ($index = array_search($field, $this->data_fields['total'])) === false) {
            return $matches[0];
        }
        if ( !($fieldObj = $this->getField($field)) instanceof I2CE_FormField) {
            return $matches[0];
        }
        $index += $this->data_row_offset; 
        if ($fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
            return "<<YEAR>>" . $index;
        } else {
            if ($fieldObj instanceof I2CE_FormField_PERCENT_INT) {
                return "<<STARTYEAR>>" . $index;
            } else if ($fieldObj instanceof I2CE_FormField_INT) {
                if ($fieldObj->getOption('no_inflate')) {
                    return "<<STARTYEAR>>" . $index;
                } else {
                    return "<<YEAR>>" . $index;
                }
            } else if ($fieldObj instanceof I2CE_FormField_PERCENT) {
                return "<<STARTYEAR>>" . $index;
            } else if ($fieldObj instanceof I2CE_FormField_TOGGLE) {
                return "<<STARTYEAR>>" . $index;
            } else if ($fieldObj instanceof I2CE_FormField_FLOAT) {
                if ($fieldObj->getOption('no_inflate')) {
                    return "<<STARTYEAR>>" . $index;
                } else {
                    return "<<YEAR>>" . $index;
                }
            }
        }                            
    }



    protected function phpreplacer($matches) {
        //example $this->data_fields['total'] =  array('students','tuition','housing','stipend','othersup');

        if ( ($index = array_search($matches[1], $this->data_fields['total'])) === false) {
            return $matches[0];
        }
        $index += $this->data_row_offset; 
        return '$values[\'' . $matches[1]  . '\']';
    }

    
    protected function _getCostByYear($start_year,$year_offset){
        return $this->evaluateYearlyFormula('total',$start_year + $year_offset);
    }

    protected function getDataFieldValues($type,$year) {
        $values = array();
        foreach ($this->data_fields[$type] as $data_field) {
            
            if ( !($fieldObj = $this->getField($data_field)) instanceof I2CE_FormField) {
                $values[$data_field] = 0;
                continue;
            }
            if ($fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
                if (!$fieldObj->keyIsSet($year)) {
                    $values[$data_field] = null;
                    continue;
                }
                $values[$data_field] =  $fieldObj->getValueOfKey($year);
            } else {
                if ($fieldObj instanceof I2CE_FormField_PERCENT_INT) {
                    $values[$data_field] = $fieldObj->getValue();
                } else if ($fieldObj instanceof I2CE_FormField_INT) {
                    if ($fieldObj->getOption('no_inflate')) {
                        $values[$data_field] =  $fieldObj->getValue();
                    } else {
                        $values[$data_field] = $this->inflateCosts($year,  $fieldObj->getValue());
                    }
                } else if ($fieldObj instanceof I2CE_FormField_PERCENT) {
                    $values[$data_field] = $fieldObj->getValue();
                } else if ($fieldObj instanceof I2CE_FormField_TOGGLE) {
                    $values[$data_field] = $fieldObj->getValue();
                } else if ($fieldObj instanceof I2CE_FormField_FLOAT) {
                    if ($fieldObj->getOption('no_inflate')) {
                        $values[$data_field] = $fieldObj->getValue();
                    } else {
                        $values[$data_field] = $this->inflateCosts($year,  $fieldObj->getValue());
                    }
                } else {
                    I2CE::raiseError("Dont know what to do with " . get_class($fieldObj) , E_USER_WARNING);
                }
            }    
            if ($values[$data_field] === null) {
                $values[$data_field ]  = 0;
            }
        }
        return $values;
    }

    protected function evaluateYearlyFormula($type,$year) {
        //really need to make sure data fields can enable/disable inflation for INT and FLOAT
        if (!array_key_exists($type,$this->formulas) || !$this->formulas[$type]) {
            return 0.0;
        }
        $tot_formula = str_replace ("\n" , " " , preg_replace_callback( '/<<(.*?)>>/',array($this,'phpreplacer'),$this->formulas[$type]));
        //example becomes $tot_formula = "$values['students']*($values['tuition'] + $values['housing'] + $values['stipend'] + $values['othersup'])";
        $values = $this->getDataFieldValues($type,$year);
        $eval = '$cost = (float)' . $tot_formula . ';';
        eval($eval);        
        if (!isset($cost)) {
            I2CE::raiseError("Cost BADNESS" . get_class($this) . "\n\t$tot_formula\n" . print_r($values,true) , E_USER_WARNING);
            return 0;
        }
        if ($cost < 0) {
            I2CE::raiseError("Cost negative BADNESS" . get_class($this)  . "\n\t$tot_formula\n" . print_r($values,true) , E_USER_WARNING);
            return 0;
        }
        return  $cost;
    }

    protected function addYearlyTotalToExcel($sheet,$other_support,$row_offset, $col_offset = 0) {
        $start_year = $this->projection->start_year->year();        
        $duration =    $this->getDuration();
        $label_col =  PHPExcel_Cell::stringFromColumnIndex( $col_offset);
        $tot_row = $row_offset + count($this->data_fields['total']) + $other_support + 3;
        $label = 'Total';

        $cells = array();
        $this->data_row_offset = $row_offset + 3;
        if (!array_key_exists('total',$this->formulas) || !$this->formulas['total']) {
            $tot_formula = '0';
        } else {
            //convert fields  references into <<YEAR>>row
            $tot_formula = preg_replace_callback( '/<<(.*?)>>/',array($this,'excelreplacer'),$this->formulas['total']);
        }
        if ($other_support > 0) {
            $min_other = $row_offset + count($this->data_fields['total']) + 3;
            $max_other = $row_offset + count($this->data_fields['total']) + 3 + $other_support - 1;
            $tot_formula = "($tot_formula)+ SUM(<<YEAR>>{$min_other}:<<YEAR>>{$max_other})";
        }

        $data = array();
        $start_year_col =  PHPExcel_Cell::stringFromColumnIndex( 1 + $col_offset);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 1 + $col_offset);
            $data[$year] = array(
                'value'=>'=' . str_replace('<<STARTYEAR>>',$start_year_col,str_replace('<<YEAR>>', $year_col,  $tot_formula)),
                'style'=>array(
                    'font'=>array('bold'=>true),
                    'numberformat'=>array('code'=>'#,##0'),
                    ));
            $cells[$year] = $year_col . $tot_row;
        }
        $this->projection->addYearlyDataArrayToExcel($sheet,$tot_row,$label,$data );
        return $cells;
    }

    protected function addSelectedCadres($sheet,$cadre_row,$col_offset = 0) {
        $cadre_col =  PHPExcel_Cell::stringFromColumnIndex($col_offset );
        $cadre_col_start =  PHPExcel_Cell::stringFromColumnIndex($col_offset +1 );
        $duration =    $this->getDuration();
        $cadre_col_end =  PHPExcel_Cell::stringFromColumnIndex($col_offset +1 +  $duration -1);
        $sheet->getStyle($cadre_col . $cadre_row)->getFont()->setItalic(true);
        $sheet->SetCellValue($cadre_col . $cadre_row ,'Targeted Cadres:');
        if ($duration > 1) {
            $sheet->mergeCells("{$cadre_col_start}{$cadre_row}:{$cadre_col_end}{$cadre_row}");
        }
        $pools = $this->getTargetedPools();
        $cadres =array();
        foreach ($pools as $poolObj) {
            $cadres[] = $poolObj->getField('name')->getValue();
        }
        $sheet->SetCellValue($cadre_col_start . $cadre_row ,implode(",",$cadres));
        
    }


    protected $pools = false;

    public function getTargetedPools() {
        if (is_array($this->pools)) {
            return $this->pools;
        }
        if (!$this->ensureProjection()) {
            return array();
        }

        $this->pools = array();
        $ff = I2CE_FormFactory::instance();
        foreach ( $this->fields['pool']->getValue() as $poolid) {
            if ( ! ($poolObj = $ff->createContainer($poolid)) instanceof iHRIS_Pool) {
                continue;
            }
            $poolObj->populate();
            if ($poolObj->getParentID() != $this->projection->getID()) {
                continue;
            }
            $this->pools[$poolObj->getID()] =  $poolObj;

        }
        return $this->pools;
    }

    protected $projection;
    protected $inflation_rate;

    //really this should be in a targeted healthworker class, but time is tight any everything is targeted anyways
    public function getTargetedHealthWorkers($pool,$year) {
        if ($this->formulas['target']) {
            return $this->getTargetedHealthWorkersByFormula($pool,$year);
        } else {
            return $this->getTargetedHealthWorkersByPool($pool,$year);
        }
    }

    public function getTargetedHealthWorkersByPool($pool,$year) {
        if (!$this->ensureProjection()) {
            I2CE::raiseError("No ensured projection");
            return 0;
        }
        $this->projection->populateChildren('pool');
        if (!array_key_exists('pool',$this->projection->children) || count($this->projection->children['pool']) == 0) {
            I2CE::raiseError("No pools");
            return 0;
        }
        if (($pos = strpos($pool,'|')) !== false) {
            $id = substr($pool,$pos+1);
        } else {
            $id = $pool;
        }
        if (!array_key_exists($id,$this->projection->children['pool'])) {
            return 0;
        }
        $poolObj = $this->projection->children['pool'][$id];
        if (!$poolObj instanceof iHRIS_Pool) {
            I2CE::raiseError("No pool obj for id $id");
            return 0;
        }
        return $poolObj->getProjectedRuralHWs($year,true);
    }


    
    public function getTargetedHealthWorkersByFormula($pool,$year) {        
        if (!$this->ensureProjection()) {
            I2CE::raiseError("Not ensure projection");
            return 0.0;
        }
        if ( ($target = max(0,  $this->evaluateYearlyFormula('target', $year))) == 0) {
            return 0.0;
        }
        //now we need to look at the total hws in all pools  targeted and computer ratio with pools
        $ratio = $this->getPoolRatio($pool,$year);
        return max(0.0,$ratio * $target);
    }

    protected $ratios = null;

    protected function getPoolRatio($pool,$year) {
        if ($this->ratios === null) {
            $this->ratios = array();
            $start_year = $this->projection->start_year->year();        
            $duration = $this->projection->duration;

            if ( ($pools = $this->getField('pool'))  instanceof I2CE_FormField_MAP_MULT) {
                $totals = array();
                for ( $y = $start_year; $y < $start_year + $duration; $y++) {
                    $total  = 0.0;
                    $targets = array();
                    foreach ($pools->getValue()  as $p) {            
                        if (!is_array($p) || count($p) != 2) {
                            continue;
                        }
                        list($p_form,$p_id) = $p;
                        $hws = 0;
                        if (array_key_exists($p_id,$this->projection->children['pool']) && ($poolObj = $this->projection->children['pool'][$p_id]) instanceof iHRIS_Pool) {
                            $hws = max(0,$poolObj->getProjectedRuralHWs($y));
                        }
                        $total += $hws;
                        $targets[$p_form . '|' . $p_id] = $hws;
                    }
                    foreach ($pools->getValue()  as $p) {            
                        if (!is_array($p) || count($p) != 2) {
                            continue;
                        }
                        $p = implode("|", $p);
                        $ratio = 0.0;
                        if ($total > 0) {
                            $ratio = $targets[$p] / $total;
                        }
                        $this->ratios[$y][$p] = $ratio;
                    }
                }
            }
        }
        if ( strpos($pool,'|')  === false) {
            $pool = "pool|" . $pool;
        }
        if (!array_key_exists($year,$this->ratios)) {
            I2CE::raiseError("No ratio for year $year");
            return 0.0;
        }
        if (!array_key_exists($pool,$this->ratios[$year])) {
            I2CE::raiseError("No ratio for year $year / $pool");
            return 0.0;
        }
        return $this->ratios[$year][$pool];
    }


    public function getHWRatio($pool,$year) {
        if (($pos = strpos($pool,'|')) !== false) {
            $id = substr($pool,$pos+1);
        } else {
            $id = $pool;
        }
        $targets = $this->getField('pool');
        if (!$targets instanceof I2CE_FormField_MAP_MULT) {
            I2CE::raiseError("Bad targets");
            return 0.0;
        }
        $found = 0.0;
        $tot  = 0.0;
        foreach ($targets->getValue()  as $target) {            
            if (!is_array($target) || count($target) != 2) {
                continue;
            }
            list($t_form,$t_id) = $target;
            $hws =  $this->getTargetedHealthWorkers($t_id,$year);
            if ($t_id == $id) {
                $found = $hws;
            }
            $tot+= $hws;
        }
        if ($tot == 0) {
            return 0.0;
        }
        return ((float) $found) / ((float) $tot);
    }


    
    public function getHWCostComponent($pool,$year) {
        $joint_interventions = array();
        if ($this->hasMeta('joint_interventions')) {
            $joint_interventions = $this->getMeta('joint_interventions');
        }
        if (!is_array($joint_interventions) || count($joint_interventions)==0) {
            $ratio = max(0,$this->getHWRatio($pool,$year));
            $total_cost = max(0,$this->getCostByYear($year));
            return ((float)$ratio) * ((float)$total_cost);
        } else {
            return $this->getJointHWCostComponent($pool,$year,$joint_interventions);            
        }
    }


    protected $joint_targeted_hws=array();

    /**
     * used for example in A1 if management costs need to be spread according to edu_support adn stupde
     */
    public function getJointHWCostComponent($pool,$year,$joint_interventions) {
        if (!$this->ensureProjection()) {
            return 0.0;
        }
        $total_cost = max(0,$this->getCostByYear($year));
        if ($total_cost == 0) {
            return 0.0;
        }
        if (($pos = strpos($pool,'|')) !== false) {
            $id = substr($pool,$pos+1);
        } else {
            $id = $pool;
        }
        if (!array_key_exists($year,$this->joint_targeted_hws) || !is_array($this->joint_targeted_hws[$year])) {            
            //first we need to see if the requested pool is a target.
            $this->joint_targeted_hws[$year] = array();

            foreach ($this->getTargetedPools()  as $t_id=>$targetObj) {            

                $this->joint_targeted_hws[$year][$t_id] = 0;
                foreach ($joint_interventions as $intervention) {
                    $this->projection->populateChildren( array($intervention) );
                    if (!array_key_exists($intervention,$this->projection->children)) {
                        continue;
                    }
                    foreach ($this->projection->children[$intervention] as $interventionObj) {
                        if (!$interventionObj->enabled) {
                            continue;
                        }
                        $this->joint_targeted_hws[$year][$t_id] += $interventionObj->getTargetedHealthWorkers($t_id,$year);
                    }
                }
            }
        }

        $total_hws = array_sum($this->joint_targeted_hws[$year]);
        if ($total_hws == 0) {
            //avoid division by 0
            return 0.0;
        }

        if (!array_key_exists($id,$this->joint_targeted_hws[$year])) {
            $this->joint_targeted_hws[$year][$id] = 0;
        }

        $ratio = max(0,$this->joint_targeted_hws[$year][$id]/$total_hws);        

        return ((float)$ratio) * ((float)$total_cost);
    }


    public function getHWCostComponents() {
        $targets = $this->getField('pool');
        if (!$targets instanceof I2CE_FormField_MAP_MULT) {
            return array();
        }
        $ratios = array();
        $start_year = $this->projection->start_year->year();        
        $duration =  $this->getDuration();
        foreach ($targets->getValue()  as $target) {
            list($t_form,$t_id) = $target;
            $poolid = 'pool|' . $t_id;
            $ratios[$poolid] = array();
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $ratios[$poolid][$year] = $this->getHWCostComponent($t_id,$year);
            }
        }
        return $ratios;
    }

    protected $costs = array();

    /**
     * Get the cost for the indicated year
     * @param int $year
     */
    public function getCostByYear($year) {
        if (!$this->ensureProjection()) {
            return 0.0;
        }
        if (array_key_exists($year,$this->costs)) {
            return $this->costs[$year];
        }
        $start_year = $this->projection->start_year->year();        
        if ($year <  $start_year || $year >= $start_year + $this->getDuration()) {
            return 0.0;
        }
        if ($this->startYearOnly() && $year > $start_year) {
            return 0.0;
        }
        $cost =$this->_getCostByYear($start_year,$year - $start_year) + $this->getOtherCostByYear($year);
        $this->costs[$year] = $cost;
        return $cost;
    }

    protected function getOtherCostByYear($year) {
        $cost = 0;
        foreach ($this->getChildren('othercost') as $otherCost) {
            if (!$otherCost instanceof iHRIS_OtherCost) {
                continue;
            }
            $cost += $otherCost->getCostByYear($year);
        }
        return $cost;
    }

    protected function ensureProjection() {
        if ($this->projection instanceof iHRIS_Projection) {
            return true;
        }
        $ff = I2CE_FormFactory::instance();
        if ( !($projection = $ff->createContainer($this->getParent())) instanceof iHRIS_Projection) {
            return false;
        }
        $this->projection = $projection;
        $this->projection->populate();
        $this->projection->populateChild('financial');
        $this->populateChild('othercost');
        $this->inflation_rate = 0;
        if (array_key_exists('financial',$this->projection->children) && is_array($this->projection->children['financial']) && count($this->projection->children['financial']) == 1) {
            reset($this->projection->children['financial']);
            $financial = current($this->projection->children['financial']);
            if (is_numeric($financial->inflation)) {
                $this->inflation_rate = $financial->inflation;
            }
        }
        return true;
    }

    protected function calculateFixedCosts($fields) {
        $cost = 0;
        foreach ($fields as $field) {
            if (!array_key_exists($field,$this->fields) ||   !$this->fields[$field]->isValid() || !is_numeric( $this->fields[$field]->getValue())  ) {
                continue;
            }
            $cost += $this->fields[$field]->getValue();
        }
        return $cost;
    }

    protected function inflateCosts($year,$cost) { 
        $start_year = $this->projection->start_year->year();                
        if ($year < $start_year) {
            return $cost;
        }
        $year_offset =  $year  - $start_year;
        for ($i=0; $i <= $year_offset -1; $i++) {
            //if $start_year is 2010, and $year_offset is 2, then we want the 2012 cost.  then $i=0,1 and $start_year + $i = 2010,2011,2012
            $cost *=    1.0+ ($this->projection->getInflationRate($start_year + $i ) / 100.0 );
        }
        return $cost;
    }





}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
