<?php
/**
* © Copyright 2012 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris-retain
* @subpackage retain
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v1.2.0.120
* @since v1.2.0.120
* @filesource 
*/ 
/** 
* Class iHRIS_Intervention_D2_Transport
* 
* @access public
*/


class iHRIS_Intervention_D2_transport extends iHRIS_Intervention {

    protected function loadFormulas() {
        $this->formulas = array('target' => false);
        $this->data_fields = array('total'=>array('cost_vehicle','cost_maintain','units','life'));
    }

    protected function _getCostByYear($start_year,$year_offset){
        $num_cars = 0;
        $values = $this->getDataFieldValues('total',$start_year);
        $life =$values['life'];
        $start = max($start_year, $start_year + $year_offset - $life + 1); 
        $num_cars = 0;
        $new_cars = 0;
        for ($year = $start; $year <= $start_year + $year_offset; $year++) {
            $values = $this->getDataFieldValues('total',$year);
            $new_cars = $values['units'];
            $num_cars += $new_cars;
        } 
        //$values will have the value of $this->getDataFieldValues('total',$start_year + $year_offset), and $new_cars will be the # of cars added in $start_year+$year_offset
        
        //I2CE::raiseMessage("For $start_year + $year_offset we have $num_cars cars with $new_cars new cars");
        return $num_cars * $values['cost_maintain'] + $new_cars * $values['cost_vehicle'];        
    }


    protected function addYearlyTotalToExcel($sheet,$other_support,$row_offset, $col_offset = 0) {
        $start_year = $this->projection->start_year->year();        
        $this->data_row_offset = $row_offset + 3;
        $duration =    $this->getDuration();
        $label_col =  PHPExcel_Cell::stringFromColumnIndex( $col_offset);
        $tot_row = $row_offset + count($this->data_fields['total']) + $other_support + 3;
        $label = 'Total';

        $cells = array();
        if ( ($index = array_search('units', $this->data_fields['total'])) === false ) {
            return $cells;
        }
        $index += $row_offset + 3;  //this is the row for the units data field


        $values = $this->getDataFieldValues('total',$start_year);
        $life =$values['life'];


        $other_formula  ='';
        if ($other_support > 0) {
            $min_other = $row_offset + count($this->data_fields['total']) + 3;
            $max_other = $row_offset + count($this->data_fields['total']) + 3 + $other_support - 1;
            $other_formula = " + SUM(<<YEAR>>{$min_other}:<<YEAR>>{$max_other})";
        }

        $data = array();
        $start_year_col =  PHPExcel_Cell::stringFromColumnIndex( 1 + $col_offset);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 1 + $col_offset);
            $current = array();
            $year_offset = $year - $start_year;
            $start = max($start_year, $start_year + $year_offset - $life + 1); 
            for ($car_year = $start; $car_year <= $start_year + $year_offset; $car_year++) {
                $current[] = PHPExcel_Cell::stringFromColumnIndex($car_year - $start_year + 1 + $col_offset) . $index;
            } 
            $current = implode("+",$current);
            $tot_formula = "<<cost_vehicle>>*<<units>> + <<cost_maintain>>*($current)";
            $tot_formula = preg_replace_callback( '/<<(.*?)>>/',array($this,'excelreplacer'),$tot_formula . $other_formula);
            $tot_formula = str_replace('<<STARTYEAR>>',$start_year_col,str_replace('<<YEAR>>', $year_col,  $tot_formula));
            $data[$year] = array(
                'value'=>'=' . $tot_formula,
                
                'style'=>array(
                    'font'=>array('bold'=>true),
                    'numberformat'=>array('code'=>'#,##0'),
                    ));
            $cells[$year] = $year_col . $tot_row;
        }
        $this->projection->addYearlyDataArrayToExcel($sheet,$tot_row,$label,$data );
        return $cells;
    }



}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
