<?php
/**
* © Copyright 2011 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris-retain
* @subpackage retain
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v1.1.1007
* @since v1.1.1007
* @filesource 
*/ 
/** 
* Class iHRIS_Module_Retain
* 
* @access public
*/


class iHRIS_Module_Retain  extends I2CE_Module {
  
    /*
     * Return the array of hooks available in this module.
     * @return array
     */
    public static function getHooks() {
        return array(
            'pre_page_action' => 'addOverview',
            );
    }


    public function addOverview($page) {
        if (!$page instanceof I2CE_Page || ! ($template = $page->getTemplate()) instanceof I2CE_Template) {
            return;
        }
        
        $template->addHeaderLink('mootools-core.js');
        $template->addHeaderLink('mootools-more.js');
        $template->addHeaderLink('messageBox.js');
        $template->addHeaderLink('messageBox.css');
    }


    public function upgrade($old_vers,$new_vers) {
        if (I2CE_Validate::checkVersion($old_vers,'<','1.2')) {
            if (! $this->addDefaultScenarios()) {
                return false;
            }
        }
        if (I2CE_Validate::checkVersion($old_vers,'<','1.2.0.61')) {
            if (! $this->addDescriptionsToInterventions()) {
                return false;
            }
        }
        if (I2CE_Validate::checkVersion($old_vers,'<','1.2.0.73')) {
            if (! $this->addCategoryToInterventions()) {
                return false;
            }
        }

        return true;
    }

    protected function addCategoryToInterventions() {
        $ff = I2CE_FormFactory::instance();
        $user = new I2CE_User();
        foreach (iHRIS_Projection::$interventions as $intervention) {
            if (! is_array($data = I2CE_FormStorage::listFields($intervention,array('category'))) ) {
                I2CE::raiseError("Could not search for intrevention:$intervention");
                continue;
            }
            $modified = false;
            foreach ($data as $id=>$data) {
                $desc = $intervention;
                I2CE::getConfig()->setIfIsSet($desc,"/modules/forms/forms/$intervention/display");
                $intObj = $ff->createContainer($intervention . '|' . $id);
                if (!$intObj instanceof iHRIS_Intervention) {
                    I2CE::raiseError("Cannont instantiate $intervention|$id");
                    return false;
                }
                $modified = true;
                I2CE::raiseError("Setting category=$desc");
                $intObj->populate();
                $intObj->getField('category')->setValue($desc);
                $intObj->save($user);
                $intObj->cleanup();            
            }
            if (!$modified) {
                continue;
            }
            $cf = null;
            try {
                $cf = new I2CE_CachedForm($intervention);
            }catch (Exception $e) {
                I2CE::raiseError("Could not create cached for $intervention");
            }
            if ($cf) {
                $cf->dropTable();
            }
        }
        return true;
    }



    protected function addDescriptionsToInterventions() {
        $ff = I2CE_FormFactory::instance();
        $user = new I2CE_User();
        foreach (iHRIS_Projection::$interventions as $intervention) {
            if (! is_array($data = I2CE_FormStorage::listFields($intervention,array('description'))) ) {
                I2CE::raiseError("Could not search for intrevention:$intervention");
                continue;
            }
            $count = 0;
            foreach ($data as $id=>$data) {
                if (!is_array($data) || ( array_key_exists('description',$data) &&  is_string($data['description']) && strlen(trim($data['description'])) > 0)) {
                    continue;
                }
                $count++;
                $desc = $intervention;
                I2CE::getConfig()->setIfIsSet($desc,"/modules/forms/forms/$intervention/display");
                $desc .=  ' (' . $count .')';
                $intObj = $ff->createContainer($intervention . '|' . $id);
                if (!$intObj instanceof iHRIS_Intervention) {
                    I2CE::raiseError("Cannont instantiate $intervention|$id");
                    return false;
                }
                $intObj->populate();
                $intObj->getField('description')->setValue($desc);
                $intObj->save($user);
                $intObj->cleanup();
            }

        }
        return true;


    }

    protected function addDefaultScenarios() {
        if (! is_array($ids = I2CE_FormStorage::search('projection'))) {
            I2CE::raiseError("Could not search for projections");
            return false;
        }
        $ff = I2CE_FormFactory::instance();
        $user = new I2CE_User();
        foreach ($ids as $id) {
            if (! ($projObj = $ff->createContainer('projection|' . $id)) instanceof iHRIS_Projection) {
                I2CE::raiseError("Could not instantiate projection $id");
                return false;
            }
            $projObj->populate();
            $child_ids = $projObj->getChildIds('scenario');                
            if (count($child_ids) > 0) {
                $projObj->cleanup();
                continue;
            }
            $projObj->addScenario('Default',"Starting Scenario", $user);
            $projObj->cleanup();
        }
        return true;
    }

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
