<?php
/**
* © Copyright 2010 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris
* @subpackage plan
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v3.1.5
* @since v3.1.5
* @filesource 
*/ 
/** 
* Class iHRIS_Intervention
* 
* @access public
*/


class iHRIS_OtherCost extends I2CE_Form {


    protected $projection = false;
    protected $intervention =false;

    /**
     * Get the cost for the indicated year
     * @param int $year
     */
    public function getCostByYear($year) {
        if (!$this->ensureProjection()) {
            return 0;
        }
        $start_year = $this->projection->start_year->year();        
        if ($year <  $start_year || $year >= $start_year + $this->projection->duration) {
            return 0;
        }
        return max(0,$this->fields['amount']->getValueOfKey($year));
    }

    protected function ensureProjection() {
        if ($this->projection instanceof iHRIS_Projection) {
            return true;
        }
        $ff = I2CE_FormFactory::instance();
        if ( !($this->intervention = $ff->createContainer($this->getParent())) instanceof iHRIS_Intervention) {
            return false;
        }
        if ( !($projection = $ff->createContainer($this->intervention->getParent())) instanceof iHRIS_Projection) {
            return false;
        }
        $this->projection = $projection;
        $this->projection->populate();
        return true;
    }
  

    
    /**
     * Ensure all the yearly inputs have the correct years as keys
     * @param int $start_year
     * @param int $end_year
     */
    public function ensureYearlyInputs($start_year,$end_year) {
        if ($end_year < $start_year) {
            return;
        }
        $input = 'amount';
        if (! ($assocList =$this->getField($input)) instanceof I2CE_FormField_ASSOC_LIST) {
            I2CE::raiseError("bad field $input");
            continue;
        }
        $existing_keys = $assocList->getKeys();
        $needed_keys = array();
        for ($year = $start_year; $year <= $end_year; $year++) {
            $needed_keys[] = $year;
            $assocList->ensureKey($year);
            if (!$assocList->keyIsSet($year)) {
                //$assocList->setValueOfKey($year,0);
            }
        }
        foreach (array_diff($existing_keys,$needed_keys) as $remKey) {
            $assocList->removeKey($remKey);
        }
    }



}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
