<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing financial forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle the adding or editing financial forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormFinancial extends iHRIS_PageFormParentProjection {
        
    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        if (!parent::loadObjects()) {
            return false;
        }
        if (! ($financial = $this->getPrimary()) instanceof iHRIS_Financial) {
            I2CE::raiseError("Primary form is not financial");
            return false;
        }
        if (!  ($projection = $this->getParent()) instanceof iHRIS_Projection) {
            I2CE::raiseError("Parent form is not a projection");
            return false;
        }
        if (!$projection->canAccess($this->user)) {
            I2CE::raiseError("User cannot access projection ". $projection->getID());
            return false;
        }

        $start_year = $projection->start_year->year();        
        $end_year = $start_year + $projection->duration -1;        
        if ($end_year < $start_year) {
            I2CE::raiseError("Bad year range $end_year > $start_year ");
            return false;
        }
        $financial->ensureYearlyInputs($start_year,$end_year);
        return true;
    }

    protected function onlyOne() {
        return true;
    }

        
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
