<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing change forms associated with a pool to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle the adding or editing change forms associated with a pool to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormIntervention extends I2CE_Page {


    /**
     * Load the HTML template files for editing.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();        
        return true;
    }

    /**
     * creates and populages a pool object based on reuqest variables
     * @returns mixed.  iHRIS_Pool on succes. false on failure
     */
    protected function loadProjection() {
        if (!$this->request_exists('parent')) {
            $this->userMessage("Invalid Projection Requested");
            return false;
        }
        if ($this->request_exists('parent')) {
            $parent = $this->request('parent');
            if (strpos($parent,'|')=== false) {
                I2CE::raiseError("Deprecated use of parent variable");
                $parent = 'projection|' . $parent;
            }
        } else {
            $parent = 'projection|0';
        }
        $factory = I2CE_FormFactory::instance();
        I2CE::raiseError("Creating $parent");
        $projection = $factory->createContainer( $parent);
        if (!$projection instanceof iHRIS_Projection) {
            return false;
        }
        $projection->populate();
        if (!$projection->canAccess($this->user)) {
            I2CE::raiseError("User cannot access projection ". $projection->getID());
            return false;
        }
        $this->template->setForm( $projection );
        return $projection;
    }

    /**
     * Initializes any data for the page
     * @returns boolean.  True on sucess. False on failture
     */
    protected function initPage() {
        if ( !($this->projection = $this->loadProjection()) instanceof iHRIS_Projection) {            
            return false;
        }

        return parent::initPage();
    }



        
    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        $factory = I2CE_FormFactory::instance();
        if ($this->isPost()) {
            $this->parentObj = $this->factory->createContainer($this->args['parent_form']);
            $this->parentObj->load($this->request());            
        } else {
            $this->parentObj = $factory->createForm( $this->request('parent'));
            $this->parentObj->populate();
        }
        parent::loadObjects();
        return true;
    }

        
        
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
