<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle the adding or editing forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormInterventions extends I2CE_Page {
        

   


        
    /**
     * Load the HTML template files for editing.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        $this->template->addHeaderLink('mootools-core.js');
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink('I2CE_SubmitButton.js');
        $this->template->appendFileById( "interventions.html", "tbody", "projection_form" );

        return true;
    }


    protected $projection;

    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function action() {
        if ( !$this->get_exists('parent') ) {
            return false;
        }
        $parent = $this->get('parent');
        if (strpos($parent,'|')=== false) {
            I2CE::raiseError("Deprecated use of parent variable");
            $parent =  'projection|' . $id;            
        }
        $factory = I2CE_FormFactory::instance();
        $this->projection = $factory->createContainer($parent);
        if (!$this->projection instanceof iHRIS_Projection) {
            I2CE::raiseError("Could not create projection form from $parent");
            return;
        }

        $this->projection->populate();
        if (!$this->projection->canAccess($this->user)) {
            I2CE::raiseError("User " .$this->user->username() . " cannot access projection ". $this->projection->getID());
            return false;
        }
        iHRIS_RetainBreadCrumbs::show($this,$this->projection);

        $interventions = array(
            'intervention_A1' => array('intervention_A1_develop_recruitment','intervention_A1_management', 'intervention_A1_recruit', 'intervention_A1_stipend','intervention_A1_edu_support','intervention_A1_other'),
            'intervention_A2' => array('intervention_A2_construct','intervention_A2_management','intervention_A2_maintenance','intervention_A2_other', 'intervention_A2_furnish_health_facility','intervention_A2_furnish_dorms', 'intervention_A2_furnish_school'),
            'intervention_A3' => array('intervention_A3_design_plan','intervention_A3_management', 'intervention_A3_student_support', 'intervention_A3_training', 'intervention_A3_other'),
            'intervention_A4' => array('intervention_A4_develop_curricula','intervention_A4_dissemination','intervention_A4_edu_materials', 'intervention_A4_other'),
            'intervention_A5' => array('intervention_A5_new_edu_program','intervention_A5_training_delivery', 'intervention_A5_support_hws', 'intervention_A5_other'),
            'intervention_B1' => array('intervention_B1_design_scope','intervention_B1_prepare_training','intervention_B1_training_delivery', 'intervention_B1_training_supervisors','intervention_B1_other'),
            'intervention_B2' => array('intervention_B2_design','intervention_B2_develop','intervention_B2_consult','intervention_B2_conduct', 'intervention_B2_support', 'intervention_B2_supervisor', 'intervention_B2_other'),
            'intervention_B3' => array('intervention_B3_cump','intervention_B3_relocation','intervention_B3_compliance','intervention_B3_management', 'intervention_B3_support', 'intervention_B3_train', 'intervention_B3_other'),
            'intervention_B4' => array('intervention_B4_design','intervention_B4_compliance','intervention_B4_management','intervention_B4_tuition', 'intervention_B4_other'),
            'intervention_C1' => array('intervention_C1_consultations','intervention_C1_financial_incentives','intervention_C1_other'),
            'intervention_D1' => array('intervention_D1_electric','intervention_D1_housing','intervention_D1_other','intervention_D1_repair','intervention_D1_school','intervention_D1_telecomm','intervention_D1_water'  ),
            'intervention_D2' => array('intervention_D2_medicine','intervention_D2_other','intervention_D2_repair','intervention_D2_routine','intervention_D2_support','intervention_D2_train','intervention_D2_upgrade','intervention_D2_transport'),
            'intervention_D3' => array('intervention_D3_grants','intervention_D3_management','intervention_D3_other','intervention_D3_outreach_support','intervention_D3_rural','intervention_D3_urban'  ),
            'intervention_D4' => array('intervention_D4_conduct','intervention_D4_consult','intervention_D4_design','intervention_D4_other','intervention_D4_training'),
            'intervention_D5'=> array('intervention_D5_conference','intervention_D5_journal','intervention_D5_networking','intervention_D5_other'),
            'intervention_D6'=> array('intervention_D6_conference','intervention_D6_design','intervention_D6_dis_awards','intervention_D6_health_days','intervention_D6_nat_awards','intervention_D6_other')
            );
        
        
        $tot = 0;
       
        foreach($interventions as $intervention => $activities){ 
            $num_activities = 0; 
            foreach($activities as $activity){
                $this->projection->populateChildren($activity);
                if (!array_key_exists($activity,$this->projection->children) || !is_array($this->projection->children[$activity])) {
                    continue;
                }
                $count =count ($this->projection->children[$activity]);
                if ($count > 0  &&  ($actList = $this->template->appendFileByName("activity_list.html","div","status_" . $activity)) instanceof DOMNode) {
                    $this->template->setDisplayData('count',$count,$actList);
                    $actListNode = $this->template->getElementByName("activity_list",0,$actList);
                    if ($actListNode instanceof DOMNode) {
                        $actListNode->setAttribute('id','activity_list_' . $activity);
                        if ( ($clickerNode = $this->template->getElementByName("clicker",0,$actList)) instanceof DOMNode) {
                            $clickerNode->setAttribute('onclick',"popper2(this,'activity_list_"  . $activity . "');");
                        }
                        foreach ($this->projection->children[$activity] as $id=>$actObj) {
                            $pools = $actObj->getTargetedPools();
                            $cadres = array();
                            foreach ($pools as $pool) {
                                if (!$pool instanceof iHRIS_Pool) {
                                    continue;
                                }
                                $cadres[] = $pool->name;
                            }
                            $descField = $actObj->getField('description');
                            $nameField = $actObj->getField('name');
                            if ($descField instanceof I2CE_FormField_STRING_LINE || $descField instanceof I2CE_FormField_STRING_MLINE) {
                                $has_desc = 1;
                                $link_desc = $descField->getDisplayValue() ;
                            } else if ($nameField instanceof I2CE_FormField_STRING_LINE || $nameField instanceof I2CE_FormField_STRING_MLINE) {
                                $has_desc = 1;
                                $link_desc = $nameField->getDisplayValue() ;
                            } else {
                                $has_desc = 0;
                                $link_desc = '';
                            }
                            $cadre_list  = implode(",",$cadres);
                            if (!($actNode = $this->template->appendFileByNode("activity_list_each.html","li",$actListNode))  instanceof DOMNode ) {
                                continue;
                            }
                            if (count($cadres) > 0) {
                                $has_cadre = 1;
                            } else {
                                $has_cadre = 0;
                            }
                            $this->template->setDisplayDataImmediate("has_cadre",$has_cadre,$actNode);
                            $this->template->setDisplayDataImmediate('has_desc',$has_desc,$actNode);

                            $url = preg_replace('/intervention_(.{2})_/','intervention\\1',$activity);
                            $link = $url . '?id=' . $activity . '|' . $id ;
                            $this->template->setDisplayDataImmediate("link",$link,$actNode);
                            $this->template->setDisplayDataImmediate("cadre_list",$cadre_list,$actNode);
                            $this->template->setDisplayDataImmediate("link_desc",$link_desc,$actNode);
                                    
                        }
                    }                            
                }
                $num_activities += $count;
                $tot += $num_activities;
            }
            if ($num_activities > 0 && ($node = $this->template->getElementById(strtolower(substr($intervention,-2)) . '_activities')) instanceof DOMElement) {
                $node->setAttribute('style','display:block');
                if (($node = $this->template->getElementById("show_" . strtolower(substr($intervention,-2)) )) instanceof DOMElement) {
                    $node->setAttribute('class','clicker_hide');
                }
            }
            $this->template->setDisplayData('count_' . $intervention,$num_activities);                
        }
        
      
        if ($tot > 0 ) {
            $this->template->setDisplayData('has_intervention',1);
        } else {
            $this->template->setDisplayData('has_intervention',0);
        }
        $this->template->setForm($this->projection);

        $this->template->setDisplayDataImmediate('instruct', '3');
        $instruct_base = array(1=>'financial',2=>'pools',3=>'intervention');
        $parent_id = $this->projection->getNameId();
        for ($i = 1; $i <=3; $i++) {
            $this->template->setDisplayDataImmediate("instruct_{$i}_link",$instruct_base[$i] . "?parent=" . $parent_id  . "&instruct=$i");
        }
        $this->template->setDisplayDataImmediate("instruct_4_link","view?noredirect=1&id=" . $parent_id  . "&instruct=$4");
    
        return true;
    }
        

        

                
    /**
     * Set the data to be displayed for the page.
     */
    protected function setDisplayData() {
        parent::setDisplayData();
        $this->template->setDisplayData( "projection_header", $this->getTitle() );
     }

        
  }


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
