<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle the adding or editing forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormParentProjection extends I2CE_PageForm {
        
    /**
     * Return the form name for this page.
     * 
     * It will be used for the default form template and php page for the form submission.
     * @param boolean $html Set to true if this is to be used for the html template page to load.
     * @return string
     */
    protected function getForm( $html=false ) { return $this->form_name; }

    /**
     * @var integer The record id number of the object being edited.
     */
    protected $id;
    /**
     * @var integer The recored if number of the parent of the object being edited
     */
    protected $parent_id;
    /**
     * The form name being edited by this page.
     * @var string
     */
    protected $form_name;

    /**
     * The link used to access this form
     * $var protected string $form_link
     */
    protected $form_link;
        
    /**
     * Sets the form link 
     * @param string $link
     */
    public function setFormLink($link) {
        $this->form_link = $link;
    }
        


    /**
     * Checks to see if there are any permissions in the page's args for the given action.
     * If so, it evaluates them.  If not returns true.
     * @returns boolean
     */
    protected function checkActionPermission($action) {
        if (!$this->form_name) {
            return false; //weirdness.  should just stop whatever is happening
        }
        return true;        
        if (!parent::checkActionPermission($action)) {
            return false;
        }
        $task =   "person_can_" . $action . "_child_form_" . $this->form_name ;
        return $this->hasPermission("task($task)");
    }



    /**
     * Create a new instance of this page.
     * 
     * This will call the parent constructor and then setup the base
     * template pages for the {@link I2CE_Template template}.  It also sets up the values
     * for the member variables.
     * @param array $args
     * @param string $request_remainder
     */
    public function __construct( $args,$request_remainder){
        parent::__construct($args,$request_remainder);
        $form_name = $args['page_form'];
        if(empty($form_name)) {
            I2CE::raiseError("No form name specified", E_USER_ERROR);
        }
        $this->form_name = $form_name;
    }
        
    /**
     * Load the HTML template files for editing.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        $this->template->appendFileById( "form_" . $this->getForm( true ) . ".html", "tbody", "projection_form" );
        iHRIS_RetainBreadCrumbs::show($this,$this->projection);
    }

    protected function onlyOne() {
        return false;
    }


                
    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        $name = $this->getForm();
        if ($this->isPost()) {
            $primary = $this->factory->createContainer($name);
            if (!$primary instanceof I2CE_Form) {
                return false;
            }
            $primary->load($this->post);
        } elseif ( $this->get_exists('id') ) {
            if ($this->get_exists('id')) {
                $id = $this->get('id');
                if (strpos($id,'|')=== false) {
                    I2CE::raiseError("Deprecated use of id variable");
                    $id = $name . '|' . $id;
                }
            } else {
                $id = $name . '|0';
            }
            $primary = $this->factory->createContainer($id);
            if (!$primary instanceof I2CE_Form || $primary->getName() != $name) {
                I2CE::raiseError("Could not create valid " . $name . "form from id:$id");
                return false;
            }
            $primary->populate();
        } elseif ( $this->get_exists('parent') ) {
            $primary = $this->factory->createContainer($name);
            if (!$primary instanceof I2CE_Form) {
                return;
            }
            $parent = $this->get('parent');
            if (strpos($parent,'|')=== false) {
                I2CE::raiseError("Deprecated use of parent variable");
                $parent =  'projection|' . $id;            
            }
            $primary->setParent($parent);
        }
        if ($this->isGet()) {
            $primary->load($this->get());
        }
        $this->projection = $this->factory->createContainer(  $primary->getParent());
        if (!$this->projection instanceof iHRIS_Projection) {
            I2CE::raiseError("Could not create projection form from " . $primary->getParent());
            return;
        }
        $this->projection->populate();
        if (!$this->projection->canAccess($this->user)) {
            I2CE::raiseError("User cannot access projection ". $this->projection->getID());
            return false;
        }
        if ($this->onlyOne() && $primary->getId() == '0' && $this->projection->getId() != '0') {
            $this->projection->populateChildren($name);
            if (array_key_exists($name,$this->projection->children) && is_array($this->projection->children[$name]) && count($this->projection->children[$name]) > 0) {
                reset($this->projection->children[$name]);
                $primary = current($this->projection->children[$name]);
            }
        }
        $this->setObject($primary);
        $this->setObject($this->projection,I2CE_PageForm::EDIT_PARENT);        
        if ($primary instanceof iHRIS_Yearly || $primary instanceof iHRIS_YearlyList) {
            $start_year = $this->projection->start_year->year();        
            $end_year = $start_year + $this->projection->duration -1;                
            $primary->ensureYearlyInputs($start_year,$end_year);
        }

        return true;
    }
        

        

                
    /**
     * Set the data to be displayed for the page.
     */
    protected function setDisplayData() {
        parent::setDisplayData();
        if ( !($form_link = $this->form_link)) {
            if ($this->module == 'I2CE') {
                $form_link  = $this->page;
            } else {
                $form_link  = $this->module .'/' . $this->page;
            }
        }
        $this->template->setDisplayData( "projection_header", $this->getTitle() );
        $this->template->setDisplayData( "projection_form", $form_link);
    }

        
    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited and return to the view page.
     * @global array
     */
    protected function save() {
        parent::save();
        if ($this->request('submit_type') == 'save') {
            $this->setRedirect(  "view?id=" . $this->getPrimary()->getParent() );
        } else if ($this->request('submit_type') == 'save_quit') {
            $this->setRedirect(  "home" );
        }
    }
                

    /**
     * Checks to see if the page is being submitted as a save operation.
     * @param boolean $check_invalid
     * @return boolean
     */
    protected function isSave($check_invalid = true) {
        if ( ! ( $this->isPost() && $this->post_exists('submit_type') && substr($this->post( 'submit_type' ),0,4) == "save" )) {
            return false;
        }
        if ($check_invalid) {
            //we are trying to save.. need to verify that all is validated
            $this->validate();
            if ($this->hasInvalid()) {
                return false;
            }
        }
        return true;
    }

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
