<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Page object to handle adding or editing projections.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle adding or editing projections.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormProjection extends I2CE_PageForm {

    /**
     * @var integer The record id number of the object being edited.
     */
    protected $id;

    /**
     * Create a new instance of this page.
     * 
     * This will call the parent constructor and then setup the base
     * template pages for the {@link I2CE_Template template}.  It also sets up the values
     * for the member variables.
     * @param array $args
     * @param string $request_remainder
     */
    public function __construct( $args,$request_remainder) {
        parent::__construct( $args,$request_remainder);
        $this->id = 0;
        if ( $this->isPost() ) {
            if ( $this->post_exists( 'id' ) ) {
                $this->id = $this->post('id');
            }
        } else {
            if ( $this->get_exists( 'id' ) ) {
                $this->id = $this->get('id');
            }
        }
    }

    /**
      * Perform the main actions of the page.
      */
    protected function action() {
        $intros = array("intro_1.html","intro_2.html","intro_3.html","intro_4.html","intro_5.html","intro_6.html","intro_7.html");
        foreach ($intros as $intro) {
            if (! ($intro_file = $this->template->findTemplate($intro ))) {
                continue;
            }
            $this->template->userMessage(file_get_contents($intro_file),'notice',false);
        }
            
        if ( $this->id != '0' ) {
            $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
            //$this->template->appendFileById( "menu_view_projection.html", "ul", "menuView" );
        } else {
            $this->template->setAttribute( "class", "active", "menuSetup", "a[@href='projection']" );
            //$this->template->appendFileById( "menu_main.html", "ul", "menuProjection" );
            $this->template->setAttribute( "class", "active", "menuProjection", "ul/li/a[@href='projection']" );
        }
        parent::action();
        $this->template->addFile( "instruct_projection.html", "div" );
    }


    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.
     */
    protected function loadObjects() {
        $factory = I2CE_FormFactory::instance();
        $this->setObject( $factory->createForm( "projection|". $this->id ) );
        return parent::loadObjects();
    }

    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited and return to the view page.
     * @global array
     */
    protected function save() {
        if (!($projection = $this->getPrimary()) instanceof iHRIS_Projection) {
            return;
        }
        if (($creator = $projection->getField("creator")) instanceof I2CE_FormField_MAP
            && (!$creator->isValid() || !$creator->isSetValue())
            && $this->user instanceof I2CE_User) {
            I2CE::raiseError("Creator of projection does not appear to be set.  Using " . $this->user->username());
            $creator->setValue(array("user"  , $this->user->username()));
            
        }
        parent::save();
        //$projection->addScenario('Default',"Starting Scenario", $this->user);
        $this->setRedirect(  "view?id=" . $this->getPrimary()->getNameId() );
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
