<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing financial forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle the adding or editing financial forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormScenario extends iHRIS_PageFormParentProjection {


    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        if ($this->request_exists('active')) {
            if ( ! (  $scenario = $this->getPrimary()) instanceof iHRIS_Scenario) {
                I2CE::raiseError("Bad scenario");
                return false;
            }
            if (! ($projection = I2CE_FormFactory::instance()->createContainer($scenario->getParent())) instanceof iHRIS_Projection) {
                I2CE::raiseError("Bad scenario parent");
                return false;
            }
            I2CE::raiseError("Setting scenario on " . $scenario->getFormID());
            $projection->setScenario($scenario,true);
            $this->setRedirect(  "view?id="  . $projection->getNameID()  . '&tab=scenarios');
            return true;
        } else if ($this->request_exists('delete')) {
            $this->loadObjects();
            if ( ! (  $scenario = $this->getPrimary()) instanceof iHRIS_Scenario) {
                I2CE::raiseError("Bad scenario");
                return false;
            }
            $parent = $this->getPrimary()->getParent();
            I2CE::raiseError("Deleting on " . $scenario->getFormID());
            $scenario->delete();
            $this->setRedirect(  "view?id="  . $parent  . '&tab=scenarios');
            return true;
        } else {
            parent::action();        
        }
    }

    protected function setupGetPost($get = null,$post=null, $strip = true) {
        parent::setupGetPost($get,$post,$strip);
        if (array_key_exists('scenario',$this->get) && array_key_exists('id',$this->get) && $this->get['id']) {
            $this->get['parent'] = $this->get['id'];
            if ($this->get['scenario']) {
                $this->get['id'] = 'scenario|' . $this->get['scenario'];
            } else {
                unset($this->get['id']);
            }
        }
    }

    protected function save() {
        parent::save();
        $this->projection->setScenario($this->getPrimary(),true,$this->user);
        $this->setRedirect(  "view?id="  . $this->getPrimary()->getParent()  . '&tab=scenarios');
    }
            
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
