<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a A2 Intervention
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a cadre projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageIntervention extends I2CE_Page{ 
    /**
     * @var iHRIS_Projection The projection object associated with this page.
     */
    protected $projection;


    /**
     * @var iHRIS_Intervention The projection object associated with this page.
     */
    protected $intervention;


    /**
     * @var int $start_year
     */
    protected $start_year;


    /**
     * @var int $end_year
     */
    protected $end_year;
   
    /**
     * Load the HTML template files for editing.
     */
    protected function breadcrumbs() {
        $this->template->addHeaderLink('mootools-core.js');
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink("I2CE_SubmitButton.js");
        $this->template->addHeaderLink('I2CE_DuplicitiveForm.js');
        $js = "if (window.addEvent) {window.addEvent('load',function(e) {new I2CE_DuplicitiveForm({always_one:false,add_location_id:'other_control_row'});});}";
        $this->template->addHeaderText($js,'script',"dupl_form");
        $this->template->appendFileById( "projection_breadcrumb.html", "div", "breadcrumb", true );
        $this->template->setAttribute( "class", "active", "breadcrumb",  ".//*[@id='breadcrumb_step_3']" );
        $this->template->setDisplayDataImmediate('instruct', '3');
        $instruct_base = array(1=>'financial',2=>'pools',3=>'intervention');
        if (!$this->projection) {
            return;
        }
        $parent_id = $this->projection->getNameID();
        for ($i = 1; $i <=3; $i++) {
            $this->template->setDisplayDataImmediate("instruct_{$i}_link",$instruct_base[$i] . "?parent=" . $parent_id  . "&instruct=$i");
        }
        $this->template->setDisplayDataImmediate("instruct_4_link","view?noredirect&id=" . $parent_id  . "&instruct=$4");
    }




     
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        parent::action();
        $this->loadObjects();
        if ($this->request_exists('delete')) {
            I2CE::raiseError("Deleting on " . $this->intervention->getFormID());
            $this->intervention->delete();
            $redirect = 'view?id=' . $this->projection->getNameId() . '#jump_intervention';                
            $this->setRedirect($redirect);
            return true;
        } else if ($this->request_exists('toggle')) {
            I2CE::raiseError("Toggling");
            $this->projection->setScenario();
            if (($scenario = $this->projection->getCurrentScenario()) instanceof iHRIS_Scenario) {
                I2CE::raiseError("Have scenario -- toggle on " . $this->intervention->getFormID());
                $scenario->toggleIntervention($this->intervention, $this->user);
                if ($this->request_exists('chart')) {
                    $redirect = 'view?chart=' . $this->request('chart') . '&id=' . $this->projection->getNameId() . '#jump_intervention';
                } else {
                    $redirect = 'view?id=' . $this->projection->getNameId() . '#jump_intervention';                
                }
                $this->setRedirect($redirect);
            }
            return true;
        }
        if ($this->post_exists('submit_type') && substr($this->post('submit_type') =='save',0,4)   && !$this->intervention->hasInvalid()) {            
            return $this->doSave();
        } else {
            return $this->doDisplay();
        }
    }


    protected function doDisplay() {
        iHRIS_RetainBreadCrumbs::show($this,$this->projection,3,false);
        //$this->breadcrumbs();
        return true;
    }

    protected function doSave() {
        $old_id = $this->intervention->getID();
        $this->intervention->save( $this->user );
        $new_id = $this->intervention->getID();
        //add this intervention to default scenario

        if (($new_id != $old_id) &&     (  ($scenario = $this->projection->getScenario('default')) instanceof iHRIS_Scenario)) {
            I2CE::raiseError("Adding new intervetion ($new_id) to default scenario");
            //this is a new intervention so we toggle it on
            $scenario->toggleIntervention($this->intervention, $this->user);
        }
        $others = $this->intervention->getChildren('othercost');
        if (is_array($others)) {
            $all_others = array_merge($this->new_others,$this->intervention->getChildren('othercost'));
        } else {
            $all_others = $this->new_others;
        }

        foreach ($all_others as $other) {
            $other->setParent($this->intervention);
            if (!$other instanceof I2CE_Form) {
                I2CE::raiseError("Bad other cost form");
                continue;
            }
            $other->save($this->user);
        }


        $page ='';
        if ($this->request_exists('redirect')) {
            $page = $this->request('redirect');
        } 
        if (!$page) {
            $page = 'intervention';
        }
        $this->setRedirect($page . '?parent=' . $this->projection->getNameId());
    }


    protected $new_others = array();
    protected function loadObjects() {
        $factory = I2CE_FormFactory::instance();
        if ($this->request_exists('id')) {
            $this->intervention = $factory->createForm($this->request('id'));
            if (!$this->intervention instanceof iHRIS_Intervention) {
                I2CE::raiseError("Bad Id=" . $this->request('id'));
                return false;
            }
            $this->intervention->populate();
        } else {
            $this->intervention = $factory->createForm($this->args['page_form']);
            if (!$this->intervention instanceof iHRIS_Intervention) {
                I2CE::raiseError("Bad intervention");
                return false;
            }
            $this->intervention->load($this->request());            
        }
        $this->template->setTitle( 'Add/Update Activity: ' . $this->intervention->category);


        $this->template->setForm($this->intervention);
        if (!$this->isPost()) {
            if ($this->request_exists('parent')) {
                $this->projection = $factory->createForm($this->request('parent'));
            } else {
                $this->projection = $factory->createForm($this->intervention->getParent());
            }
            if (!($this->projection instanceof iHRIS_Projection)) {
                I2CE::raiseError("Bad Projection");
                return false;
            }
            $this->projection->populate();
        } else {
            $this->projection = $factory->createForm('projection');
            if (!($this->projection instanceof iHRIS_Projection)) {
                I2CE::raiseError("Bad posted Projection");
                return false;
            }
            $this->projection->load( $this->request());
            //$this->projection->populate();
        }
        $this->intervention->setParent($this->projection);
        $this->template->setForm($this->projection);
        $this->start_year = $this->projection->start_year->year();
        $this->end_year = $this->projection->start_year->year() + $this->projection->duration -1;        
        $this->intervention->ensureYearlyInputs($this->start_year,$this->end_year);
        $cadreNode = $this->template->getElementById('cadre_select');    
        if ($this->intervention instanceof iHRIS_Intervention) {
            $add_node = $this->template->loadFile('form_selectcadres.html','tbody');
            if ($cadreNode instanceof DOMNode && $add_node instanceof DOMNode) {
                $this->template->appendNode($add_node,$cadreNode);
            }            
            if (substr($this->intervention->getName(),-5) == 'other') {
                foreach ($this->template->query('.//*[@class="activity_subtitle"]',$add_node) as $sub_tit_node) {
                    $this->template->removeNode($sub_tit_node);
                }
            }
        }
        $page ='';
        if ($this->request_exists('redirect')) {
            $page = $this->request('redirect');
        } 
        if (!$page) {
            $page = 'intervention';
        }
        $this->template->setDisplayDataImmediate('redirect',$page);
        if ( ($returns = $this->template->query("//*[@id='button_return']")) instanceof DOMNodeList) {
            foreach ($returns as $return) {
                if (!$return instanceof DOMElement || !$return->hasAttribute('href')) {
                    continue;
                }
                $href = $return->getAttribute('href');
                $pos = strpos($href,'?');
                if ($pos === false) {
                    continue;
                }
                $return->setAttribute('href', $page . substr($href,$pos));
            }
        }
        
        $has_other = in_array('othercost',$this->intervention->getChildForms());
        if ($has_other) {
            $this->intervention->populateChild('othercost');
        }
        $others = $this->intervention->getChildren('othercost');
        if (!is_array($others)) {
            $others = array();
        }
        if ($has_other) {
            if ($this->isPost()) {
                foreach ($others as $other) {
                    I2CE::raiseError("Loading from post " . $other->getNameID() );
                    if (!$other instanceof I2CE_Form) {
                        I2CE::raiseError("Bad other cost form");
                        continue;
                    }
                    $other->load($this->request());                   
                }
                //now we need to add in any new ones that we are added to the page
                $req = $this->request();
                if (array_key_exists('forms',$req) && is_array($req['forms']) && array_key_exists('othercost',$req['forms']) && is_array($req['forms']['othercost']) && array_key_exists(0,$req['forms']['othercost']) && is_array($req['forms']['othercost'][0])) {
                    $ff = I2CE_FormFactory::instance();
                    foreach ($req['forms']['othercost'][0] as $otherData) {
                        $otherObj = $ff->createContainer('othercost');
                        if (!$otherObj instanceof I2CE_Form) {
                            I2CE::raiseError("Could not create other cost");
                            continue;
                        }
                        $otherObj->setFromPost($otherData,false);
                        $this->new_others[] = $otherObj;
                        $otherObj->setParent($this->intervention);
                        //$this->intervention->addChildForm($otherObj); -- can't use this for more than one anonymous forms as it overwrites the id=0 index
                    }
                }
            } else {
                if (count($others) == 0) {
                    //make sure there is at least one
                    if ( ($otherCost =$factory->createContainer('othercost')) instanceof iHRIS_OtherCost) {
                        $this->intervention->addChildForm($otherCost);
                    }
                }
            }
        }
        if ($has_other) {
            if (is_array($others)) {
                $all_others = array_merge($this->new_others,$this->intervention->getChildren('othercost'));
            } else {
                $all_others = $this->new_others;
            }
        } else {
            $all_others = array();
        }
        
        if ($has_other) {
            $add_node = $this->template->loadFile('form_othercosts.html','tbody');
            $otherNode = $this->template->getElementById('add_content');            
            if ( $otherNode instanceof DOMNode && $add_node instanceof DOMNode) {
                foreach ($all_others as $i=>$otherCost) {
                    if (!$otherCost instanceof I2CE_Form) {
                        I2CE::raiseError("Bad other cost form");
                        continue;
                    }
                    $cloneNode = $add_node->cloneNode(true);
                    $otherCost->ensureYearlyInputs($this->start_year,$this->end_year);
                    $this->template->appendNode($cloneNode,$otherNode);
                    $this->template->setForm($otherCost,$cloneNode);
                }
                
            }

        }

        if ($this->isPost()) {
            $this->intervention->validate();
            if ($has_other) {
                foreach ($all_others as $other) {
                    if (!$other instanceof I2CE_Form) {
                        I2CE::raiseError("Bad other cost form");
                        continue;
                    }

                    $other->validate();
                }
            }
        }
    }


    
    

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
