<?php
/**
 * @copyright © 20011 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @package iHRIS
 * @subpackage Retain
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @since v1.1.0
 * @version v1.1.0
 */
/**
 * Handles working with pChart to display the image based on a CSV file for data.
 * 
 * @package iHRIS
 * @subpackage Retain
 * @access public
 */

class iHRIS_PagePChartRetain extends I2CE_PagePChart {

    /**
     * The projection for this chart.
     * @var iHRIS_Projection
     */
    protected $projection;

    /***
     * @var protected mixed $ymax.  False if no y max.  otherwise an int.
     */
    protected $ymax =false;
    /**
     * Set the data for the chart.
     * @return boolean
     */
    protected function setData() {
        $id = false;
        if ( $this->request_exists('id') && $this->request('id') ) {
            $id = $this->request('id');
        }
        $factory = I2CE_FormFactory::instance();
        if ( !$id || !( $this->projection = $factory->createForm( $id ) ) 
                instanceOf iHRIS_Projection ) {
            I2CE::raiseError( "No valid id given for chart projection." );
            return false;
        }
        $this->projection->populate();
        if (!$this->projection->canAccess($this->user)) {
            I2CE::raiseError("User " . $this->user->username() . " cannot access projection ". $this->projection->getID());
            return false;
        }
        if ( count( $this->request_remainder ) == 0 ) {
            I2CE::raiseError( "No valid chart type given for chart." );
            return false;
        }
        $this->setPalette( "retain" );

        $this->projection->populate();
        $this->projection->fixupFinancials();
        $this->projection->fixupPools();
        $this->projection->fixupInterventions();
        $comp_id = false;
        $compProj = false;
        if ( $this->request_exists('compare') && $this->request('compare') && ($comp_id = $this->request('compare')) &&  ($compProj = $factory->createForm("projection|" .$comp_id )) instanceof iHRIS_Projection) {
            $this->style == 'line';
            $base = $this->projection->getTotalCostsByYear();
            foreach ($base as $key=>$vals) {
                $yearb = array_keys($vals);
                $this->chartData->AddPoints($vals,$key . '-' . $this->projection->getField('name')->getValue());
            }
            $compProj->populate();
            $compProj->fixupFinancials();
            $compProj->fixupPools();
            $compProj->fixupInterventions();            
            $comp = $compProj->getTotalCostsByYear();
            foreach ($comp as $key=>$vals) {
                $yearc = array_keys($vals);
                $this->chartData->AddPoints($vals,$key . '-' . $compProj->getField('name')->getValue());
            }     
            $years = array_unique(array_merge($yearb,$yearc));    
            sort($years);
            $this->chartData->addPoints( $years, "Years" );
            $this->chartData->setSerieDescription( "Years", "Year" );
            $this->chartData->setAbscissa( "Years" );

            
        }  else {
            reset($this->request_remainder);
            $chart = current( $this->request_remainder );
            $method = 'get_chart_data_' . $chart;
            if ( !method_exists( $this->projection, $method ) ) {
                I2CE::raiseError( "Method ($method) not found in projection." );
                return false;
            }
            $data = $this->projection->$method();
            $headers = array_shift( $data );
            $corner = array_shift( $headers );
            $max = array();
            foreach( $data as $row ) {
                $title = array_shift( $row );
                $this->chartData->addPoints( $row, $title );
                if ($this->style == 'stackedbar') {
                    foreach ($row as $year=>$val) {
                        if (!array_key_exists($year,$max)) {
                            $max[$year] = 0;
                        }
                        if (is_scalar($val) && $val > 0) {
                            $max[$year] += $val;
                        }
                    }
                }
            }
            if ($this->style == 'stackedbar') {
                $this->ymax = (int) (1.05 *max($max));
            }

            $this->chartData->addPoints( $headers, "Years" );
            $this->chartData->setSerieDescription( "Years", "Year" );
            $this->chartData->setAbscissa( "Years" );
        }
        return true;
    }

    /**
     * Get the options for drawing the given style.
     * @param string $style
     */
    protected function getStyleOptions( $style ) {
        return array( 
                'Surrounding' => -100,
                );
    }

    /**
     * Draw the image.
     * @return boolean
     */
    protected function drawImage() {
        $this->setupImage( 750, 480 );
        reset($this->request_remainder);
        $chart = current( $this->request_remainder );        
        $this->chartImage->drawFilledRectangle( 0, 0, 750, 480,
                array( 'R' => 102, 'G' => 102, 'B' => 102 ) );
        $this->chartImage->setGraphArea( 60, 50, 710, 400 );
        $this->chartImage->drawFilledRectangle( 60, 50, 710, 400,
                array( 'R' => 95, 'G' => 105, 'B' => 85,
                   'BorderR' => 0, 'BorderG' => 0, 'BorderB' => 0 ) );
        $this->chartImage->setFontProperties( array( 
                    'R' => 255, 'G' => 255, 'B' => 238,
                    ) );
        if ($this->ymax > 0) {
            $this->chartImage->drawScale( 
                array( 
                    'GridR' => 0, 'GridG' => 0, 'GridB' => 0,
                    'GridAlpha' => 20,
                    'GridTicks' => 0,
                    'Mode' => SCALE_MODE_MANUAL,
                    'ManualScale' => array( 0 => array( 'Min' => 0, 'Max' => $this->ymax)),
                    'DrawXLines' => false,
                    ) );
        }else {
            $this->chartImage->drawScale( 
                array( 
                    'GridR' => 0, 'GridG' => 0, 'GridB' => 0,
                    'GridAlpha' => 20,
                    'GridTicks' => 0,
                    'Mode' => SCALE_MODE_START0,
                    'DrawXLines' => false,
                    ) );
        }
        $this->chartImage->setFontProperties( array( 
                    'R' => 0, 'G' => 0, 'B' => 0,
                    ) );
        if ($this->style == 'line') {
            $this->chartImage->drawLegend( 25, 20, array( 
                                               "Mode" => LEGEND_VERTICAL,
                                               "Style" => LEGEND_BOX,
                                               'R' => 255, 'G' => 255, 'B' => 255,
                                               'Alpha' => 35,
                                               'BorderR' => 154, 'BorderG' => 154, 'BorderB' => 154,

                                               ) );
        } else {
            $this->chartImage->drawLegend( 25, 20, array( 
                                               "Mode" => LEGEND_HORIZONTAL,
                                               "Style" => LEGEND_BOX,
                                               'R' => 255, 'G' => 255, 'B' => 255,
                                               'Alpha' => 35,
                                               'BorderR' => 154, 'BorderG' => 154, 'BorderB' => 154,

                                               ) );
        }
        $title = iHRIS_Projection::getChartTitle($chart);
        $TextSettings = array("R"=>255,"G"=>255,"B"=>255,"Align"=>TEXT_ALIGN_TOPMIDDLE , 'FontSize'=>16); 
        $this->chartImage->drawText(375,450,$title,$TextSettings);


        return true;
    }

}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
