<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageViewProjection extends I2CE_Page{ 
    protected $projection;
        
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $this->template->addHeaderLink("mootools-core.js");
        $this->template->addHeaderLink("mootools-more.js");
        $this->template->addHeaderLink("I2CE_AjaxTabPanel.js");
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink("I2CE_SubmitButton.js");
        $this->template->addHeaderLink("I2CE_AjaxSubmitButton.js");
        $factory = I2CE_FormFactory::instance();
        $id = false;
        if ($this->request_exists('id') && $this->request('id')) { 
            $id = $this->get('id');
        } else  if ($this->request_exists('parent') && $this->request('parent')) {
            if (substr($this->request('parent'),0,11) == 'projection|') {
                $id = $this->get('parent');
            }
        }
        if (!$id || ! ($this->projection = $factory->createForm($id )) instanceof iHRIS_Projection) {
            $this->setRedirect(  "home" );
            return false;
        }
        $this->projection->populate();
        $step = iHRIS_RetainBreadCrumbs::show($this,$this->projection);
        if ($this->request_exists('tab')) {
            $tab = $this->request('tab');
        } else {
            if ($step < 1) {
                $step = 1;
            }
            $tabs = array(1=>'basic',2=>'cadre',3=>'intervention',4=>'intervention',5=>'scenarios',6=>'reports');
            $tab = $tabs[$step];
        }        
        if ($step < 6 && count($this->projection->getChildIds('scenario')) == 0) {
            //disable the scenarios tab
            if ( ($node = $this->template->getElementByID('scenarios')) instanceof DOMElement) {
                $node->setAttribute('class', $node->getAttribute('class')  . ' disabled_tab');
            }
        }

        $js = 'document.addEvent("domready", function() {
	     var tabPane = new I2CE_AjaxTabPanel("projection");	     
	     tabPane.showTab("' . $tab . '");
});';
        $this->template->addHeaderText($js,'script','create_tabs');

        $this->projection->fixupFinancials();
        $this->projection->fixupPools();
        //$this->projection->fixupInterventions();
        parent::action();

        $this->template->setForm( $this->projection );

        $this->template->addHeaderLink("view.js");
        $this->template->addHeaderLink("missing.css");
        $this->template->addHeaderLink("missing_iehacks.css", array("ie6" => true) );
        $this->projection->populateChildren('financial');

        $action_chain  = array(
            array('redirect'=>'financial','test'=>'hasFinancial','step'=>1,'form'=>'financial'),
            array('redirect'=>'pools','test'=>'hasPool','step'=>2,'form'=>'pool'),
            array('redirect'=>'intervention','test'=>'hasInterventions','step'=>3),
            array('form'=>'scenario'),
            );
        
        foreach ($action_chain as $action) {
            if (array_key_exists('test',$action)) {
                $test = $action['test'];
                if (!$this->projection->$test()) {
                    if (!$this->request_exists('noredirect') && array_key_exists('redirect',$action)) {
                        $redirect= $action['redirect'] . "?parent=" . $this->projection->getNameId();
                        if (array_key_exists('step',$action)) {
                            $redirect.='&instruct=' . $action['step']  ;                
                        }
                        $this->setRedirect( $redirect);
                    }
                    break;
                }
            }
            if (array_key_exists('form',$action)) {
                if (is_array($action['form'])) {
                    
                } else {
                    $forms = array($action['form']);
                }
                foreach ($forms as $form) {
                    $this->projection->populateChildren($form);
                    if (!array_key_exists($form,$this->projection->children) || !is_array($this->projection->children[$form])) {
                        continue;
                    }
                    foreach ($this->projection->children[$form] as $obj) {
                        if (! ($node = $this->template->appendFileById( "view_" . $form . ".html", "div", $form, false ))  instanceof DOMNode) {
                            I2CE::raiseError("Invalid view_$form.html");
                            break;
                        }
                        $this->template->setForm( $obj, $node );
                    }
                }
            }            
        }
                   
        return true;
    }


  }


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
