<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * The projection form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The projection form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Projection extends I2CE_Form{

    /**********************************************************************************************************************************
     *
     *                                    Access
     *
     **********************************************************************************************************************************/
    protected $checked_access  = array();
    public function canAccess($user) {
        if (!$user instanceof I2CE_User) {
            I2CE::raiseError("Bad user");
            return false;
        }
        $username = $user->username();
        if ($this->getID() == 0  || $username == 'i2ce_admin') {
            return true;
        }
        if (array_key_exists($username,$this->checked_access)) {
            return $this->checked_access[$username];
        }
        $where =array(
            'operator'=>'OR',
            'operand'=>
            array(
                array(
                    'field'=>'creator',
                    'operator'=>'FIELD_LIMIT',
                    'style'=>'equals',
                    'data'=>array(
                        'value'=>'user|' . $username
                        )
                    ),
                array(
                    'field'=>'access',
                    'operator'=>'FIELD_LIMIT',
                    'style'=>'contains',
                    'data'=>array(
                        'value'=>'user|' . $username
                        )
                    )
                )
            ); 
        $this->checked_access[$username] =  ($this->checkWhereClause($where) === true);
        return $this->checked_access[$username];
    }


    public function getStartYear() {
        if (! ($startField = $this->getField('start_year')) instanceof I2CE_FormField_DATE_Y) {
            return false;
        }
        if (! ($date = $startField->getValue()) instanceof I2CE_Date) {
            return false;
        }
        return $date->year();
    }


    public function getDuration() {
        if (!($durField = $this->getField('duration')) instanceof I2CE_FormField_INT_RANGE) {
            return false;
        }
        return $durField->getValue();
    }

    /**********************************************************************************************************************************
     *
     *                                    Scenarios
     *
     **********************************************************************************************************************************/
    public function addScenario($name,$description,$user) {
        if ( $this->getScenario($name) instanceof iHRIS_Scenario) {            
            //already exists, don't add.            
            return;
        }
        $ff = I2CE_FormFactory::instance();
        if (! ($scenario = $ff->createContainer('scenario')) instanceof iHRIS_Scenario) {
            I2CE::raiseError("Could not create scenario: $name");
            return;
        }
        $scenario->getField('name')->setValue($name);
        $scenario->getField('description')->setValue($description);
        $scenario->setParent($this);
        $scenario->save($user);
    }

    public function getPools() {
        $ids =     I2CE_FormStorage::listFields('pool',array('name'),$this->getFormID(), array(),array('name'));
        if (!is_array($ids)) {
            return array();
        }
        foreach ($ids as $i=>&$data) {
            if (!is_array($data) || !array_key_exists('name',$data) || !$data['name']) {
                unset($ids[$i]);
                continue;
            }
            $data = $data['name'];
        }
        unset($data);
        return $ids;
    }


    public function getScenarios() {
        $ids =     I2CE_FormStorage::listFields('scenario',array('name'),$this->getFormID(), array(),array('name'));
        if (!is_array($ids)) {
            return array();
        }
        foreach ($ids as $i=>&$data) {
            if (!is_array($data) || !array_key_exists('name',$data) || !$data['name']) {
                unset($ids[$i]);
                continue;
            }
            $data = $data['name'];
        }
        unset($data);
        return $ids;
    }



    public function getScenario($name = null) {
        //$name  is a name or and id
        $scenario = null;
        $ff = I2CE_FormFactory::instance();
        if (!$name) {
            //try to see if there is a default scenario
            $save = false;
            if ( ($selected = $this->getField('scenario')) instanceof I2CE_FormField_MAP) {
                if (!$selected->isValid()) {
                    I2CE::raiseError("Selected is not set, using last available:" . print_r($selected->getValue(),true));
                    $ids = $this->getScenarios();
                    if (count($ids)  > 0) {
                        end($ids);
                        $selected->setValue(array('scenario',key($ids)));
                        $scenario = $selected->getMappedFormObject();
                    }
                } else {
                    $scenario = $selected->getMappedFormObject();
                }
            }
            
        } else if (strpos($name,'|') >  0) {
            //it is an id
            if (! ($scenario = $ff->createContainer($name)) instanceof iHRIS_Scenario) {
                return false;
            }            
        } else {
            $where =
                array(
                    'field'=>'name',
                    'operator'=>'FIELD_LIMIT',
                    'style'=>'equals',
                    'data'=>array(
                        'value'=>$name
                        )
                    );

            $ids = I2CE_FormStorage::search('scenario',$this->getFormID(),$where);
            if (!is_array($ids) || count($ids) != 1) {
                return false;
            }
            $id = current($ids);
            if ($id == 0 || $id == '|') {
                return false;
            }

            if (! ($scenario = $ff->createContainer('scenario|' . $id)) instanceof iHRIS_Scenario) {
                return false;
            }
        }
        if (!$scenario instanceof iHRIS_Scenario) {
            I2CE::raiseError("No scenario");
            return null;
        }
        $scenario->populate();
        return $scenario;
    }


    protected $current_scenario = null;


    public function getCurrentScenario() {
        return $this->current_scenario;
    }

    public function disableInterventions($which = null) {
        $this->populateInterventions();
        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) || !is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $intObj) {
                if (!$intObj instanceof iHRIS_Intervention) {
                    continue;
                }
                if (is_array($which)) {
                    if (in_array($intObj->getFormID(),$which)) {
                        $intObj->enabled =0;
                    }
                } else {
                    $intObj->enabled =0;
                }
            }
        }
    }
    public function enableInterventions($which = null) {
        $this->populateInterventions();
        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) || !is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $intObj) {
                if (!$intObj instanceof iHRIS_Intervention) {
                    continue;
                }
                if (is_array($which)) {
                    if (in_array($intObj->getFormID(),$which)) {
                        $intObj->enabled =1;
                    }
                } else {
                    $intObj->enabled =1;
                }
            }
        }
    }




    public function setScenario($name = null,$save = false, $user = null) {
        $scenarioObj = false;
        if (!$name) {
            //try to see if there is a default scenario
            $save = false;
            if ( ($selected = $this->getField('scenario')) instanceof I2CE_FormField_MAP) {
                if (!$selected->isValid()) {
                    I2CE::raiseError("Selected is not set, using last available:" . print_r($selected->getValue(),true));
                    $ids = $this->getScenarios();
                    if (count($ids)  > 0) {
                        end($ids);
                        $selected->setValue(array('scenario',key($ids)));
                        $scenarioObj = $selected->getMappedFormObject();
                    }
                } else {
                    $scenarioObj = $selected->getMappedFormObject();
                }
            }
        } else if (!$name instanceof iHRIS_Scenario) {
            $scenarioObj = $this->getScenario($name);
        } else {
            $scenarioObj = $name;
        }
        if (!$scenarioObj instanceof iHRIS_Scenario
            || ! ($interventions = $scenarioObj->getField('interventions')) instanceof I2CE_FormField_MAP_MULTUNION) {
            $this->current_scenario = null;
            return $scenarioObj;
        }
        $this->current_scenario = $scenarioObj;
        $enabled = array();
        foreach ($interventions->getValue() as $intervention) {
            $enabled[] = implode("|", $intervention);
        }
        $this->populateInterventions();
        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) || !is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $intObj) {
                if (!$intObj instanceof iHRIS_Intervention) {
                    continue;
                }
                if (in_array($intObj->getFormID(),$enabled)) {
                    $intObj->enabled =1;
                }else {
                    $intObj->enabled =0;
                }
            }
        }
        if ($save && 
            ( ($selected = $this->getField('scenario')) instanceof I2CE_FormField_MAP)) {
            if (!$user instanceof I2CE_User) {
                $user = new I2CE_User();
            }
            $selected->setFromDB($scenarioObj->getFormID());
            if ($selected->isValid()) {
                $this->save( $user );   
            }
        }
        return $scenarioObj;
    }


    /**********************************************************************************************************************************
     *
     *                                    Interventions
     *
     **********************************************************************************************************************************/
    
    public static $interventions =         array(
            'intervention_A1_develop_recruitment',
            'intervention_A1_edu_support',
            'intervention_A1_management',
            'intervention_A1_other',
            'intervention_A1_recruit',
            'intervention_A1_stipend',
            'intervention_A2_construct',
            'intervention_A2_furnish_dorms',
            'intervention_A2_furnish_health_facility',
            'intervention_A2_furnish_school',
            'intervention_A2_maintenance',
            'intervention_A2_management',
            'intervention_A2_other',
            'intervention_A3_design_plan',
            'intervention_A3_management',
            'intervention_A3_other',
            'intervention_A3_student_support',
            'intervention_A3_training',
            'intervention_A4_develop_curricula',
            'intervention_A4_dissemination',
            'intervention_A4_edu_materials',
            'intervention_A4_other',
            'intervention_A5_new_edu_program',
            'intervention_A5_other',
            'intervention_A5_support_hws',
            'intervention_A5_training_delivery',
            'intervention_B1_design_scope',
            'intervention_B1_other',
            'intervention_B1_prepare_training',
            'intervention_B1_training_delivery',
            'intervention_B1_training_supervisors',
            'intervention_B2_conduct',
            'intervention_B2_consult',
            'intervention_B2_design',
            'intervention_B2_develop',
            'intervention_B2_other',
            'intervention_B2_supervisor',
            'intervention_B2_support',
            'intervention_B3_compliance',
            'intervention_B3_cump',
            'intervention_B3_management',
            'intervention_B3_other',
            'intervention_B3_relocation',
            'intervention_B3_support',
            'intervention_B3_train',
            'intervention_B4_compliance',
            'intervention_B4_design',
            'intervention_B4_management',
            'intervention_B4_other',
            'intervention_B4_tuition',
            'intervention_C1_consultations',
            'intervention_C1_financial_incentives',
            'intervention_C1_other',
            'intervention_D1_electric',
            'intervention_D1_housing',
            'intervention_D1_other',
            'intervention_D1_repair',
            'intervention_D1_school',
            'intervention_D1_telecomm',
            'intervention_D1_water',
            'intervention_D2_medicine',
            'intervention_D2_transport',
            'intervention_D2_other',
            'intervention_D2_repair',
            'intervention_D2_routine',
            'intervention_D2_support',
            'intervention_D2_train',
            'intervention_D2_upgrade',
            'intervention_D3_grants',
            'intervention_D3_management',
            'intervention_D3_other',
            'intervention_D3_outreach_support',
            'intervention_D3_rural',
            'intervention_D3_urban',
            'intervention_D4_conduct',
            'intervention_D4_consult',
            'intervention_D4_design',
            'intervention_D4_other',
            'intervention_D4_training',
            'intervention_D5_conference',
            'intervention_D5_journal',
            'intervention_D5_networking',
            'intervention_D5_other',
            'intervention_D6_conference',
            'intervention_D6_design',
            'intervention_D6_dis_awards',
            'intervention_D6_health_days',
            'intervention_D6_nat_awards',
            'intervention_D6_other'
            );


    public function fixupInterventions() {
        $this->populateInterventions();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        foreach (self::$interventions as $intervention) {
            $key = substr($intervention,13,1);
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                //do the projected costs
                for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                    $costObj = $interventionObj->getField('projected_costs');
                    if (!$costObj instanceof I2CE_FormField_ASSOC_INT) {
                        continue;
                    }
                    if ($costObj->keyIsSet($year)) {
                        continue;
                    }
                    $cost =(int) $interventionObj->getCostByYear($year);
                    $costObj->setValueOfKey($year, $cost);
                }   
            }
        }
    }

    protected $populated = false;
    public function populateInterventions() {
        if ($this->populated) {
            return ;
        }
        $this->populateChildren(self::$interventions);
        $this->populated = true;
    }

    protected $has_intervention = null;
    
    public function hasInterventions() {
        if ($this->has_interevention === null) {
            $this->has_intervention = false;
            foreach (self::$interventions as $intervention) {
                if (count($this->getChildIds($intervention)) > 0) {
                    $this->has_intervention = true;
                    break;
                }
            }
        }
        return $this->has_intervention;
    }



    public function hasPool() {
        return count($this->getChildIds('pool')) > 0;
    }
    public function hasFinancial() {
        return count($this->getChildIds('financial')) > 0;
    }
    protected $costs = array();
    protected $icosts = array();




    public function getTargetedHealthWorkersByPool($pool,$year) {
        $this->populateChildren('pool');
        if (!array_key_exists('pool',$this->children) || count($this->children['pool']) == 0) {
            return 0;
        }
        if (($pos = strpos($pool,'|')) !== false) {
            $id = substr($pool,$pos+1);
        } else {
            $id = $pool;
        }
        if (!array_key_exists($id,$this->children['pool'])) {
            return 0;
        }
        $poolObj = $this->children['pool'][$id];
        if (!$poolObj instanceof iHRIS_Pool) {
            return 0;
        }
        return $poolObj->getProjectedRuralHWs($year,true);
    }



    public function getTargetedHealthWorkers($year,$pool = null) {
        $total = 0;
        $this->populateInterventions();
        $ids = array();
        $id = null;
        if ($pool !== null) {
            if (is_string($pool) && ($pos = strpos($pool,'|')) !== false) {
                $id = substr($pool,$pos+1);
            } else {
                $id = $pool;
            }
        }

        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                if (!$interventionObj instanceof iHRIS_Intervention || !$interventionObj->enabled) {
                    continue;
                }
                $pools = $interventionObj->getTargetedPools();
                if ($id === null) {
                    $ids = array_unique(array_merge($ids,array_keys($pools)));
                } else if (in_array($id,array_keys($pools))) {
                    $ids = array($id);
                    break 2;
                }
            }
        }
        foreach ($ids as $id) {
            $total += max(0,$this->getTargetedHealthWorkersByPool($id,$year));
        }
        return $total;
    }


    public function getTotalCostsByYear($check_enabled = true) {
        if ($check_enabled &&  $this->current_scenario === null) {
            $this->setScenario();
        }
        if ($check_enabled &&  !$this->current_scenario instanceof iHRIS_Scenario) {
            return array();
        }
        $current_scenario = $this->current_scenario->getNameID();
        if ($check_enabled && array_key_exists($current_scenario,$this->costs) && count($this->costs[$current_scenario]) > 0) {
            return $this->costs[$current_scenario];
        }
        $this->populateInterventions();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $total_costs =array();
        $total_costs['total'] = array();
        $total_costs['A'] = array();
        $total_costs['B'] = array();
        $total_costs['C'] = array();
        $total_costs['D'] = array();            
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $total_costs['total'][$year] = 0;
            $total_costs['A'][$year] = 0;
            $total_costs['B'][$year] = 0;
            $total_costs['C'][$year] = 0;
            $total_costs['D'][$year] = 0;
        }
        foreach (self::$interventions as $intervention) {
            $key = substr($intervention,13,1);
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                if (!$interventionObj instanceof iHRIS_Intervention || ($check_enabled &&  !$interventionObj->enabled)) {
                    continue;
                }
                for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                    $costObj = $interventionObj->getField('projected_costs');
                    if (!$costObj instanceof I2CE_FormField_ASSOC_INT) {
                        continue;
                    } 
                    if (!$costObj->keyIsSet($year)) {
                        $cost = $interventionObj->getCostByYear($year);
                        $costObj->setValueOfKey($year,(int) $cost);
                    } else {
                        $cost =$costObj->getValueOfKey($year);
                    }
                    $total_costs['total'][$year] += $cost;
                    $total_costs[$key][$year] += $cost;
                }                        
            }
        }
        if ($check_enabled) {
            $this->costs[$current_scenario] = $total_costs;
        }
        return $total_costs;
    }
    
    public function getIndividualCostsByYear($check_enabled = true, $cadreID = null) {
        if ($check_enabled &&  $this->current_scenario === null) {
            $this->setScenario();
        }
        if ($check_enabled && !$this->current_scenario instanceof iHRIS_Scenario) {
            return array();
        }
        $current_scenario = $this->current_scenario->getNameID();
        if ($check_enabled && array_key_exists($current_scenario . $cadreID,$this->icosts) && count($this->icosts[$current_scenario . $cadreID]) > 0) {
            return $this->icosts[$current_scenario. $cadreID];
        }
        $this->populateInterventions();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $costs =array();
        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                if (!$interventionObj instanceof iHRIS_Intervention || ($check_enabled &&  !$interventionObj->enabled)) {
                    continue;
                }
                $id = $interventionObj->getNameID();
                if ($cadreID ) {
                    $found = false;
                    foreach ($interventionObj->getField('pool')->getValue() as $pool) {
                        if (implode("|",$pool) == $cadreID ) {
                            $found = true; 
                            break;
                        }
                    }
                    if (!$found) {
                        continue;
                    }
                }
                $costs[$id] = array();
                for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                    $costObj = $interventionObj->getField('projected_costs');
                    $cost = 0.0;
                    if ($cadreID) {
                        $cost = $interventionObj->getHWCostComponent($cadreID,$year);                            
                    } else {
                        if ($costObj instanceof I2CE_FormField_ASSOC_INT) {
                            if (!$costObj->keyIsSet($year)) {
                                $cost = $interventionObj->getCostByYear($year);
                                $costObj->setValueOfKey($year,(int) $cost);
                            } else {
                                $cost =$costObj->getValueOfKey($year);
                            }
                        }
                    }
                    $costs[$id][$year] = max(0.0,$cost);
                }                        
                $costs[$id]['total'] = array_sum($costs[$id]);
                $costs[$id]['description'] = $interventionObj->description;
                    
            }
        }
        if ($check_enabled) {
            $this->icosts[$current_scenario. $cadreID] = $costs;
        }
        return $costs;
    }


    /**********************************************************************************************************************************
     *
     *                                    Copying
     *
     **********************************************************************************************************************************/


    /**
     * Copy a projection to a new one with the same parameters appending the given
     * suffix to the projection name.
     * @param mixed $id, integer $id or iHRIS_Projection
     * @param string $suffix
     * @return integer The new projection id of the copy.
     **/
    public static function copy( $id, $user, $name  ) {
        $factory = I2CE_FormFactory::instance();       
        if (is_string($id)) {
            $orig = $factory->createForm(  $id );
        } else {
            $orig = $id;
        }
        if (!$orig instanceof iHRIS_Projection) {
            I2CE::raiseError("Couldn't instantiate");
            return false;
        }
        $orig->populate();
        $copy = $orig->duplicate(true,true,null,$user);
        if (!$copy instanceof iHRIS_Projection) {
            I2CE::raiseError("Couldn't copy");
            return false;
        }
        if (is_string($name) && strlen($name) > 0 && ($nameField = $copy->getField('name')) instanceof I2CE_FormField_STRING_LINE) {
            I2CE::raiseError("Setting name of copy to $name");
            $nameField->setValue($name);
            $copy->save( $user );
        }
        //now we need to fix up the intervetion target ids.
        $copy->populateChildren('pool');
        $new_pool_map = array();
        if (array_key_exists('pool',$copy->children) && is_array($copy->children['pool'])) {
            foreach ($copy->children['pool'] as $id=>$poolObj) {
                $new_pool_map[ $poolObj->getField('name')->getValue()]  = $id;
            }
        }
        $old_pool_map = array();
        if (array_key_exists('pool',$orig->children) && is_array($orig->children['pool'])) {
            foreach ($orig->children['pool'] as $id=>$poolObj) {
                $old_pool_map[$id] = $poolObj->getField('name')->getValue();
            }
        }        
        I2CE::raiseError("Scenarios not copied over!");
        $this->populateInterventions();
        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$copy->children) || !is_array($copy->children[$intervention])) {
                continue;
            }
            foreach ($copy->children[$intervention] as $intObj) {
                if (!$intObj instanceof iHRIS_Intervention) {
                    continue;
                }
                if (! ($targ = $intObj->getField('pool')) instanceof I2CE_FormField_MAP_MULT) {
                    continue;
                }
                $old_vals = $targ->getValue();
                $new_vals = array();
                foreach ($old_vals as  $oldid) {
                    if (!array_key_exists($oldid[1],$old_pool_map)) {
                        continue;
                    }
                    if (!array_key_exists($old_pool_map[$oldid[1]],$new_pool_map)) {
                        continue;
                    }
                    $new_vals[] = array('pool',$new_pool_map[$old_pool_map[$oldid[1]]]);
                }
                $targ->setValue($new_vals);
                $intObj->save($user);
            }

        }
        return $copy;
    }


    public static function getNextAvailableName($name,$name_next) {
        $existing_projs =     I2CE_FormStorage::listFields('projection',array('name'));
        foreach ($existing_projs as $id=>&$existing_proj) {
            if (!is_array($existing_proj) || !array_key_exists('name',$existing_proj)) {
                unset($existing_projs[$id]);
                continue;
            }
            $existing_proj = $existing_proj['name'];
        }
        unset($existing_proj);
        $counter = 0;
        $new_name = $name;
        while (true) {
            if (!in_array($new_name,$existing_projs)) {
                break;
            }
            $counter++;
            $new_name = sprintf($name_next,$counter);
        }
        return $new_name;
    }

    public function fixupPools() {
        $this->populateChildren('pool');
        if (!array_key_exists('pool',$this->children) || !is_array($this->children['pool'])) {
            return;
        }
        foreach ($this->children['pool'] as $i=>$pool) {
            $pool->projectUrban();
            $pool->projectRural();
        }
    }
            



    /**********************************************************************************************************************************
     *
     *                                    Excel
     *
     **********************************************************************************************************************************/

    protected $phpExcelObj;

    public function addExcelHeader($sheet,$header, $row = 1) {
        $sheet->setCellValue('A' . $row,$header);
        if ($row == 1) {
            $style = array(
                'font'=>array(
//                'name' => 'Arial',
                    'size' => 16,
                    'bold' => true
                    )
                );
        } else {
            $style = array(
                'font'=>array(
                    'size' => 14,
                    'bold' => true
                    )
                );

        }
        $sheet->getStyle('A' . $row)->applyFromArray($style);
        if ($row == 1) {
            $sheet->getRowDimension($row)->setRowHeight(18);
        } else {
            $sheet->getRowDimension($row)->setRowHeight(16);
        }
        $col =  PHPExcel_Cell::stringFromColumnIndex($this->duration);
        $sheet->mergeCells("A{$row}:{$col}{$row}");
    }



    public static function getChartTitle($chart) {
        $title = self::$charts[$chart];
        I2CE::getConfig()->setIfIsSet($title,"/translations/chart_titles/$chart");
        return $title;
    }

    public static $charts = array(

        'chart_total'=>'Intervention Costs As A Percentage Of Total Budget',
        'chart_breakdown'=>'Yearly Intervention Breakdown',
        'chart_salary'=>'Intervention Costs As A Percentage Of Personnel Budget',
        'chart_recurrent'=>'Intervention Costs As A Percentage Of Recurrent Expenditures',
        'chart_by_cadre'=>'Annual Costs By Cadre',
        'chart_by_hw'=>'Annual Costs By Per Health Worker By Cadre',
        'compare'=>'Comparison of Annual Total Costs'
        );


    public static $chart_types = array(
        //'chart_breakdown'=>'line',
        'chart_breakdown'=>'stackedbar',
        'chart_salary'=>'bar',
        'chart_total'=>'bar',
        'chart_recurrent'=>'bar',
        'chart_by_cadre'=>'stackedbar',
        'chart_by_hw'=>'stackedbar'
        );

    protected function addChartImage($sheet, $chart,  $row, $col = 0) {
        $wrangler = new I2CE_Wrangler();
        $get = array('id'=>$this->getFormID());
        $post = array();
        
        $chartPage = $wrangler->wrangle("chart/" . self::getChartTitle($chart) . "/" . $chart,false,$get,$post); 
        if (!$chartPage instanceof I2CE_PagePChart) {
            I2CE::raiseError("Could not generate chart image 4chart");
            return $row;
        }
        $tmp_dir = sys_get_temp_dir();
        $count =0;
        do{
            $tmpfname = $tmp_dir . "/RETAIN_EXCEL_CHART_" . mt_rand() . '.png';
            $fp = @fopen($tmpfname, 'x');
            $count++;
            if ($count > 10) {
                I2CE::raiseError("Coulnd not create tmp file in $tmp_dir");
                return $row;
            }
        }while(!$fp);
        fclose($fp);


        $chartPage->display(true);
        $pImage = $chartPage->getImage();
        if (!$pImage instanceof pImage) {
            I2CE::raiseError("No pImage created for chart");
            return $row;
        }
        $pImage->render($tmpfname);
        $height = $pImage->getHeight();

        //the number of rows need is based on the # pixels the image is high / 16 (the default height of a cell)

        $sheet->SetCellValue(PHPExcel_Cell::stringFromColumnIndex( $col)  . $row ,self::getChartTitle($chart));
        $row++;
        $objDrawing = new PHPExcel_Worksheet_Drawing();
        $objDrawing->setName(self::getChartTitle($chart));
        $objDrawing->setDescription(self::getChartTitle($chart));
        $objDrawing->setPath($tmpfname);
        $objDrawing->setCoordinates(PHPExcel_Cell::stringFromColumnIndex( $col ) . $row);
        $objDrawing->getShadow()->setVisible(true);
        $objDrawing->setWorksheet($sheet);
        
        return $row + ((int) ($height / 16)) + 1;
    }



    public function createExcel() {
        try {
            $this->phpExcelObj = new PHPExcel();
            $tot_sheet = $this->phpExcelObj->getActiveSheet();
            $tot_sheet->setTitle("Summary Reports");
            $tot_sheet->getColumnDimension('A')->setAutoSize(true);
            $this->addExcelHeader($tot_sheet,'Summary Reports');
            
            $fin_sheet = $this->phpExcelObj->createSheet();
            $fin_sheet->setTitle(self::$excelFinancialName);
            $fin_sheet->getColumnDimension('A')->setAutoSize(true);
            $this->addExcelHeader($fin_sheet,'Background Financial Information');
            $row = 2;
            $this->addYearlyHeaderToExcel($fin_sheet,$row, $col_offset = 0 ) ;
            $row++;
            $start_year = $this->start_year->year();        
            $duration = $this->duration;
            $this->fixupFinancials();
            $data = array();
            $rates =array();
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $data[$year] = array(
                    'value'=>$this->getInflationRate($year ) / 100.0,
                    'style'=>array('numberformat'=>array('code'=>'0.0%'))
                    );
                $rates[$year] = array(
                    'value'=>$this->getExchangeRate($year,false),
                    'style'=>array('numberformat'=>array('code'=>'$#,##0.00_-'))
                    );

            }
            $this->addYearlyDataArrayToExcel($fin_sheet,$row,"Inflation Rate",$data );
            $row++;
            $this->addYearlyDataArrayToExcel($fin_sheet,$row,"Total Health Sector Budget",    $this->getTotalHealthBudgetsByYear());
            $row++;
            $this->addYearlyDataArrayToExcel($fin_sheet,$row,"Total Salary Budget",    $this->getTotalSalaryBudgetsByYear());
            $row++;
            $this->addYearlyDataArrayToExcel($fin_sheet,$row,"Donor Contributions",    $this->getTotalDonorContributionsByYear());
            $row++;
            $this->addYearlyDataArrayToExcel($fin_sheet,$row,"Recurrent Budget",    $this->getTotalRecurrentBudgetByYear());
            $row++;
            $this->addYearlyDataArrayToExcel($fin_sheet,$row,"Exchange Rate (USD)",    $rates );
            $range = 'A2:' . PHPExcel_Cell::stringFromColumnIndex($duration ) . $row;
            $fin_sheet->getStyle($range)->applyFromArray(self::$excelBlockStyle);
                

            $this->populateInterventions();
            $results = array();
            $row_offsets = array();
            $sheets = array();
            foreach (self::$interventions as $intervention) {
                if (!array_key_exists($intervention,$this->children) || !is_array($this->children[$intervention])) {
                    continue;
                }
                foreach ($this->children[$intervention] as $intObj) {
                    if (!$intObj instanceof iHRIS_Intervention || !$intObj->enabled) {
                        continue;
                    }              
                    $title = substr(trim(preg_replace('/^intervention/i','',preg_replace('/\s+/',' ',str_replace(PHPExcel_Worksheet::getInvalidCharacters(),' ',$intObj->getDisplayName())))),0,30);;
                    $prefix = substr(ltrim(preg_replace('/^intervention_/i','',$intObj->getName())),0,2);
                    if (!array_key_exists($prefix,$row_offsets)) {
                        $row_offsets[$prefix]  = 2;
                        $sheets[$prefix] = $this->phpExcelObj->createSheet();
                        $this->addExcelHeader($sheets[$prefix], $prefix . ' Interventions');
                        $sheets[$prefix]->setTitle($prefix);
                    }
                    $sheet = $sheets[$prefix];
                    $row_offset = $row_offsets[$prefix];
                    $t_results = $intObj->createDataEntryWorksheet($sheet,$row_offset );
                    $t_results['sheet']  = $prefix;
                    $row_offsets[$prefix] = $t_results['row_offset'] + 1;                    
                    $results[] = $t_results;
                }
            
            }

            //now produce the summary reports

        
            $formulas = array(
                'yearly'=>array_fill($start_year,$duration,array()),
                'interventions'=>array(),
                'cadres'=>array()
                );

            $this->populateChildren('pool');
            $pools = array();
            foreach ($this->children['pool'] as $poolObj) {
                $poolid = $poolObj->getID();
                $pools[$poolid] = $poolObj->getField('name')->getValue(); 
            }        
            foreach ($pools as $poolid=>$name) {
                $formulas['cadres'][$poolid]  = array_fill($start_year,$duration,array());
            }
        
            foreach ($results as $results) {
                if (!is_array($results)) {
                    continue;
                }
                $sheet = $results['sheet'];
                if (array_key_exists('yearly',$results) && is_array($results['yearly'])) {
                    for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                        if (!array_key_exists($year,$results['yearly'])) {
                            continue;
                        }
                        $formulas['yearly'][$year][] = "'" .$sheet ."'!" . $results['yearly'][$year];
                        if (!array_key_exists($sheet,$formulas['interventions'])) {
                            $formulas['interventions'][$sheet] = array_fill($start_year,$duration,array());
                        }
                        $formulas['interventions'][$sheet][$year][] = "'" .$sheet ."'!" . $results['yearly'][$year];
                    }
                }
                if (array_key_exists('cadres',$results) && is_array($results['cadres'])) {
                    foreach ($pools as $poolid=>$name) {
                        if (!array_key_exists($poolid,$results['cadres'])) {
                            continue;
                        }
                        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                            if (array_key_exists($year,$results['cadres'][$poolid])) {
                                $formulas['cadres'][$poolid][$year][] = "'" . $sheet ."'!" .  $results['cadres'][$poolid][$year];
                            }
                        }
                    }
                }
            }

            //create total cost worksheet
            $row = 2;
            $this->addExcelHeader($tot_sheet,'Yearly Totals',$row);
            $row++;
            $row_start = $row;
            $this->addYearlyHeaderToExcel($tot_sheet,$row, $col_offset = 0 ) ;
            $row++;
            $total_all = array();
            foreach ($formulas['yearly'] as $year=>$cells) {             
                foreach ($cells as $i=>$cell) {
                    if (strlen(trim($cell)) == 0) {
                        unset($cells[$i]);
                    }
                }
                if (count($cells) > 0) {
                    $total_all[$year] = '=' . implode('+',$cells);
                } else {
                    $total_all[$year] = '';
                }
            }
            $this->addYearlyDataArrayToExcel($tot_sheet,$row,"Cost Of All Interventions",$total_all );
            $row++;
            foreach ($formulas['interventions'] as $sheet=>$data) {
                $total_intervention = array();
                foreach ($data as $year=>$cells) {             
                    if (count($cells) > 0) {
                        $total_intervention[$year] = '=(' . implode('+',$cells) . ")";
                    } else {
                        $total_intervention[$year] = '';
                    }
                }
                $this->addYearlyDataArrayToExcel($tot_sheet,$row,"Cost Of " . $sheet . " Interventions",$total_intervention );
                $row++;
            }
            $range = 'A' . $row_start . ':' . PHPExcel_Cell::stringFromColumnIndex($duration ) . ($row-1);
            $tot_sheet->getStyle($range)->applyFromArray(self::$excelBlockStyle);
            //now repeat for USD.

            $row++;
            $row++;
            $this->addExcelHeader($tot_sheet,'Yearly Totals (USD)',$row);
            $row++;
            $row_start = $row;
            $this->addYearlyHeaderToExcel($tot_sheet,$row, $col_offset = 0 ) ;
            $row++;
            $total_all = array();
            foreach ($formulas['yearly'] as $year=>$cells) {             
                //$year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 2);
                $year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 1);
                $total_all[$year] = '=(' . implode('+',$cells) . ")/'" . self::$excelFinancialName . "'!" . $year_col . '8' ;
            }
            $this->addYearlyDataArrayToExcel($tot_sheet,$row,"Cost Of All Interventions (USD)",$total_all );
            $row++;
            foreach ($formulas['interventions'] as $sheet=>$data) {
                $total_intervention = array();
                foreach ($data as $year=>$cells) {             
                    //$year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 2);
                    $year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 1);
                    if (count($cells) > 0) {
                        $total_intervention[$year] = '=(' . implode('+',$cells) . ")/'" . self::$excelFinancialName . "'!" . $year_col . '8' ;
                    } else {
                        $total_intercention[$year] = '';
                    }
                }
                $this->addYearlyDataArrayToExcel($tot_sheet,$row,"Cost Of " . $sheet . " Interventions (USD)",$total_intervention );
                $row++;
            }
            $range = 'A' . $row_start . ':' . PHPExcel_Cell::stringFromColumnIndex($duration ) . ($row-1);
            $tot_sheet->getStyle($range)->applyFromArray(self::$excelBlockStyle);


            // $row += 2;
            // foreach (array_keys(self::$charts) as $chart_type) {
            //     $row = $this->addChartImage($tot_sheet, $chart_type, $row);
            //     $row++;
            // }



            //create total cadre breakdown sheet
            $cad_sheet = $this->phpExcelObj->createSheet();
            $cad_sheet->setTitle("Costs By Cadre");
            $cad_sheet->SetCellValue('B2','Year');
            $cad_sheet->SetCellValue('A3','Cadre');
            $cadre_row =3;
            foreach ($pools as $poolid=>$name) {
                $cad_sheet->SetCellValue('B' . $cadre_row,$name);
                foreach ($formulas['cadres'][$poolid] as $year=>$cells) {
                    $year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 2);
                    if ($cadre_row == 3) {
                        $cad_sheet->SetCellValue($year_col .'2',$year);
                    }
                    if (count($cells) > 0) {
                        $cad_sheet->SetCellValue($year_col . $cadre_row,'=' . implode('+',$cells));            
                    }
                    $row++;
                }
                $cadre_row++;
            }
            $this->phpExcelObj->setActiveSheetIndexByName('Summary Reports');
        } catch (Exception $e) {
            I2CE::raiseError("Could not save spreadheet to excel: " . $e->getMessage());
            return false;
        }
        return $this->phpExcelObj;
    }


    public function addYearlyHeaderToExcel($sheet,$row_index, $col_offset = 0 ) {
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $data[$year] = $year;
        }
        $this->addYearlyDataArrayToExcel($sheet,$row_index,'Year',$data, $col_offset);        
    }

    public function addStartYearHeaderToExcel($sheet,$row_index, $col_offset = 0 ) {
        $start_year = $this->start_year->year();        
        $data[$start_year] = $start_year;
        $this->addYearlyDataArrayToExcel($sheet,$row_index,'Year',$data, $col_offset);        
    }




    public static $excelBlockStyle =array();
      
    protected function hasExcel() {
        include_once('PHPExcel/PHPExcel.php');        
        return class_exists('PHPExcel',false);
    }

    /**
     * Create a new instance of a I2CE_Form object.
     * @param I2CE_FieldContainer_Factory $factory
     * @param string $name The name of this form
     * @param integer $id
     */
    public function __construct( $factory, $name, $id='0' ) {
        parent::__construct($factory,$name,$id);
        if ($this->hasExcel()) {
            self::$excelBlockStyle =   array(
                'borders' => array(
                    'top' => array( 'style' => PHPExcel_Style_Border::BORDER_THIN),
                    'bottom' => array( 'style' => PHPExcel_Style_Border::BORDER_THIN),
                    'left' => array( 'style' => PHPExcel_Style_Border::BORDER_THIN),
                    'right' => array( 'style' => PHPExcel_Style_Border::BORDER_THIN)
                    ),
                'fill' => array(
                    'type'=>PHPExcel_Style_Fill::FILL_SOLID,
                    'startcolor' => array('argb' => 'FFFFCC99')
                    )
                );
        }
    }


    public function addYearlyDataArrayToExcel($sheet,$row_index,$label,$data , $col_offset = 0 ) {
        $label_col = PHPExcel_Cell::stringFromColumnIndex($col_offset);
        $start_year_col =  PHPExcel_Cell::stringFromColumnIndex($col_offset + 1);
        $sheet->setCellValue($label_col . $row_index,$label);
        if (false !==  strpos($label,"\n")) {
            $sheet->getStyle($label_col . $row_index)->getAlignment()->setWrapText(true);
            $sheet->getRowDimension($row_index)->setRowHeight(30);
        }
        $sheet->getStyle($label_col . $row_index)->getFont()->setBold(true);
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            if (!array_key_exists($year,$data)) {
                continue;
            }
            $style =array();
            if (is_scalar($data[$year])) {
                $value = $data[$year];
            } else if (is_array($data[$year]) && array_key_exists('value',$data[$year])) {
                $value = $data[$year]['value'];
                if (array_key_exists('style',$data[$year]) && is_array($data[$year]['style'])) {
                    $style= $data[$year]['style'];
                }
            }
            $year_col = PHPExcel_Cell::stringFromColumnIndex($year - $start_year + $col_offset + 1);
            $sheet->SetCellValue($year_col . $row_index,$value);
            if (count($style) > 0) {
                $sheet->getStyle($year_col.$row_index)->applyFromArray($style);
            }
        }            
    }


    public function addStartYearDataToExcel($sheet,$row_index,$fieldObj,  $col_offset = 0 ) {
        if (!$fieldObj instanceof I2CE_FormField) {
            return false;
        }
        $label = $fieldObj->getHeader();
        $data = array();
        $year = $this->start_year->year();        
        $start_year_col =  PHPExcel_Cell::stringFromColumnIndex($col_offset + 1);
        $style = array();
        $format = false;       
        $is_percent =false;
        if ($fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
            if (!$fieldObj->keyIsSet($year)) {
                return false;
            }
            $value =  $fieldObj->getValueOfKey($year);
            if ($fieldObj instanceof I2CE_FormField_ASSOC_INT) {
                $format = '#,##0';
            } else if ($fieldObj instanceof I2CE_FormField_ASSOC_PERCENT) {
                $is_percent =true;
                $format = '0.0%';
            } else if ($fieldObj instanceof I2CE_FormField_ASSOC_FLOAT) {
                $format = '#,##0.00';
            }
        } else {
            if ($fieldObj instanceof I2CE_FormField_PERCENT_INT) {
                $is_percent =true;
                $format = '0%';
            } else if ($fieldObj instanceof I2CE_FormField_INT) {
                $format = '#,##0';
            } else if ($fieldObj instanceof I2CE_FormField_PERCENT) {
                $is_percent =true;
                $format = '0.0%';
            } else if ($fieldObj instanceof I2CE_FormField_FLOAT) {
                $format = '#,##0.00';
            }
            $value =   $fieldObj->getValue();
        }                            
        if ($format) {
            $style['numberformat'] = array('code'=>$format);
        }
        if ($is_percent) {
            $value = $value / 100.0;
        }
        $data[$year] = array('style'=>$style,'value'=>$value);
        return $this->addYearlyDataArrayToExcel($sheet,$row_index,$label,$data,$col_offset);
    }


    public function addYearlyDataToExcel($sheet,$row_index,$fieldObj,  $col_offset = 0 ) {
        if (!$fieldObj instanceof I2CE_FormField) {
            return false;
        }
        $label = $fieldObj->getHeader();
        $data = array();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $start_year_col =  PHPExcel_Cell::stringFromColumnIndex($col_offset + 1);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $style = array();
            $format = false;       
            $is_percent =false;
            if ($fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
                if (!$fieldObj->keyIsSet($year)) {
                    continue;
                }
                $value =  $fieldObj->getValueOfKey($year);
                if ($fieldObj instanceof I2CE_FormField_ASSOC_INT) {
                    $format = '#,##0';
                } else if ($fieldObj instanceof I2CE_FormField_ASSOC_PERCENT) {
                    $is_percent =true;
                    $format = '0.0%';
                } else if ($fieldObj instanceof I2CE_FormField_ASSOC_FLOAT) {
                    $format = '#,##0.00';
                }
            } else {
                if ($fieldObj instanceof I2CE_FormField_PERCENT_INT) {
                    $is_percent =true;
                    $format = '0%';
                } else if ($fieldObj instanceof I2CE_FormField_INT) {
                    $format = '#,##0';
                } else if ($fieldObj instanceof I2CE_FormField_PERCENT) {
                    $is_percent =true;
                    $format = '0.0%';
                } else if ($fieldObj instanceof I2CE_FormField_FLOAT) {
                    $format = '#,##0.00';
                }
                if ($year == $start_year || $fieldObj->getOption('no_inflate')) {
                    $value =   $fieldObj->getValue();
                } else {
                    $value =  '=' . $this->inflateCostsExcel($start_year_col . $row_index,$year);
                }
            }                            
            if ($format) {
                $style['numberformat'] = array('code'=>$format);
            }
            if ($is_percent) {
                $value = $value / 100.0;
            }
            $data[$year] = array('style'=>$style,'value'=>$value);
        }
        return $this->addYearlyDataArrayToExcel($sheet,$row_index,$label,$data,$col_offset);
    }


    public static $excelFinancialName = 'Financial';

    public function getExcelFinancialSheet() { 
        if (!$this->phpExcelObj instanceof PHPExcel) {
            return false;
        }
        return $this->phpExcelObj->getSheetByName(self::$excelFinancialName);
    }

    public function inflateCostsExcel($input,$year_end) {
        $start_year = $this->start_year->year();        
        if ($year_end <= $start_year) {
            return $input;
        }        
        $rates = array();
        for ( $year = $start_year; $year < $year_end; $year++) {
            $year_col =  PHPExcel_Cell::stringFromColumnIndex($year - $start_year + 1);
            $rates[] = "(1.0 + '" .  self::$excelFinancialName . "'!{$year_col}3  )";           
        }
        if (count($rates) == 0) {
            return $input;
        }
        return  implode('*',$rates) . '*' . $input;
    }

    



    /**********************************************************************************************************************************
     *
     *                                    Financial Conveinence Methods
     *
     **********************************************************************************************************************************/


    public function fixupFinancials() {
        if ( ($financial = $this->getFinancial())) {
            $financial->fixup();
        }
    }
    public function getTotalHealthBudgetsByYear() {
        if (! ($financial = $this->getFinancial())) {
            return array();
        }
        return $financial->getTotalHealthBudgetsByYear();
    }

    public function getTotalSalaryBudgetsByYear() {
        if (! ($financial = $this->getFinancial())) {
            return array();
        }
        return $financial->getTotalSalaryBudgetsByYear();
    }

    public function getTotalDonorContributionsByYear() {
        if (! ($financial = $this->getFinancial())) {
            return array();
        }
        return $financial->getTotalDonorContributionsByYear();
    }

    public function getTotalRecurrentBudgetByYear() {
        if (! ($financial = $this->getFinancial())) {
            return array();
        }
        return $financial->getTotalRecurrentBudgetByYear();
    }



    public function getCurrencyID() {
        $default = '';
        if (! ($financial = $this->getFinancial())) {
            return $default;
        }        
        return $financial->getField('currency')->getDBValue();
    }

    public function getCurrencyCode() {
        $default = '';
        if (! ($financial = $this->getFinancial())) {
            return $default;
        }        
        return $financial->getCurrencyCode();
    }

    public function convertCurrency($number,$year) { 
        return ((float)$number) * $this->getExchangeRate($year);
    }
    

    protected function getFinancial() {
        $this->populateChildren('financial');
        if (!array_key_exists('financial', $this->children)) {
            return false;
        }
        if ( count($this->children['financial']) != 1) {
            I2CE::raiseError("Invalid number of financial");
            return false;
        }
        reset($this->children['financial']);
        $financial = current($this->children['financial']);
        if (!$financial instanceof iHRIS_Financial) {
            I2CE::raiseError("Invalid financial");
            return false;
        }
        return $financial;
    }

    public function getInflationRate($year) {
        $default = 0.0;
        if (! ($financial = $this->getFinancial())) {
            return $default;
        }        
        return $financial->getInflationRate($year,$default);
    }
    public function getExchangeRate($year,$invert = true) {
        $default = 1.0;
        if (! ($financial = $this->getFinancial())) {
            return $default;
        }      
        $rate = (float) $financial->getExchangeRate($year,$default);  
        if ($rate <= 0) {
            $rate = 1.0;
        }
        if ($invert) {
            return  1.0/$rate;
        } else {
            return  $rate;
        }
    }



    /**********************************************************************************************************************************
     *
     *                                   REPORTING DATA
     *
     **********************************************************************************************************************************/


    public function getCostsByYear() {
        if ( $this->current_scenario === null) {
            $this->setScenario();
        }
        if (!$this->current_scenario instanceof iHRIS_Scenario) {
            return array();
        }
        $current_scenario = $this->current_scenario->getNameID();
        if (array_key_exists($current_scenario,$this->costs) && count($this->costs[$current_scenario]) > 0) {
            return $this->costs[$current_scenario];
        }
        $this->populateInterventions();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $total_costs =array();
        $total_costs['total'] = array();
        $total_costs['A'] = array();
        $total_costs['B'] = array();
        $total_costs['C'] = array();
        $total_costs['D'] = array();            
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $total_costs['total'][$year] = 0;
            $total_costs['A'][$year] = 0;
            $total_costs['B'][$year] = 0;
            $total_costs['C'][$year] = 0;
            $total_costs['D'][$year] = 0;
        }
        foreach (self::$interventions as $intervention) {
            $key = substr($intervention,13,1);
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                if (!$interventionObj instanceof iHRIS_Intervention || !$interventionObj->enabled) {
                    continue;
                }
                for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                    $costObj = $interventionObj->getField('projected_costs');
                    if (!$costObj instanceof I2CE_FormField_ASSOC_INT) {
                        continue;
                    } 
                    if (!$costObj->keyIsSet($year)) {
                        $cost = $interventionObj->getCostByYear($year);
                        $costObj->setValueOfKey($year,(int) $cost);
                    } else {
                        $cost =$costObj->getValueOfKey($year);
                    }
                    $total_costs['total'][$year] += $cost;
                    $total_costs[$key][$year] += $cost;
                }                        
            }
        }
        $this->costs[$current_scenario] = $total_costs;
        return $this->costs[$current_scenario];
    }


    public function get_chart_data_chart_by_cadre() {

        $export_data = array();
        $this->populateInterventions();
        $this->populateChildren('pool');
        $totals = array();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $pool_map = array();
        $pools = array();
        $cadres = array();
        $ff = I2CE_FormFactory::instance();

        $totals = array();
        foreach ($this->children['pool'] as $poolObj) {
            $poolid = $poolObj->getID();
            $pools[$poolid] = $poolObj->getField('name')->getValue(); 
            $totals[$poolid] = array();
        }        
        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                if (!$interventionObj->enabled) {
                    continue;
                }
                foreach ($pools as $poolid=>$poolname) {
                    for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                        if (!array_key_exists($year,$totals[$poolid])) {
                            $totals[$poolid][$year] = 0;
                        }
                        $totals[$poolid][$year] += $interventionObj->getHWCostComponent($poolid,$year);
                    }
                }
            }
        }
        $row_data = array();
        $row_data[] = ''; //first cell is blank
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $year;
        }
        $export_data[] = $row_data ;

        foreach ($pools as $poolid=>$poolname) {
            $row_data = array();
            $row_data[] = $poolname;
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $row_data[] = $totals[$poolid][$year];
            }
            $export_data[] = $row_data ;
        }
        return $export_data;
    }





    public function get_chart_data_chart_by_hw() {
        $export_data = array();
        $this->populateInterventions();
        $this->populateChildren('pool');
        $totals = array();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $pool_map = array();
        $pools = array();
        $cadres = array();
        $cadre_tots = array();
        $ff = I2CE_FormFactory::instance();
        foreach ($this->children['pool'] as $poolObj) {
            $c_id = $poolObj->getID();
            $pools[$c_id] = $poolObj->getField('name')->getValue(); 
            if (!array_key_exists($c_id,$cadre_tots)) {
                $cadre_tots[$c_id] = array();
            }
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                if (!array_key_exists($year,$cadre_tots[$c_id])) {
                    $cadre_tots[$c_id][$year] = 0;
                }
                $cadre_tots[$c_id][$year] += $poolObj->getProjectedRuralHWs($year);
            }
        }        
        // i1 has c1 and c2 with n1.1 and n1.2  it has a total cost of t1 for each of y1..y5
        // i2 has c1 and c2 with n2.1 and n2.2  it has a total cost of t2
        // i3 has c1 and c2 with n3.1 and n3.2  it has a total cost of t3

        // getHWCostComponent(c1,y1) = hwratio(c1,y1)*cost(y1)
        // hwratio(c1,y1) = targeted(c1,y1) / (targeted(c1,y1) + targeted(c2,y1) )
        // by default, targeted(c1,y1) = n1
                                                                                    
            

        foreach (self::$interventions as $intervention) {
            if (!array_key_exists($intervention,$this->children) ||!is_array($this->children[$intervention])) {
                continue;
            }
            foreach ($this->children[$intervention] as $interventionObj) {
                if (!$interventionObj->enabled) {
                    continue;
                }
                foreach ($pools as $poolid=>$poolname) {
                    if (!array_key_exists($poolid,$totals)) {
                        $totals[$poolid] = array();
                    }
                    for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                        if (!array_key_exists($year,$totals[$poolid])) {
                            $totals[$poolid][$year] = 0;
                        }
                        $totals[$poolid][$year] += $interventionObj->getHWCostComponent($poolid,$year);
                    }
                }
            }
        }

        $row_data = array();
        $row_data[] = ''; //first cell is blank
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $year;
        }
        $export_data[] = $row_data ;

        foreach ($pools as $c_id=>$name) {
            $row_data = array();
            $row_data[] =$name;
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                if ($cadre_tots[$c_id][$year] == 0) {
                    $num = 0;
                } else {
                    $num =          $totals[$c_id][$year] / $cadre_tots[$c_id][$year] ;
                }
                $row_data[] = $num;
            }
            $export_data[] = $row_data ;
        }
        return $export_data;
        
    }

   

    public function get_chart_data_chart_breakdown() {
        $export_data = array();
        $total_costs = $this->getTotalCostsByYear();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $totals = array ('total'=>0,'A'=>0,'B'=>0,'C'=>0,'D'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $totals['total'] += $total_costs['total'][$year];
            $totals['A'] += $total_costs['A'][$year];
            $totals['B'] += $total_costs['B'][$year];
            $totals['C'] += $total_costs['C'][$year];
            $totals['D'] += $total_costs['D'][$year];
        }
        $total_bud = $this->getTotalHealthBudgetsByYear();
        $total_sal = $this->getTotalSalaryBudgetsByYear();
        $row_data = array();
        $row_data[] = ''; //first cell is blank
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
           $row_data[] = $year;
        }
        $export_data[] = $row_data ;


        foreach (array('A','B','C','D') as $type) {
            $row_data = array();
            $row_data[] = $type;

            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $row_data[] = $total_costs[$type][$year];
            }
            $export_data[] = $row_data ;
        }

        return $export_data;
    }


    public function get_chart_data_chart_total() {
        $export_data = array();
        $total_costs = $this->getTotalCostsByYear();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $totals = array ('total'=>0,'A'=>0,'B'=>0,'C'=>0,'D'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $totals['total'] += $total_costs['total'][$year];
            $totals['A'] += $total_costs['A'][$year];
            $totals['B'] += $total_costs['B'][$year];
            $totals['C'] += $total_costs['C'][$year];
            $totals['D'] += $total_costs['D'][$year];
        }
        $total_bud = $this->getTotalHealthBudgetsByYear();

        $row_data = array();
        $row_data[] = ''; //first cell is blank
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $year;
        }
        $export_data[] = $row_data ;

        $row_data = array();
        $row_data[] = 'Total Budget';  //needs to eb translated! 
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] =$total_bud[$year];
        }
        $export_data[] = $row_data ;

        $row_data = array();
        $row_data[] = 'Cost Of Interventions';
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $total_costs['total'][$year];
        }
        $export_data[] = $row_data ;
        return $export_data;
        
    }


    public function get_chart_data_chart_recurrent() {
        $export_data = array();
        $total_costs = $this->getTotalCostsByYear();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $totals = array ('total'=>0,'A'=>0,'B'=>0,'C'=>0,'D'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $totals['total'] += $total_costs['total'][$year];
            $totals['A'] += $total_costs['A'][$year];
            $totals['B'] += $total_costs['B'][$year];
            $totals['C'] += $total_costs['C'][$year];
            $totals['D'] += $total_costs['D'][$year];
        }
        $total_bud = $this->getTotalRecurrentBudgetByYear();
        $row_data = array();
        $row_data[] = ''; //first cell is blank
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $year;
        }
        $export_data[] = $row_data ;
        $row_data = array();
        $row_data[] = 'Total Recurrent Expenditures';  //needs to be translated
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $total_bud[$year];
        }
        $export_data[] = $row_data ;
        $row_data = array();
        $row_data[] = 'Cost Of Interventions';
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] =$total_costs['total'][$year];
        }
        $export_data[] = $row_data ;
        return $export_data ;
    }





    public function get_chart_data_chart_salary() {
        $export_data = array();
        $total_costs = $this->getTotalCostsByYear();
        $start_year = $this->start_year->year();        
        $duration = $this->duration;
        $totals = array ('total'=>0,'A'=>0,'B'=>0,'C'=>0,'D'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $totals['total'] += $total_costs['total'][$year];
            $totals['A'] += $total_costs['A'][$year];
            $totals['B'] += $total_costs['B'][$year];
            $totals['C'] += $total_costs['C'][$year];
            $totals['D'] += $total_costs['D'][$year];
        }
        $total_sal = $this->getTotalSalaryBudgetsByYear();
        $row_data = array();
        $row_data[] = ''; //first cell is blank
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $year;
        }
        $export_data[] = $row_data ;
        $row_data = array();
        $row_data[] = 'Total Salary';
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $total_sal[$year];
        }
        $export_data[] = $row_data ;
        $row_data = array();
        $row_data[] = 'Cost Of Interventions';
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $row_data[] = $total_costs['total'][$year];
        }
        $export_data[] = $row_data ;
        return $export_data;
    }

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
