<?php
/**
* © Copyright 2011 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package retain
* @subpackage retain
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v1.2
* @since v1.2
* @filesource 
*/ 
/** 
* Class iHRIS_Scenario
* 
* @access public
*/


class iHRIS_Scenario extends I2CE_SimpleList {
    

    

    protected $set_limits = false;
    /**
     * Return an array of limit_add options to add
     * for this user to limit fields.
     * @param I2CE_Form $formObj
     * @param array $args
     * @return array
     */
    public function setInterventionLimits() {
        if ($this->set_limits) {
            return;
        }
        if (! ($interventionsField = $this->getField('interventions')) instanceof I2CE_FormField_MAP_MULT) {
            I2CE::raiseError("could not get interventions field");
            return array();
        }
        $forms =  $interventionsField->getSelectableForms();
        $where =
            array(
                'field'=>'parent',
                'operator'=>'FIELD_LIMIT',
                'style'=>'equals',
                'data'=>array(
                    'value'=>$this->getParent()
                    )
            );
        //
        $limit_add = array();
        foreach ($forms as $form) {
            $limit_add[$form] = $where;
        }
        $interventionsField->setOption(array('meta','enable_limits_add','retain'),1);
        $interventionsField->setOption(array('meta','limits_add','retain'), $limit_add);
        $this->set_limits = true;
    }


    public function populate($repopulate = false) {
        $mf = I2CE_ModuleFactory::instance();
        $fs = $mf->getClass("forms-storage");
        if (!$fs instanceof I2CE_FormStorage) {
            return false;
        }
        $fs->populate($this,$repopulate);
        $this->setInterventionLimits();
    }


    public function setParent( $parent, $set_id = false ) {
        parent::setParent($parent,$set_id);
        $this->setInterventionLimits();
    }


    public function toggleIntervention($intervention,$user) {
        if (!$intervention instanceof iHRIS_Intervention) {
            I2CE::raiseError("Trying to toggle non-intervention");
            return;
        }
        $form = $intervention->getName();
        $id = $intervention->getID();
        if (! ($interventionsField = $this->getField('interventions')) instanceof I2CE_FormField_MAP_MULT) {
            I2CE::raiseError("could not get interventions field");
            return;
        }
        $interventions = $interventionsField->getValue();
        $found_idx = false;
        foreach ($interventions  as $idx => $data) {
            if (!is_array($data) || count($data) != 2 || $data[0] !== $form || $data[1] !== $id) {
                continue;
            }            
            $found_idx = $idx;
            break;
        }
        if ($found_idx === false) {
            $interventions[] = array($form,$id);
        } else {
            unset($interventions[$found_idx]);
        }
        $interventionsField->setValue($interventions);
        $this->save($user);
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
