<?php
/**
* © Copyright 2012 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris-retain
* @subpackage ihris-retain
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v1.2.0.91
* @since v1.2.0.91
* @filesource 
*/ 
/** 
* Class iHRIS_Tab_Base
* 
* @access public
*/


class iHRIS_Tab_Base extends I2CE_Page{



    protected function addDataTable($name,$headers,$data,$node = null) {
        if ( ($hNode = $this->template->getElementById('report_name',$node)) instanceof DOMNode) {
            $hNode->setAttribute('colspan',count($headers));
        }
        $this->template->setDisplayDataImmediate('report_name',$name);
        if ( ($lNode = $this->template->getElementById('labels',$node)) instanceof DOMNode) {
            foreach ($headers as $key=>$label) {
                $html = "<td class='costout_label'><b>$label</b></td>";
                if (($ltNode = $this->template->importText($html,'td')) instanceof DOMNode) {
                    $lNode->appendChild($ltNode);
                }
            }
        }
        if ( ($dNode = $this->template->getElementById('data',$node)) instanceof DOMNode) {
            foreach ($data as $values) {
                if ($values === true) {
                    $html = '<tr><td style="min-height:1em" colspan="' . count($headers) . '"/></tr>';
                } else {
                    $html = '<tr>';
                    foreach ($headers as $key=>$label) {
                        $val = '';
                        if (array_key_exists($key,$values)) {
                            $val = $values[$key];
                        }
                        $html .= "<td>$val</td>";
                    }
                    $html.='</tr>';                
                }
                if (($dtNode = $this->template->importText($html,'tr')) instanceof DOMNode) {
                    $dNode->appendChild($dtNode);
                }
            }
        }

        if ($this->isExport()) {
            $export_data = array();
            if ( ($rows = $this->template->query('//div[contains(@class,"dataTable")]//table//tr',$node)) instanceof DOMNodeList) {
                foreach ($rows as $row) {
                    $row_data = array();                    
                    if ( ($cells = $this->template->query('./td | ./th',$row)) instanceof DOMNodeList) {
                        foreach ($cells as $cell) {
                            $row_data[] = $cell->textContent;
                        }
                    }
                    $export_data[] = $row_data ;
                }
            }
            I2CE::raiseError(print_r($export_data,true));
            $this->exportData($export_data);
        }

        return true;
    }

    /**
     * Display the template as HTML/XML.  Sets the header and displays any buffered warnings/echoed text.
     */
    protected function _display($supress_output) {        
        if ($this->isExport()) {
            $this->template->clearHeaders();
        }
        parent::_display($supress_output);
    }


    public function display($supress_output = false) {
        if ( $this->isExport()) {
            $errors = '';
            while  (ob_get_level() > 0) {
                $errors .= ob_get_contents();
                ob_end_clean();
            }
            if ($errors) {
                I2CE::raiseError("Got errors:\n$errors");
            }
            //$chart = $this->request('chart');
            header("Content-disposition: attachment; filename=\"retain_export.xls\"");
            // if (preg_match('/\s+MSIE\s+\d\.\d;/',$_SERVER['HTTP_USER_AGENT'])) {
                header("Content-type: application/vnd.ms-excel");
            // } else{
            //     header("Content-type: text/csv; charset=UTF-8");  
            // }
            flush(); 
        }
        parent::display($supress_output);
    }


    protected function exportData($export_data) {
        try {
            I2CE::raiseError("Trying excel");
            $phpExcelObj = new PHPExcel();
            $tot_sheet = $phpExcelObj->getActiveSheet();
            $tot_sheet->setTitle("Summary Reports");
            $row_index = 0;
            foreach ($export_data as $row) {
                $row_index++;
                $col = 0;
                foreach ($row as $cell) {
                    $col++;
                    $col_name = PHPExcel_Cell::stringFromColumnIndex($col);
                    $tot_sheet->SetCellValue($col_name . $row_index,$cell);
                }
            }
            $objWriter = new PHPExcel_Writer_Excel5($phpExcelObj);
            $objWriter->setPreCalculateFormulas(false);
            $tmpfname = tempnam(sys_get_temp_dir(), 'RETAIN_EXCEL_'); 
            $objWriter->save($tmpfname);
            $errors = '';
            while  (ob_get_level() > 0) {
                $errors .= ob_get_contents();
                ob_end_clean();
            }
            $errors = trim($errors);
            if ($errors) {
                I2CE::raiseError("Got errors:\n$errors");
            }
            readfile($tmpfname);
            exit;
        } catch (Exception $e) {
            I2CE::raiseError("Could not save spreadheet to excel: " . $e->getMessage());
        }
        I2CE::raiseError("CSV");
        //fallback to csv
        foreach ($export_data as &$row) {
            foreach ($row as &$cell) {
                $cell = '"' . str_replace('"','""',trim(str_replace("\n"," ",$cell)))  . '"';
            }
            unset($cell);
            $row = implode(",", $row);
        }
        unset($row);
        echo implode("\n",$export_data)  . "\n";
        flush();
        exit;
    }


    protected function isExport() {
        return ($this->request_exists('submit_type') && $this->request('submit_type') == 'export') || ($this->request_exists('export')  && $this->request('export'));
    }


    protected $useUSD = true;

    protected function convertCurrency($number,$year =null) {
        if ($year !== null && $this->useUSD) {
            $number = $this->projection->convertCurrency($number,$year);
        } 
        return $number;
    }


    protected function convertedCurrencyFormat( $number, $year = null, $decimals = 0 ,  $dec_point = '.' ,  $thousands_sep = ',' ) {
        if ($year !== null) {
            $number = $this->projection->convertCurrency($number,$year);
        }  
        $number = (float) $this->convertCurrency($number,$year);
        $prefix = $this->projection->getCurrencyCode();
        if (!is_numeric($number)) {
            $number = 0;
        } else {
            $number = (float) $number;
        }
        if ($this->isExport()) {
            return number_format($number,$decimals,$dec_point,$thousands_sep);
        } else {
            return $prefix .  number_format($number,$decimals,$dec_point,$thousands_sep);
        }

        return $this->currencyFormat($number,$decimals,$dec_point,$thousands_sep);
    }


    protected function currencyFormat( $number, $year = null, $decimals = 0 ,  $dec_point = '.' ,  $thousands_sep = ',' ) {
        if ( $this->useUSD) {
            $prefix = '$';
            $number = $this->convertCurrency($number,$year);
        } else {
            $prefix = $this->projection->getCurrencyCode();
        }
        if (!is_numeric($number)) {
            $number = 0;
        } else {
            $number = (float) $number;
        }
        if ($this->isExport()) {
            return number_format($number,$decimals,$dec_point,$thousands_sep);
        } else {
            return $prefix .  number_format($number,$decimals,$dec_point,$thousands_sep);
        }
    }





}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
