<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Tab_FileManagement extends I2CE_Page{ 
    protected $projection;
        

    public function display($supress_output = false) {
        if ( $this->isExcel()) {
            $errors = '';
            while  (ob_get_level() > 0) {
                $errors .= ob_get_contents();
                ob_end_clean();
            }
            if ($errors) {
                I2CE::raiseError("Got errors:\n$errors");
            }
        }
        parent::display($supress_output);
    }


    protected function isExcel() {
        return ($this->request_exists('submit_type') && $this->request('submit_type') == 'excel' && $this->hasExcel());
    }

    protected function hasExcel() {
        include_once('PHPExcel/PHPExcel.php');        
        return class_exists('PHPExcel',false);
    }

    /**
     * Display the template as HTML/XML.  Sets the header and displays any buffered warnings/echoed text.
     */
    protected function _display($supress_output) {        
        if ($this->isExcel()) {
            $this->template->clearHeaders();
        }
        parent::_display($supress_output);
    }


    protected function actionExcel() {
        if (! ($phpExcelObj = $this->projection->createExcel()) instanceof PHPExcel) {
            return false;
        }
        $this->projection->fixupFinancials();
        $this->projection->fixupPools();
        $this->projection->fixupInterventions();

        try {
            $objWriter = new PHPExcel_Writer_Excel5($phpExcelObj);
            $objWriter->setPreCalculateFormulas(false);
            $tmpfname = tempnam(sys_get_temp_dir(), 'RETAIN_EXCEL_'); 
            $objWriter->save($tmpfname);
        } catch (Exception $e) {
            I2CE::raiseError("Could not save spreadheet to excel: " . $e->getMessage());
        }        
        $filename = 'iHRIS Retain Full Export.xls';
        $errors = '';
        while  (ob_get_level() > 0) {
            $errors .= ob_get_contents();
            ob_end_clean();
        }
        if ($errors) {
            I2CE::raiseError("Got errors:\n$errors");
        }
        header("Content-disposition: attachment; filename=\"" . addslashes($filename) . "\"");
        header("Content-type: application/vnd.ms-excel");
        readfile($tmpfname);
        exit;
    }

    

    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $this->template->addHeaderLink("mootools-core.js");
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink("I2CE_SubmitButton.js");
        $factory = I2CE_FormFactory::instance();
        $id = false;
        if ($this->request_exists('id') && $this->request('id')) { 
            $id = $this->get('id');
        } else  if ($this->request_exists('parent') && $this->request('parent')) {
            if (substr($this->request('parent'),0,11) == 'projection|') {
                $id = $this->get('parent');
            }
        }
        if (!$id || ! ($this->projection = $factory->createForm($id )) instanceof iHRIS_Projection) {
            $this->setRedirect(  "home" );
            return false;
        }
        $this->projection->populate();

        if ( $this->request_exists('copy') && $this->request('copy') == 1 && $this->request_exists('id') ) {
            if ($this->request_exists('copy_name') && strlen($this->request('copy_name')) > 0) {
                $name = $this->request('copy_name');
                $name = iHRIS_Projection::getNextAvailableName($name . ' (Copy)', $this->projection->name . " (Copy %d)");  
            } else {
                $name = iHRIS_Projection::getNextAvailableName($this->projection->name . ' (Copy)', $this->projection->name . " (Copy %d)");  
            }
            $copy = iHRIS_Projection::copy( $this->projection, $this->getUser() ,  $name);
            if (! $copy instanceof iHRIS_Projection) {
                return false;
            }
            if ($this->request_exists('copy_description') && $this->request('copy_description')) {
                $copy->description = $this->request('copy_description');
                $copy->save( $this->getUser() );
            }
            $this->redirect( "view?id=" . $copy->getNameID() );
            return true;
        }else if ($this->isExcel()) {
            return $this->actionExcel();
        }
        parent::action();
        $this->template->setDisplayData('id', $this->projection->getNameID());
        $this->template->setDisplayDataImmediate('user_access',"projection?id=" . $this->projection->getNameID());
        $this->template->setDisplayDataImmediate('hasExcel',$this->hasExcel() ? 1: 0);
        $this->template->setForm( $this->projection );
        return true;
    }
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
