<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */

class iHRIS_Tab_Intervention extends iHRIS_Tab_Base{
    protected $projection;

    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $this->template->addHeaderLink("mootools-core.js");
        $factory = I2CE_FormFactory::instance();
        $id = false;
        if ($this->request_exists('id') && $this->request('id')) { 
            $id = $this->get('id');
        } else  if ($this->request_exists('parent') && $this->request('parent')) {
            if (substr($this->request('parent'),0,11) == 'projection|') {
                $id = $this->get('parent');
            }
        }
        if (!$id || ! ($this->projection = $factory->createForm($id )) instanceof iHRIS_Projection) {
            I2CE::raiseError("Bad ID");
            $this->setRedirect(  "home" );
            return false;
        }
        $this->projection->populate();
        $this->projection->fixupInterventions();

        $this->template->setForm( $this->projection );
        $this->template->setDisplayData('id', $this->projection->getNameID());        
        
        if ( ($scenarioObj = $this->projection->setScenario(null,false))  instanceof iHRIS_Scenario) {
            $this->template->setForm($scenarioObj);      
        }

        $this->template->setForm( $this->projection );

        $this->template->setDisplayData('id', $this->projection->getNameID());        
        $this->projection->populateInterventions();
        foreach (iHRIS_Projection::$interventions as $form) {
            if (!array_key_exists($form,$this->projection->children) || !is_array($this->projection->children[$form])) {
                continue;
            }
            foreach ($this->projection->children[$form] as $obj) {
                if (! ($node = $this->template->appendFileById( "view_" . $form . ".html", "div", $form, false ))  instanceof DOMNode) {
                    I2CE::raiseError("Invalid view_$form.html");
                    break;
                }
                $this->template->setForm( $obj, $node );
            }
        }
        return true;
    }



}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
