<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Tab_Reports extends iHRIS_Tab_Base {
    protected $projection;
        
    protected $action = 'default';
    public  function __construct( $args,$request_remainder , $get = null, $post = null) {
        parent::__construct($args,$request_remainder,$get,$post);
        if (is_array($this->request_remainder) && count($this->request_remainder) > 0) {
            reset($this->request_remainder);
            $this->action = current($this->request_remainder);
        }
        
    }


    /**
     * Load the  template (HTML or XML) files to the template object.
     *  
     * 
     */  
    protected function loadHTMLTemplates() {
        if ($this->defaultHTMLFile === false) {
            return true;
        }
        switch($this->action) {
        case 'summary':
        case 'cadre':
            $this->template->addFile( 'reports_base.html');
            break;
        default:
            if (is_array($this->defaultHTMLFile)) {
                $files = $this->defaultHTMLFile;
            } else {
                $files = array($this->defaultHTMLFile);
            }
            foreach ($files as $file) {
                if ( ! ($file === null || (is_string($file) && strlen($file) == 0))) {
                    $this->template->addFile( $file);
                }
            }
            break;
        }
        return true;
    }


    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $this->template->addHeaderLink("mootools-core.js");
        $this->template->addHeaderLink("mootools-more.js");
        $this->template->addHeaderLink("I2CE_AjaxTabPanel.js");
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink("I2CE_SubmitButton.js");
        $this->template->addHeaderLink("I2CE_AjaxSubmitButton.js");
        $factory = I2CE_FormFactory::instance();
        $id = false;
        if ($this->request_exists('pool') && $pool = $this->request('pool')) {
            I2CE::raiseError("Doing pool $pool");
            if (! ($poolObj = $factory->createContainer($pool)) instanceof iHRIS_Pool) {
                I2CE::raiseError("Bad pool:$pool");
                return false;
            }
            $poolObj->populate();
            $id = $poolObj->getParent();
            I2CE::raiseError("Got pool with parent $id");
        } else if ($this->request_exists('id') && $this->request('id')) { 
            $id = $this->get('id');
        } else  if ($this->request_exists('parent') && $this->request('parent')) {
            if (substr($this->request('parent'),0,11) == 'projection|') {
                $id = $this->get('parent');
            }
        }
        if (!$id || ! ($this->projection = $factory->createForm($id )) instanceof iHRIS_Projection) {
            I2CE::raiseError("bad id for $id");
            $this->setRedirect(  "home" );
            return false;
        }
        $this->projection->populate();
        $this->template->setForm( $this->projection );
        $this->template->setDisplayData('id', $this->projection->getNameID());


        if ( $this->isExport() && $this->request_exists('chart') && substr($this->request('chart'),0,6) == 'chart_') {
            $this->chart_data();
        }
        
        $report = $this->request('submit_type');
        switch($this->action) {
        case 'summary': //summary reports

            $this->projection->fixupFinancials();
            $this->projection->fixupPools();
            $this->projection->fixupInterventions();
            if ( ($scenarioObj = $this->projection->setScenario(null,false)) instanceof iHRIS_Scenario) {
                $this->template->setForm($scenarioObj);      
            }

            if (!$report) {
                return false;
            }
            parent::action();            
            switch ($report) {
            case 'summary_USD':
                return $this->costOutSummary();
            case 'summary_local':
                $this->useUSD =false;
                return $this->costOutSummary();
                break;
            default:
                if (substr($report,0,7) == 'summary') {
                    I2CE::raiseError("Making table $report");
                    return $this->costOutTable($report);
                }else  if (substr($report,0,5) == 'chart') {
                    I2CE::raiseError("Making chart $report");
                    return $this->makeChart($report);
                } else {
                    I2CE::raiseError("No report");
                    return false;
                }
            }
            break;
        case 'cadre': //cadre specific reports
            $this->projection->fixupFinancials();
            $this->projection->fixupPools();
            $this->projection->fixupInterventions();
            if ( ($scenarioObj = $this->projection->setScenario(null,false)) instanceof iHRIS_Scenario) {
                $this->template->setForm($scenarioObj);      
            }

            I2CE::raiseError("CS reports");
            if (!$report) {
                I2CE::raiseError("No report");
                return false;
            }
            parent::action();            
            //make sure we get the selected cadre.
            if (! ($cadreSelect = $factory->createContainer('cadreselection')) instanceof iHRIS_CadreSelection) {
                I2CE::raiseError("No selection");
                return false;
            }
            if ($this->request_exists('pool') && $pool = $this->request('pool')) {
                
            } else {
                if (! ($cadreField = $cadreSelect->getField('pool')) instanceof I2CE_FormField_MAP) {
                    I2CE::raiseError("pool is not a map");
                    return false;
                }
                $cadreSelect->load($this->request());
                $pool = $cadreField->getDBValue();
            }
            $cadreSelect->setParent($this->projection,false);
            $this->template->setForm($cadreSelect);                
            switch ($report) {
            case 'cost_all_incentives':
                return $this->costOutAllIncentivesByCadre($pool);
            case 'summary_scenarios':
                return $this->costOutScenariosByCadre($pool);
            case 'hw_cost':
                return $this->costOutScenariosHWCost($pool);
            }
            break;
        default:              
            $this->projection->fixupFinancials();
            $this->projection->fixupPools();


            if ( ($cadreSelect = I2CE_FormFactory::instance()->createContainer('cadreselection')) instanceof iHRIS_CadreSelection) {
                $cadreSelect->setParent($this->projection,false);
                $this->template->setForm($cadreSelect);                
            }
            $pools = $this->projection->getPools();
            $scenarios = $this->projection->getScenarios();
            $this->template->setDisplayDataImmediate('has_more_than_one_package',  count($scenarios) > 1 ?  1 : 0);
            $this->template->setDisplayDataImmediate('pool',  $pools);
            parent::action();
            break;
        }
        return true;
    }


    protected function costOutScenariosHWCost($cadreForm) {
        if (!($containerNode = $this->template->appendFileById( "costout_table_shwc.html", "div", "report_display" )) instanceof DOMNode) {
            return false;
        }
        $currentScenario  = $this->projection->getCurrentScenario();

        $headers['description'] = 'Description';
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;
        $headers['total'] = 'Total Cost';
        $headers['per_hw'] = 'Cost Per Health Worker Per Year';

        $all_costs = array();
        $scenarios = $this->projection->getScenarios();
        foreach ($scenarios as $scenario=>$name) {
            if (! ($scenarioObj = $this->projection->getScenario('scenario|' . $scenario)) instanceof iHRIS_Scenario) {
                continue;
            }
            $this->projection->setScenario('scenario|' . $scenario);
            $costs = $this->projection->getIndividualCostsByYear(true,$cadreForm);
            if (count($costs) == 0) {
                continue;
            }
            $total = 0;            
            foreach ($costs as $data) {
                if (!array_key_exists('total',$data)) {
                    continue;
                }
                $total += max(0,$data['total']);
            }
            $hws = array();
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                //note that this _does_ depend on the set scenario.
                if ($cadreForm) {
                    $hw = (float) $this->projection->getTargetedHealthWorkers($year,$cadreForm);
                } else {
                    $hw = (float) $this->projection->getTargetedHealthWorkers($year);
                }
                $hws[$year] = max(0,$hw);
            }
            $hws = (int) floor(array_sum($hws)/$duration);
            if ($hws > 0) {
                $per_hws = $total / ($hws * $duration);
            } else {
                $per_hws =0.0;
            }
            $all_costs[] = array(
                'description'=>"<b>" . $scenarioObj->getField('name')->getValue() . "</b>", 
                'total'=>$this->currencyFormat($total,$start_year),
                'per_hw'=>$this->currencyFormat($per_hws,$start_year)
                );

        }
        if ($currentScenario instanceof iHRIS_Scenario) {
            $this->projection->setScenario($currentScenario);
        }
        $cadreID = substr($cadreForm,5);
        if ($cadreID) {
            $name = 'Retention Packages and Preference Impact ' . I2CE_List::lookup($cadreID,'pool');
        } else {
            $name = 'Retention Packages and Preference Impact';
        }
        $this->addDataTable($name,$headers,$all_costs,$containerNode);
        return true;
    }


    protected function costOutScenariosByCadre($cadreForm) {
        if (!($containerNode = $this->template->appendFileById( "costout_table_sc.html", "div", "report_display" )) instanceof DOMNode) {
            return false;
        }
        $currentScenario  = $this->projection->getCurrentScenario();

        $headers['description'] = 'Description';
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $headers[$year] = $year;
        }
        $headers['total'] = 'Total';

        $all_costs = array();
        $scenarios = $this->projection->getScenarios();
        foreach ($scenarios as $scenario=>$name) {
            if (! ($scenarioObj = $this->projection->getScenario('scenario|' . $scenario)) instanceof iHRIS_Scenario) {
                continue;
            }
            $this->projection->setScenario('scenario|' . $scenario);
            $costs = $this->projection->getIndividualCostsByYear(true,$cadreForm);
            if (count($costs) == 0) {
                continue;
            }
            $all_costs[] = array('description'=>"<b>" . $scenarioObj->getField('name')->getValue() . "</b>" );
            $totals = array('description'=>'Total','total'=>0);
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $totals[$year] = 0;
            }
            foreach ($costs as $data) {
                foreach ($data as $k=>&$v) {
                    if (is_numeric($k)) { //its a year
                        $totals[$k] += $v;
                        $v = $this->currencyFormat( $v, $k);
                    } else if ($k =='total') {
                        $totals[$k] += $v;
                        $v = $this->currencyFormat( $v, $start_year);
                    }
                    
                }
                unset($v);
                $all_costs[] = $data;
            }
            foreach ($totals as $k=>&$v) {
                if (is_numeric($k)) { //its a year
                    $v = $this->currencyFormat( $v, $k);
                } else if ($k =='total') {
                    $v = $this->currencyFormat( $v, $start_year);
                }
            }
            unset($v);
            $all_costs[] = $totals;
            $all_costs[] = true;
        }
        if ($currentScenario instanceof iHRIS_Scenario) {
            $this->projection->setScenario($currentScenario);
        }
        $cadreID = substr($cadreForm,5);
        if ($cadreID) {
            $name = 'Retention Package for ' . I2CE_List::lookup($cadreID,'pool');
        } else {
            $name = 'Retention Package ';
        }
        $this->addDataTable($name,$headers,$all_costs,$containerNode);
        return true;
    }



    protected function costOutAllIncentivesByCadre($cadreForm) {
        if (!($containerNode = $this->template->appendFileById( "costout_table_ic.html", "div", "report_display" )) instanceof DOMNode) {
            return false;
        }
        $costs = $this->projection->getIndividualCostsByYear(false,$cadreForm);
        $headers['description'] = 'Description';
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;
        $totals = array('description'=>'<b>Total</b>','total'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $headers[$year] = $year;
            $totals[$year]  = 0;
        }
        $headers['total'] = 'Total';
        foreach ($costs as &$data) {
            foreach ($data as $k=>&$v) {
                if (is_numeric($k)) { //its a year
                    $totals[$k] += $v;
                    $v = $this->currencyFormat( $v, $k);
                } else if ($k =='total') {
                    $totals['total'] += $v;
                    $v = $this->currencyFormat( $v, $start_year);
                }

            }
            unset($v);
        }
        unset($data);

        foreach ($totals as $k=>&$v) {
            if (is_numeric($k)) { //its a year
                $v = $this->currencyFormat( $v, $k);
            } else if ($k =='total') {
                $v = $this->currencyFormat( $v, $start_year);
            }
        }
        $costs[] = $totals;
        $cadreID = substr($cadreForm,5);
        if ($cadreID) {
            $name = 'Cost of all Interventions for ' . I2CE_List::lookup($cadreID,'pool');
        } else {
            $name = 'Cost of all Interventions';
        }
        $this->addDataTable($name,$headers,$costs,$containerNode);
        return true;
    }



    protected function costOutSummary() {        
        $this->template->setDisplayDataImmediate('currency_USD',  $this->useUSD ? 1 : 0);

        $total_costs = $this->projection->getTotalCostsByYear();
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;
        $totals = array ('total'=>0,'A'=>0,'B'=>0,'C'=>0,'D'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $totals['total'] += $this->convertCurrency($total_costs['total'][$year],$year);
            $totals['A'] += $this->convertCurrency($total_costs['A'][$year],$year);
            $totals['B'] += $this->convertCurrency($total_costs['B'][$year],$year);
            $totals['C'] += $this->convertCurrency($total_costs['C'][$year],$year);
            $totals['D'] += $this->convertCurrency($total_costs['D'][$year],$year);
        }
        $total_bud = $this->projection->getTotalHealthBudgetsByYear();
        $total_sal = $this->projection->getTotalSalaryBudgetsByYear();
        $total_rec = $this->projection->getTotalRecurrentBudgetByYear();
        if ($this->useUSD) {
            $file = "costout_summary_USD.html";
        } else {
            $file = "costout_summary_local.html";
        }
        if (!($containerNode = $this->template->appendFileById( $file, "div", "report_display" )) instanceof DOMNode) {
            return false;
        }
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $node =$this->template->appendFileById( "costout_summary_each.html", "tr", "costout_each" );
            $this->template->setDisplayDataImmediate('costout_label',$year,$node);
            foreach (array('total','A','B','C','D') as $type) {
                $cost = $total_costs[$type][$year];
                $cost = $this->currencyFormat($cost,$year);
                $this->template->setDisplayDataImmediate('costout_' . $type,$cost,$node);
            }
            $this->template->setDisplayDataImmediate('budget',$this->currencyFormat($total_bud[$year],$year),$node);
            $this->template->setDisplayDataImmediate('salary',$this->currencyFormat($total_sal[$year],$year),$node);
        }

        $node =$this->template->appendFileById( "costout_summary_each.html", "tr", "costout_each" );
        $this->template->setDisplayDataImmediate('costout_label',"Total",$node);
        foreach (array('total','A','B','C','D') as $type) {
            $this->template->setDisplayDataImmediate('costout_' . $type ,$this->currencyFormat($totals[$type]),$node);
        }
        if ($this->isExport()) {
            $export_data = array();
            if ( ($rows = $this->template->query('./table//tr',$containerNode)) instanceof DOMNodeList) {
                foreach ($rows as $row) {
                    $row_data = array();                    
                    if ( ($cells = $this->template->query('./td | ./th',$row)) instanceof DOMNodeList) {
                        foreach ($cells as $cell) {
                            $row_data[] = $cell->textContent;
                        }
                    }
                    $export_data[] = $row_data ;
                }
            }
            $this->exportData($export_data);
        }

        return true;
    }

    protected function costOutTable($chart) {        
        if (!substr($chart,0,7) == 'summary') {
            I2CE::raiseError("Not a summary");
            return false;
        }
        $export = $this->isExport();
        $this->template->setDisplayDataImmediate('currency_USD',  $this->useUSD ? 1 : 0);
        $total_costs = $this->projection->getCostsByYear();
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;
        $totals = array ('total'=>0,'A'=>0,'B'=>0,'C'=>0,'D'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $totals['total'] += $this->convertCurrency($total_costs['total'][$year],$year);
            $totals['A'] += $this->convertCurrency($total_costs['A'][$year],$year);
            $totals['B'] += $this->convertCurrency($total_costs['B'][$year],$year);
            $totals['C'] += $this->convertCurrency($total_costs['C'][$year],$year);
            $totals['D'] += $this->convertCurrency($total_costs['D'][$year],$year);
        }
        $total_bud = $this->projection->getTotalHealthBudgetsByYear();
        $total_sal = $this->projection->getTotalSalaryBudgetsByYear();
        $total_rec = $this->projection->getTotalRecurrentBudgetByYear();
        if ( !($containerNode = $this->template->appendFileById( "costout_". $chart . ".html", "div", "report_display" )) instanceof DOMNode) {
            I2CE::raiseMessage("Could not add costout_{$chart}.html");
            return false;
        }

        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $node =$this->template->appendFileById( "costout_" . $chart . "_each.html", "tr", "costout_each" );
            $this->template->setDisplayDataImmediate('costout_label',$year,$node);
            foreach (array('total','A','B','C','D') as $type) {
                $cost = $total_costs[$type][$year];
                if ($chart == 'summary_percent_budget') {
                    if ($export) {
                        if ($total_bud[$year] > 0) {
                            $cost = sprintf ("%3.1f", (((float)$cost) / ($total_bud[$year])) * 100.0);
                        } else {
                            $cost = '0';
                        }
                    } else {
                        if ($total_bud[$year] > 0) {
                            $cost = sprintf ("%3.1f %%", (((float)$cost) / ($total_bud[$year])) * 100.0);
                        } else {
                            $cost = '0%';
                        }
                    }
                } else if ($chart == 'summary_percent_salary') {
                    if ($export) {
                        if ($total_sal[$year] > 0) {
                            $cost = sprintf ("%3.1f", (((float)$cost) / ($total_sal[$year])) * 100.0);
                        } else {
                            $cost = '0';
                        }
                    } else {
                        if ($total_sal[$year] > 0) {
                            $cost = sprintf ("%3.1f %%", (((float)$cost) / ($total_sal[$year])) * 100.0);
                        } else {
                            $cost = '0%';
                        }
                    }
                } else if ($chart == 'summary_percent_recurrent') {
                    if ($export) {
                        if ($total_sal[$year] > 0) {
                            $cost = sprintf ("%3.1f", (((float)$cost) / ($total_rec[$year])) * 100.0);
                        } else {
                            $cost = '0';
                            //$cost == 'Error';
                        }

                    } else {
                        if ($total_sal[$year] > 0) {
                            $cost = sprintf ("%3.1f %%", (((float)$cost) / ($total_rec[$year])) * 100.0);
                        } else {
                            $cost = '0%';
                        }
                    }
                } else {
                    $cost = $this->currencyFormat($cost,$year);
                }
                $this->template->setDisplayDataImmediate('costout_' . $type,$cost,$node);
            }
            $this->template->setDisplayDataImmediate('budget',$this->currencyFormat($total_bud[$year],$year),$node);
            $this->template->setDisplayDataImmediate('salary',$this->currencyFormat($total_sal[$year],$year),$node);
        }
        if ($export) {
            $export_data = array();
            if ( ($rows = $this->template->query('./table//tr',$containerNode)) instanceof DOMNodeList) {
                foreach ($rows as $row) {
                    $row_data = array();                    
                    if ( ($cells = $this->template->query('./td | ./th',$row)) instanceof DOMNodeList) {
                        foreach ($cells as $cell) {
                            $row_data[] = trim(str_replace("\n"," ",$cell->textContent));
                        }
                    }
                    $export_data[] = $row_data ;
                }
            }
            $this->exportData($export_data);
        }

        return true;
  }





    protected function makeChart($chart) {
        if (!($containerNode = $this->template->appendFileById( "costout_chart.html", "div", "report_display" )) instanceof DOMNode) {
            return false;
        }
        $this->template->setDisplayDataImmediate(
            'chart_image', 
            'chart/' .  iHRIS_Projection::$chart_types[$chart]      . '/' 
            . $chart . "?id=" . $this->projection->getNameId() );

        $this->template->setDisplayDataImmediate('chart_title',iHRIS_Projection::getChartTitle($chart),$containerNode);
              
        return true;
    }


    
    //exportData() does not execpt $row to be a string
    // protected function chart_data($chart) {
    //     $method = 'get_chart_data_' . $chart;
    //     $export_data = $this->projection->$method();
    //     $chart_title = iHRIS_Projection::getChartTitle($chart);
    //     foreach ($export_data as &$row) {
    //         foreach ($row as &$c)  {
    //             $c= '"' . str_replace('"','""',trim(str_replace("\n"," ",$c)))  . '"';
    //         }
    //         unset($c);
    //         $row = implode(",",$row);
    //     }
    //     unset($row);
    //     $this->exportData($export_data);
    // }




}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
