<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Tab_ScenarioGroups extends iHRIS_Tab_Base {
    protected $projection;
        
    protected $action = 'default';
    public  function __construct( $args,$request_remainder , $get = null, $post = null) {
        parent::__construct($args,$request_remainder,$get,$post);
        if (is_array($this->request_remainder) && count($this->request_remainder) > 0) {
            reset($this->request_remainder);
            $this->action = current($this->request_remainder);
        }
        
    }




    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $this->template->addHeaderLink("mootools-core.js");
        $this->template->addHeaderLink("mootools-more.js");
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink("I2CE_AjaxSubmitButton.js");
        $factory = I2CE_FormFactory::instance();
        $id = false;
        if ($this->request_exists('id') && $this->request('id')) { 
            $id = $this->get('id');
        } else  if ($this->request_exists('parent') && $this->request('parent')) {
            if (substr($this->request('parent'),0,11) == 'projection|') {
                $id = $this->get('parent');
            }
        }
        if (!$id || ! ($this->projection = $factory->createForm($id )) instanceof iHRIS_Projection) {
            $this->setRedirect(  "home" );
            return false;
        }
        $this->projection->populate();
        $this->projection->fixupFinancials();
        $this->projection->fixupPools();
        $this->projection->fixupInterventions();

        $this->template->setForm( $this->projection );
        $this->template->setDisplayData('id', $this->projection->getNameID());        
        
        if ( ($scenarioObj = $this->projection->setScenario(null,false))  instanceof iHRIS_Scenario) {
            $this->template->setForm($scenarioObj);      
        }

        switch($this->action) {
        case 'compare':
            parent::action();
            $this->costOutScenarioGroups();
            break;
        case 'cost':
            parent::action();
            $this->costOutOneScenarioGroup();
            break;
        case 'costcadre':
            parent::action();
            $this->costOutOneScenarioGroupByCadre();
            break;

        default:          
            if (!$this->action_menu()) {
                return false;
            }
            break;        
        }
        return true;
    }


    protected function action_menu() {
        $scenarios = $this->projection->getScenarios();
        $scenariogroups = $this->projection->getScenarioGroups();
        // $sgs = I2CE_FormFactory::instance()->createContainer('scenariogroupselection');
        // $sgs->setParent($this->projection);
        // $this->template->setForm($sgs);
        $this->template->setDisplayDataImmediate('has_more_than_one_package',  count($scenarios) > 1 ?  1 : 0);
        $this->template->setDisplayDataImmediate('has_at_least_one_package_group',  count($scenariogroups) >= 1 ? 1: 0);
        $this->template->setDisplayDataImmediate('scenariogroup',  $scenariogroups);
        return true;

    }
    
    /**
     * Load the  template (HTML or XML) files to the template object.
     *  
     * 
     */  
    protected function loadHTMLTemplates() {
        if ($this->defaultHTMLFile === false) {
            return true;
        }
        switch($this->action) {
        case 'compare':
        case 'cost':
            $this->template->addFile( 'scenariogroups_compare.html');
            break;
        default:
            if (is_array($this->defaultHTMLFile)) {
                $files = $this->defaultHTMLFile;
            } else {
                $files = array($this->defaultHTMLFile);
            }
            foreach ($files as $file) {
                if ( ! ($file === null || (is_string($file) && strlen($file) == 0))) {
                    $this->template->addFile( $file);
                }
            }
            break;
        }
        return true;
    }


    protected function costOutScenarioGroups() {
        $factory = I2CE_FormFactory::instance();
        if (! ($scenarioGroupSelect = $factory->createContainer('scenariogroupselection')) instanceof iHRIS_ScenarioGroupSelection) {
            I2CE::raiseError("No selection");
            return false;
        }
        $this->template->setForm($scenarioGroupSelect);                
        $currentScenario  = $this->projection->getCurrentScenario();
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;
        $scenarioGroupSelect->setParent($this->projection,false);
        $scenarioGroupSelect->load($this->request());
        $sgsField = $scenarioGroupSelect->getField('scenariogroup');
        $headers['description'] = 'Package Group';
        $headers['avg_cost'] = 'Average Annual Cost';
        $headers['avg_pref'] = 'Average Preference Rate';
        $headers['hsb'] = 'Average Health Sector Budget';
        $headers['hsb_percent'] = 'Percentage Of Health Sector Budget';
        $headers['wb'] = 'Average Wage Budget';
        $headers['wb_percent'] = 'Percentage Of Wage Budget';
        $all_costs  =array();
        $hsb = array_sum($this->projection->getTotalHealthBudgetsByYear()) / (float) $duration;
        $wb = array_sum($this->projection->getTotalSalaryBudgetsByYear()) / (float) $duration;
        foreach ($sgsField->getValue() as $sg) {            
            if ( ! ($sgObj = $factory->createContainer(implode('|',$sg))) instanceof iHRIS_ScenarioGroup) {
                I2CE::raiseError("Bad " . print_r($sg,true));
                continue;
            }
            $this->projection->disableInterventions();
            $sgObj->populate();
            //each scenario group = is a set of packages 
            $sField = $sgObj->getField('scenario');

            $which = array();


            foreach ($sField->getValue() as $scenario) {
                if ( ! ($sObj = $factory->createContainer(implode('|',$scenario))) instanceof iHRIS_Scenario) {
                    continue;
                }
                $this->projection->setScenario(implode('|' , $scenario));
                $sObj->populate();
                $t_which = $sObj->getField("interventions")->getValue();
                foreach ($t_which as &$d)  {
                    $d= implode("|", $d);
                }
                unset($d);
                $which = array_unique(array_merge($which,$t_which));
            }
            $this->projection->enableInterventions($which);

            $tot_hws =0;
            $pref = 0;
            foreach ($sField->getValue() as $scenario) {
                if ( ! ($sObj = $factory->createContainer(implode('|',$scenario))) instanceof iHRIS_Scenario) {
                    continue;
                }
                $this->projection->setScenario(implode('|' , $scenario));
                $sObj->populate();

                $hws = array();
                for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                    //note that this _does_ depend on the set scenario.
                    $hws[$year] = (float) $this->projection->getTargetedHealthWorkers($year);                
                }
                $hws = (int) floor(array_sum($hws) /  ((float)$duration)); //average number of hws for this scenario
                $tot_hws+= $hws;
                $pref += $hws * (float) $sObj->getField('preference')->getValue();
            }
            if ($tot_hws > 0) {
                $pref = $pref / $tot_hws ;  //weighted average of prefences among all scenarios in our groupr
            } else {
                $pref = 0;
            }

            $cost = $this->projection->getTotalCostsByYear();
            $cost = $this->convertCurrency(array_sum($cost['total'])/(float)$duration, $year);

            if ($wb > 0) {
                $wb_percent = (int) round((100*$cost/$wb)); 
                $wb_percent .= '%';
            } else {
                $wb_percent = '';
            }
            if ($hsb > 0) {
                $hsb_percent = (int) round((100*$cost/$hsb)); 
                $hsb_percent .= '%';
            } else {
                $hsb_percent = '';
            }

            $all_costs[] = array(
                'description'=> $sgObj->getField('name')->getValue(),
                'avg_cost'=>$this->currencyFormat($cost),
                'avg_pref'=>((int)round( $pref))  . '%',
                'hsb'=>$this->currencyFormat($hsb),
                'hsb_percent'=> $hsb_percent,
                'wb'=>$this->currencyFormat($wb),
                'wb_percent'=> $wb_percent
                );
            
        }
        $name = 'Cost of Package Options and Current Expenditures ' . $sgsField->getDisplayValue();
        $this->addDataTable($name,$headers,$all_costs);
        if ($currentScenario instanceof iHRIS_Scenario) {
            $this->projection->setScenario($currentScenario);
        }
        return true;

    }


    protected function addDataTable($name,$headers,$data,$node = null) {
        if ( ($hNode = $this->template->getElementById('report_name',$node)) instanceof DOMNode) {
            $hNode->setAttribute('colspan',count($headers));
        }
        $this->template->setDisplayDataImmediate('report_name',$name);
        if ( ($lNode = $this->template->getElementById('labels',$node)) instanceof DOMNode) {
            foreach ($headers as $key=>$label) {
                $html = "<td class='costout_label'><b>$label</b></td>";
                if (($ltNode = $this->template->importText($html,'td')) instanceof DOMNode) {
                    $lNode->appendChild($ltNode);
                }
            }
        }
        if ( ($dNode = $this->template->getElementById('data',$node)) instanceof DOMNode) {
            foreach ($data as $values) {
                if ($values === true) {
                    $html = '<tr><td style="min-height:1em" colspan="' . count($headers) . '"/></tr>';
                } else {
                    $html = '<tr>';
                    foreach ($headers as $key=>$label) {
                        $val = '';
                        if (array_key_exists($key,$values)) {
                            $val = $values[$key];
                        }
                        $html .= "<td>$val</td>";
                    }
                    $html.='</tr>';                
                }
                if (($dtNode = $this->template->importText($html,'tr')) instanceof DOMNode) {
                    $dNode->appendChild($dtNode);
                }
            }
        }
        if ($this->isExport()) {
            I2CE::raiseError("Doing export");
            $export_data = array();
            if ( ($rows = $this->template->query('//table[@id="compare_table"]//tr',$node)) instanceof DOMNodeList) {
                foreach ($rows as $row) {
                    I2CE::raiseMessage("Row");
                    $row_data = array();                    
                    if ( ($cells = $this->template->query('./td | ./th',$row)) instanceof DOMNodeList) {
                        foreach ($cells as $cell) {
                            $row_data[] = $cell->textContent;
                        }
                    }
                    $export_data[] = $row_data ;
                }
            }
            $this->exportData($export_data);
        }


        return true;
    }
    


    protected function costOutOneScenarioGroupByCadre() {
        if (! $this->request_exists('scenariogroup') || !$this->request('scenariogroup')) {
            I2CE::raiseError('Bad sg');
            return false;
        }
        $factory = I2CE_FormFactory::instance();
        if ( ! ($sGroup = $factory->createContainer('scenariogroup|' . $this->request('scenariogroup'))) instanceof iHRIS_ScenarioGroup) {
            I2CE::raiseError('Bad sg obj');
            return false;
        }
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;

        $sGroup->populate();
        $sField = $sGroup->getField('scenario');


        $headers = array(
            'description'=>'',
            'pref'=>'Preference'           
            );
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $headers[$year] = $year;
        }
        $headers['total'] = 'Total';
        $grand_totals = array('description'=>'<b>Package Group Total</b>','pref'=>'', 'total'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $grand_totals[$year] = 0;
        }


        foreach ($this->projection->children['pool'] as $i=>$pool) {

        
            $which = array();
            foreach ($sField->getValue() as $scenario) {
                if ( ! ($sObj = $factory->createContainer(implode('|',$scenario))) instanceof iHRIS_Scenario) {
                    continue;
                }
                $this->projection->setScenario(implode('|' , $scenario));
                $sObj->populate();
                $t_which = $sObj->getField("interventions")->getValue();
                foreach ($t_which as &$d)  {
                    $d= implode("|", $d);
                }
                unset($d);
                $which = array_unique(array_merge($which,$t_which));
            }
            $this->projection->enableInterventions($which);
            $pool_costs = $this->projection->getIndividualCostsByYear(true,"pool|" . $i);
            if (count($pool_costs) == 0) {
                continue;
            }

            $pref = 0;
            $tot_hws = 0;
            foreach ($sField->getValue() as $scenario) {
                if ( ! ($sObj = $factory->createContainer(implode('|',$scenario))) instanceof iHRIS_Scenario) {
                    continue;
                }
                $this->projection->setScenario(implode('|' , $scenario));
                $sObj->populate();
                $hws = array();
                for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                    $hws[$year] = (float) $this->projection->getTargetedHealthWorkers($year,"pool|" . $i);                
                }
                $hws = (int) floor(array_sum($hws) /  ((float)$duration)); //average number of hws for this scenario
                $tot_hws+= $hws;
                $pref += $hws * ((float) $sObj->getField('preference')->getValue());
            }
            if ($tot_hws > 0) {
                $pref = $pref / $tot_hws ;  //weighted average of prefences among all scenarios in our groupr
            } else {
                $pref = 0;
            }
            



            $c_data = array(
                'description'=> "<b>" . I2CE_List::lookup($i,'pool') . "</b>",
                'pref'=>((int)round($pref)) . '%',
                'total'=>''
                );
            $totals = array('description'=>'Total','pref'=>'', 'total'=>0);
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $c_data[$year] = '';
                $totals[$year] = 0;
            }
            $all_costs[] =$c_data;


            foreach ( $pool_costs as $costs) {
                foreach ($costs as $k=>&$v) {
                    if ($k == 'total') {
                        $totals['total'] += $v;
                        $grand_totals['total'] += $v;
                        $v = $this->convertedCurrencyFormat($v,$start_year);
                    } else if (is_numeric($k)) {
                        $totals[$k] += $v;
                        $grand_totals[$k] += $v;
                        $v = $this->convertedCurrencyFormat($v,$k);
                    }
                }
                unset($v);
                $all_costs[] = $costs;
            }
            foreach ($totals as $k=>&$v) {
                if ($k == 'total') {
                    $v = $this->convertedCurrencyFormat($v,$start_year);
                } else if (is_numeric($k)) {
                    $v = $this->convertedCurrencyFormat($v,$k);
                }
            }
            unset($v);
            $all_costs[] = $totals;
            $all_costs[] = true;
        }

        foreach ($grand_totals as $k=>&$v) {
            if ($k == 'total') {
                $v = $this->convertedCurrencyFormat($v,$start_year);
            } else if (is_numeric($k)) {
                $v = $this->convertedCurrencyFormat($v,$k);
            }
        }
        unset($v);

        $all_costs[] = $grand_totals;
        
        $name = 'Cost Of Incentives For Package Group By Cadre: ' . I2CE_List::lookup($this->request('scenariogroup'),'scenariogroup');
        $this->addDataTable($name,$headers,$all_costs);
        return true;
    }


    protected function costOutOneScenarioGroup() {
        if (! $this->request_exists('scenariogroup') || !$this->request('scenariogroup')) {
            I2CE::raiseError('Bad sg');
            return false;
        }
        $factory = I2CE_FormFactory::instance();
        if ( ! ($sGroup = $factory->createContainer('scenariogroup|' . $this->request('scenariogroup'))) instanceof iHRIS_ScenarioGroup) {
            I2CE::raiseError('Bad sg obj');
            return false;
        }
        $start_year = $this->projection->start_year->year();        
        $duration = $this->projection->duration;

        $sGroup->populate();
        $sField = $sGroup->getField('scenario');


        $headers = array(
            'description'=>'',
            'pref'=>'Preference'           
            );
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $headers[$year] = $year;
        }
        $headers['total'] = 'Total';
        $grand_totals = array('description'=>'<b>Package Group Total</b>','pref'=>'', 'total'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $grand_totals[$year] = 0;
        }

        
        
        $which = array();
        $s_interventions = array();
        $s_names = array();
        $s_prefs = array();
        foreach ($sField->getValue() as $scenario)   {
            $scenarioID  = implode("|",$scenario);
            if ( ! ($sObj = $factory->createContainer($scenarioID)) instanceof iHRIS_Scenario) {
                continue;
            }
            $this->projection->setScenario($scenarioID);
            $sObj->populate();
            $s_names[$scenarioID] = $sObj->getField('name')->getValue();
            $s_prefs[$scenarioID] = $sObj->getField('preference')->getDisplayValue();
            $t_which = $sObj->getField("interventions")->getValue();
            foreach ($t_which as &$d)  {
                $d= implode("|", $d);
            }
            unset($d);
            $s_interventions[$scenarioID] = $t_which;
            $which = array_unique(array_merge($which,$t_which));
        }
        $this->projection->enableInterventions($which);

        $sg_costs = $this->projection->getIndividualCostsByYear(); //keys are "$intervnention_form|$id"
        
        $all_costs[] = array();
        $totals = array('total'=>0);
        $all_totals = array('description'=>'<b>Package Group Total<b>','pref'=>'', 'total'=>0);
        for ( $year = $start_year; $year < $start_year + $duration; $year++) {
            $all_totals[$year] = 0;
        }
        foreach ($sg_costs as $costs) {
            foreach ($costs as $k=>$v) {
                if ($k == 'total') {
                    $all_totals['total'] += $v;
                } else if (is_numeric($k)) {
                    $all_totals[$k] += $v;
                }        
            }
                    
        }

        foreach ($s_interventions as $scenarioID=>$interventions) {
            //$interventions is an array of $intercnetion_form|$id
            $totals = array('description'=>'<b>Package Total</b>','pref'=>'', 'total'=>0);
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $totals[$year] = 0;
            }
            $scenario_costs = array();
            foreach ($interventions as $interventionID) {
                if (!array_key_exists($interventionID,$sg_costs)) {
                    continue;
                }
                $costs = $sg_costs[$interventionID];
                foreach ($costs as $k=>&$v) {
                    if ($k == 'total') {
                        $totals['total'] += $v;
                        $v = $this->convertedCurrencyFormat($v,$start_year);
                    } else if (is_numeric($k)) {
                        $totals[$k] += $v;
                        $v = $this->convertedCurrencyFormat($v,$k);
                    }                    
                }
                unset($v);
                $scenario_costs[$interventionID] = $costs;
            }
            if (count($scenario_costs) == 0) {
                //no interventions in this scenario.
                continue;
            }

            $all_costs[] = array(
                'description'=> $s_names[$scenarioID],
                'pref'=>$s_prefs[$scenarioID]);
            foreach ($scenario_costs as $costs) {
                $all_costs[]= $costs;
            }
            foreach ($totals as $k=>&$v) {
                if ($k == 'total') {
                    $v = $this->convertedCurrencyFormat($v,$start_year);
                } else if (is_numeric($k)) {
                    $v = $this->convertedCurrencyFormat($v,$k);
                }
            }
            unset($v);

            $all_costs[] = $totals;
            $all_costs[] = true;
        }
        foreach ($all_totals as $k=>&$v) {
            if ($k == 'total') {
                $v = $this->convertedCurrencyFormat($v,$start_year);
            } else if (is_numeric($k)) {
                $v = $this->convertedCurrencyFormat($v,$k);
            }
        }
        unset($v);
        $all_costs[] = $all_totals;
        $name = 'Cost Of Incentives For Package Group: ' . I2CE_List::lookup($this->request('scenariogroup'),'scenariogroup');
        $this->addDataTable($name,$headers,$all_costs);
        return true;
    }


    

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
