<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Tab_Scenarios extends iHRIS_Tab_Base{
    protected $projection;
        
    protected $action = 'default';
    public  function __construct( $args,$request_remainder , $get = null, $post = null) {
        parent::__construct($args,$request_remainder,$get,$post);
        if (is_array($this->request_remainder) && count($this->request_remainder) > 0) {
            reset($this->request_remainder);
            $this->action = current($this->request_remainder);
        }
        
    }

    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $this->template->addHeaderLink("mootools-core.js");
        $this->template->addHeaderLink("I2CE_ClassValues.js");
        $this->template->addHeaderLink("I2CE_SubmitButton.js");
        $factory = I2CE_FormFactory::instance();
        $id = false;
        if ($this->request_exists('id') && $this->request('id')) { 
            $id = $this->get('id');
        } else  if ($this->request_exists('parent') && $this->request('parent')) {
            if (substr($this->request('parent'),0,11) == 'projection|') {
                $id = $this->get('parent');
            }
        }
        if (!$id || ! ($this->projection = $factory->createForm($id )) instanceof iHRIS_Projection) {
            I2CE::raiseError("Bad ID");
            $this->setRedirect(  "home" );
            return false;
        }
        $this->projection->populate();
        $this->projection->fixupFinancials();
        $this->projection->fixupPools();
        $this->projection->fixupInterventions();

        $this->template->setForm( $this->projection );
        $this->template->setDisplayData('id', $this->projection->getNameID());        
        
        if ( ($scenarioObj = $this->projection->setScenario(null,false))  instanceof iHRIS_Scenario) {
            $this->template->setForm($scenarioObj);      
        }
      
        switch($this->action) {
        case 'comparegroup':
            parent::action();
            $this->costOutScenarioGroups();
            break;
        case 'compare':
            parent::action();
            $this->costOutScenarios();
            break;
        default:            
            if ( $this->request_exists('submit_type') && $this->request('submit_type') == 'select_scenario') {            
                $this->setRedirect(  "view?id="  . $this->request('id') . '&tab=scenario');
                $scenario = 'scenario|' . $this->request('scenario');
            }   else if ( $this->request_exists('submit_type') && $this->request('submit_type') == 'delete_scenario') {
                $this->setRedirect(  "view?id="  . $this->request('id') . '&tab=scenario');
                if (! ($scenarioObj =   $factory->createContainer('scenario|' . $this->request('scenario'))) instanceof iHRIS_Scenario) {
                    I2CE::raiseError("Bad scenario object");
                    return false;
                }
                I2CE::raiseError("delete scenario object scenario|"  . $this->request('scenario'));
                $scenarioObj->delete();                
                return true;                
            }  else if (!$this->action_menu()) {
                return false;
            }
            break;        
        }
        return true;
    }


    /**
     * Load the  template (HTML or XML) files to the template object.
     *  
     * 
     */  
    protected function loadHTMLTemplates() {
        //die($this->action);
        switch($this->action) {
        case 'comparegroup':
        case 'costcadre':
        case 'cost':
            $this->template->addFile( 'scenarios_' . $this->action . '.html');
            break;
        case 'compare':
            $this->template->addFile( 'scenarios_compare.html');
            break;
        default:
            if (is_array($this->defaultHTMLFile)) {
                $files = $this->defaultHTMLFile;
            } else {
                $files = array($this->defaultHTMLFile);
            }
            foreach ($files as $file) {
                if ( ! ($file === null || (is_string($file) && strlen($file) == 0))) {
                    $this->template->addFile( $file);
                }
            }
            break;
        }
        return true;
    }



    protected function action_menu() {
        $this->template->setForm( $this->projection );
        $scenarioObj = $this->projection->setScenario();
        parent::action();
        if ($scenarioObj instanceof iHRIS_Scenario) {
            $this->template->setForm($scenarioObj);      
        } else {
            I2CE::raiseError("Bad scenario");
            return false;
        }

        $this->template->setDisplayData('id', $this->projection->getNameID());        
        $scenarios = $this->projection->getScenarios();
        $this->template->setDisplayDataImmediate('scenario',  $scenarios);

        $scenarios = $this->projection->getScenarios();
        $pools = $this->projection->getPools();
        $this->template->setDisplayDataImmediate('pool',  $pools);

        $this->template->setDisplayDataImmediate('has_more_than_one_package',  count($scenarios) > 1 ?  1 : 0);
        return true;

    }


    public static function prefSorter($a,$b) {
        //The comparison function must return an integer less than, equal to, or greater than zero if the first argument is considered to be respectively less than, equal to, or greater than the second. 
        //we want no pref set to be at the bottom so it should be the biggest, e.g. 1
        if (!is_array($a) || !array_key_exists('scenario_pref',$a) || !is_numeric($a['scenario_pref'])) {
            return 1;
        }
        if (!is_array($b) || !array_key_exists('scenario_pref',$b) || !is_numeric($b['scenario_pref'])) {
            return 1;
        }
        //otherwise we want the highest pref at the top, so it should considered smaller, e.g. -1
        if ( $a['scenario_pref'] < $b['scenario_pref']) {
            return 1;
        } else if ($a['scenario_pref'] > $b['scenario_pref']) {
            return -1;
        } else {
            return 0;
        }
            
    }


    protected function costOutScenarios() {
        if ( ! ($compare_node = $this->template->getElementById('compare_scenarios')) instanceof DOMNode) {
            I2CE::raiseError("Dont know where to add scenario comparison");
            return;
        }
        $currentScenario  = $this->projection->getCurrentScenario();
        $start_year =$this->projection->start_year->year();        
        $duration = $this->projection->duration;
        if ($duration == 0) {
            I2CE::raiseError("Invalid duration");
            return;
        }
        $scenarios = $this->projection->getScenarios();
        foreach ($scenarios as $scenario=>$name) {
            if (! ($scenarioObj = $this->projection->getScenario('scenario|' . $scenario)) instanceof iHRIS_Scenario) {
                continue;
            }
            $this->projection->setScenario('scenario|' . $scenario);
            $costs = $this->projection->getTotalCostsByYear();
            
            $total = 0;
            if (array_key_exists('total',$costs)) {
                $total = max(0,array_sum($costs['total']));
            }
            
            if ($scenarioObj->getField('preference')->isValid()) {
                $pref = $scenarioObj->getField('preference')->getValue();
                if ($pref <= 0 || $pref > 100) {
                    $scenarioObj->getField('preference')->setValue(100); //dont save
                    $pref = '100 %';
                } else {
                    $pref = $scenarioObj->getField('preference')->getDisplayValue();
                }
            } else {
                $pref = '';
            }
            $hws = array();
            for ( $year = $start_year; $year < $start_year + $duration; $year++) {
                $hws[$year] = max(0,(float) $this->projection->getTargetedHealthWorkers($year) * (float) $scenarioObj->getField('preference')->getValue() / 100.0);
            }
            $hws = (int) floor(array_sum($hws) /  ((float)$duration));
            if ($hws > 0) {
                $cost = $this->currencyFormat( ((float)$total) / ($hws*$duration));
            } else {
                $cost ='';
            }
            $datas[] = array(
                'scenario_name'=>$name . ' : ' . $scenarioObj->getField('description')->getValue(),
                'scenario_pref'=>$pref,
                'scenario_total_cost'=>$this->currencyFormat($total),
                'scenario_hws'=>$hws,
                'scenario_yearly_hw_cost'=>$cost
                );
        }
        usort($datas,array('self','prefSorter'));
        $count = 0;
        foreach ($datas as $data) {
            if (!  ($compNode = $this->template->appendFileByNode("compare_packages.html","tr",$compare_node)) instanceof DOMNode) {
                continue;
            }
            $count++;
            if ($count %2 == 0) {
                $compNode->setAttribute('class','even');
            } else {
                $compNode->setAttribute('class','odd');
            }
            foreach ($data as $k=>$v) {
                $this->template->setDisplayDataImmediate($k,$v ,$compNode);
            }
        }

        if ($currentScenario instanceof iHRIS_Scenario) {
            $this->projection->setScenario($currentScenario);
        }
        if ($this->isExport()) {
            $export_data = array();
            if ( ($rows = $this->template->query('//table[@id="compare_scenarios"]//tr',$compare_node)) instanceof DOMNodeList) {
                foreach ($rows as $row) {
                    $row_data = array();                    
                    if ( ($cells = $this->template->query('./td | ./th',$row)) instanceof DOMNodeList) {
                        foreach ($cells as $cell) {
                            $row_data[] = $cell->textContent;
                        }
                    }
                    $export_data[] = $row_data ;
                }
            }
            $this->exportData($export_data);
        }


    }


  }


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
    