<?php
/**
* © Copyright 2011 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package ihris-retain
* @subpackage retain
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v1.1
* @since v1.1
* @filesource 
*/ 
/** 
* Class iHRIS_Finanical
* 
* @access public
*/


class iHRIS_Yearly extends I2CE_Form {
    
    protected $yearly_inputs;

    /**
     * Get the field names that are yearly inputs
     * @returns array of string
     */ 
    public function yearlyInputs() {
        if (is_array($this->yearly_inputs)) {
            return $this->yearly_inputs;
        }
        $this->yearly_inputs = array();
        foreach ($this->fields as $field=>$fieldObj) {
            if (!$fieldObj instanceof I2CE_FormField_ASSOC_LIST) {
                continue;
            }
            $this->yearly_inputs[] = $field;
        }
        return $this->yearly_inputs;
    }

    /**
     * Ensure all the yearly inputs have the correct years as keys
     * @param int $start_year
     * @param int $end_year
     */
    public function ensureYearlyInputs($start_year,$end_year) {
        if ($end_year < $start_year) {
            return;
        }
        foreach ($this->yearlyInputs() as  $input) {            
            if (! ($assocList =$this->getField($input)) instanceof I2CE_FormField_ASSOC_LIST) {
                I2CE::raiseError("bad field $input");
                continue;
            }
            $existing_keys = $assocList->getKeys();
            $needed_keys = array();
            for ($year = $start_year; $year <= $end_year; $year++) {
                $needed_keys[] = $year;
                $assocList->ensureKey($year);
                if (!$assocList->keyIsSet($year)) {
                    //$assocList->setValueOfKey($year,0);
                }
            }
            foreach (array_diff($existing_keys,$needed_keys) as $remKey) {
                $assocList->removeKey($remKey);
            }
        }
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
