/*
--- adopted from TabPane Class 
license: MIT-style
authors: akaIDIOT
version: 0.1
*/

var I2CE_AjaxTabPanel = new Class({
    
    Implements: [Events, Options],

    options: {
        tabSelector: '.tab',
        contentSelectorClass: 'content',
	loadingClass: 'stub-ajax-loading',
        activeClass: 'active'
    },

    container: null,
    showNow: false,

    initialize: function(container, options) {
        this.setOptions(options);

        this.container = document.id(container);
	if (!this.container) {
	    return;
	}
        this.container.getElements('.' + this.options.contentSelectorClass).setStyle('display', 'none');

        this.container.addEvent('click:relay(' + this.options.tabSelector + ')', function(event, tab) {
            this.showTab(tab.get('id'));
        }.bind(this));

        this.container.getElements(this.options.tabSelector).addClass(this.options.activeClass);
        this.container.getElements('.' + this.options.contentSelectorClass).setStyle('display', 'block');
    },

    showTab: function(id) {
	if (!this.container) {
	    return;	    
	}
        var content = this.container.getElement('#tab_' + id);
        var tab = this.container.getElement('#' + id);
        if (!tab ||  !content) {
	    return;
	}
	if (tab.hasClass('disabled_tab')) {
	    return;
	}
	if (content.get('tag') == 'a') {
	    //try to ajax load it
	    this.requestTab(tab,content);
	}  else {
	    this.makeVisible(tab,content);
	}
    },

    requestTab: function(tab,content) {
	var url = content.get('href');
	var id = tab.get('id');
	var new_content = new Element(
	    'span',
	    {'id':'tab_' + id,
	     'class': this.options.contentSelectorClass + ' ' + this.options.loadingClass
	    });
	content.getChildren().each(function(elem) {
	    new_content.adopt(elem);
	});
        new_content.replaces( content);
	var loadingClass = this.options.loadingClass;
	new Request.HTML(
	    {
		url : url,
		onSuccess: function() {
		    var ajax_content = this.response.elements.filter('#tab_' + id);		    
		    if (!ajax_content) {
			content.appendText("Error Loading Tab");			
			return;
		    }
		    new_content.empty();
		    $exec(this.response.javascript);		    
		    new_content.adopt(ajax_content);
		    var top = new_content.getCoordinates().top;
		    var getLowest =  function(elem) {
			var bottom = elem.getCoordinates().bottom;
			elem.getChildren().each(function(child) {
			    bottom = Math.max(bottom,getLowest(child));
			});
			return bottom;
		    };
		    var bottom = getLowest(new_content);
		    var height = Math.max(30,bottom - top);
		    new_content.setStyle( 'height',height);
		    new_content.removeClass(loadingClass);		    
		}
		
	    }
		
	).get();
	this.makeVisible(tab,new_content);
    },
    


    makeVisible: function(tab,content)  {
	if (!this.container) {
	    return;
	}

        this.container.getElements(this.options.tabSelector).removeClass(this.options.activeClass);
        this.container.getElements('.' + this.options.contentSelectorClass).setStyle('display', 'none');
        tab.addClass(this.options.activeClass);
	this.fixHeight(content);
        content.setStyle('display', 'block');
        this.fireEvent('change', tab.get('id'));
    },

    fixHeight: function(elem) {
	if (!$defined(elem)) {
	    return;
	}
	var top = elem.getCoordinates().top;
	var getLowest =  function(elem) {
	    var bottom = elem.getCoordinates().bottom;
	    elem.getChildren().each(function(child) {
		bottom = Math.max(bottom,getLowest(child));
	    });
	    return bottom;
	};
	var bottom = getLowest(elem);
	var height = Math.max(30,bottom - top);
	elem.setStyle( 'height',height);
	
}





});


